package lbl.akenti.certs;

import java.io.IOException;
import java.text.ParseException;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.util.ParsingException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


public class PolicyCertificate extends AkentiCertificate {
  private String resource;
  private CertificateAuthorityInfo[] certAuthInfos;
  private Directory[] attributeDirs;
  private UseCondIssuerGroup[] ucIssuerGroups;
  private long cacheTime;
  
  public PolicyCertificate() {
    super("Policy");
    resource = "NO_RESOURCE_NAME";
    certAuthInfos = new CertificateAuthorityInfo[0];
    attributeDirs = new Directory[0];
    ucIssuerGroups = new UseCondIssuerGroup[0];
    cacheTime = -1;
  }
  
  public PolicyCertificate(AkentiInputStream in) throws IOException, ParseException {
    super(in);
  }
  
  public String getHashString() {
    return getResource();
  }
  
  protected void setResource(String resource) {
    if (resource.length() == 0) {
      throw new IllegalArgumentException();
    }
    
    this.resource = resource;
  }
  
  public String getResource() {
    return resource;
  }
  
  public boolean
  containsCertificateAuthorityInfos() {
    return certAuthInfos.length > 0;
  }
  
  protected void
  setCertificateAuthorityInfos(CertificateAuthorityInfo[] certAuthInfos) {
    if (certAuthInfos.length == 0) {
      throw new IllegalArgumentException();
    }
    
    this.certAuthInfos = certAuthInfos;
  }
  
  public CertificateAuthorityInfo[]
  getCertificateAuthorityInfos() {
    return certAuthInfos;
  }
  
  protected void
  setUseCondIssuerGroups(UseCondIssuerGroup[] ucIssuerGroups) {
    if (ucIssuerGroups.length == 0) {
      throw new IllegalArgumentException();
    }
    
    this.ucIssuerGroups = ucIssuerGroups;
  }
  
  public UseCondIssuerGroup[]
  getUseCondIssuerGroups() {
    return ucIssuerGroups;
  }
  
  protected void setAttributeDirectories(Directory[] attributeDirs) {
    if (attributeDirs.length == 0) {
      throw new IllegalArgumentException();
    }
    
    this.attributeDirs = attributeDirs;
  }
  
  public Directory[]
  getAttributeDirectories() {
    return attributeDirs;
  }
  
  protected void setCacheTime(long cacheTime) {
    this.cacheTime = cacheTime;
  }
  
  public long getCacheTime() {
    return cacheTime;
  }
  
  protected Certificate newInstance() {
    return new PolicyCertificate();
  }
  
  protected void
  doWrite(AkentiOutputStream drain) throws IOException {
    drain.write(resource);
    drain.write(' ');
    drain.write(certAuthInfos.length);
    
    for (int i=0; i < certAuthInfos.length; i++) {
      drain.write(' ');
      certAuthInfos[i].  writeObject(drain);
    }
    
    drain.write(' ');
    drain.write(ucIssuerGroups.length);
    
    for (int i=0; i < ucIssuerGroups.length; i++) {
      drain.write(' ');
      ucIssuerGroups[i].writeObject(drain);
    }
    
    drain.write(' ');
    drain.write(attributeDirs.length);
    
    for (int i=0; i < attributeDirs.length; i++) {
      drain.write(' ');
      attributeDirs[i].writeObject(drain);
    }
    
    drain.write(' ');
    drain.write(cacheTime);
  }
  
  protected void
  doRead(AkentiInputStream source) throws IOException, ParseException {
    resource = source.readString();
    
    int size = source.readInt();
    
    if (size < 0) {
      throw new ParsingException("number of ca infos < 0");
    }
    
    certAuthInfos = new CertificateAuthorityInfo[size];;
    
    for (int i = 0; i < size; i++) {
      certAuthInfos[i] = new CertificateAuthorityInfo(source);
    }
    
    size = source.readInt();
    
    if (size <= 0) {
      throw new ParsingException("number of UseCondGroups <= 0");
    }
    
    ucIssuerGroups = new UseCondIssuerGroup[size];
    
    for (int i = 0; i < size; i++) {
      ucIssuerGroups[i] = new UseCondIssuerGroup(source);
    }
    
    size = source.readInt();
    
    if (size < 0) {
      throw new ParsingException("number of attrDirs < 0");
    }
    
    attributeDirs = new Directory[size];
    
    for (int i = 0; i < size; i++) {
      attributeDirs[i] = new Directory(source);
    }
    
    cacheTime = source.readLong();
  }
  
  protected void doToXML(Element policyCertInfo) {
    Document doc = policyCertInfo.getOwnerDocument();
    Element temp = doc.createElement("ResourceName");
    
    temp.appendChild(doc.createTextNode(resource));
    policyCertInfo.appendChild(temp);
    
    for (int i=0; i < certAuthInfos.length; i++) {
      temp = doc.createElement("CAInfo");
      certAuthInfos[i].toXML(temp);
      policyCertInfo.appendChild(temp);
    }
    
    for (int i=0; i < ucIssuerGroups.length; i++) {
      temp = doc.createElement("UseCondIssuerGroup");
      ucIssuerGroups[i].toXML(temp);
      policyCertInfo.appendChild(temp);
    }
    
    if (attributeDirs.length > 0) {
      Element attrDirs = doc.createElement("AttrDirs");
      
      for (int i=0; i < attributeDirs.length; i++) {
        temp = doc.createElement("URL");
        attributeDirs[i].toXML(temp);
        attrDirs.appendChild(temp);
      }
      
      policyCertInfo.appendChild(attrDirs);
    }
    
    if (cacheTime != -1) {
      temp = doc.createElement("CacheTime");
      temp.appendChild(doc.createTextNode(Long.toString(cacheTime)));
      policyCertInfo.appendChild(temp);
    }
  }
  
  protected void
  doFromXML(Element policyCertInfo) throws ParseException {
    Node child = policyCertInfo.getFirstChild();
    
    if (child == null || child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    Node temp = child.getFirstChild();
    
    if (temp  == null) {
      throw new ParsingException();
    }
    
    resource = temp.getNodeValue();
    
    if (resource == null) {
      throw new ParsingException();
    }
    
    NodeList list = policyCertInfo.getElementsByTagName("CAInfo");
    
    certAuthInfos = new CertificateAuthorityInfo[list.getLength()];
    
    for (int i=0; i < certAuthInfos.length; i++) {
      child = list.item(i);
      certAuthInfos[i] = new CertificateAuthorityInfo((Element) child);
    }
    
    list = policyCertInfo.getElementsByTagName("UseCondIssuerGroup");
    
    int len = list.getLength();
    
    if (len == 0) {
      throw new ParsingException("no uc groups");
    }
    
    ucIssuerGroups = new UseCondIssuerGroup[len];
    
    for (int i=0; i < len; i++) {
      child = list.item(i);
      ucIssuerGroups[i] = new UseCondIssuerGroup((Element) child);
    }
    
    child = child.getNextSibling();
    
    if (child == null) {
      attributeDirs = new Directory[0];
      cacheTime = -1;
      return;
    } else if (child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    // XXX
    if (child.getNodeName().equals("AttrDirs")) {
      list = ((Element) child).getElementsByTagName("URL");
      len = list.getLength();
      
      if (len == 0) {
        throw new ParsingException();
      }
      
      attributeDirs = new Directory[len];
      
      for (int i=0; i < len; i++) {
        attributeDirs[i] = new Directory((Element) list.item(i));
      }
      
      child = child.getNextSibling();
      
      if (child == null) {
        cacheTime = -1;
        return;
      } else if (child.getNodeType() != Node.ELEMENT_NODE) {
        throw new ParsingException();
      }
    }
    
    temp = child.getFirstChild();
    
    if (temp == null) {
      throw new ParsingException();
    }
    
    String rep = temp.getNodeValue();
    
    if (rep == null) {
      throw new ParsingException();
    }
    
    // XXX
    // NumberFormatException
    cacheTime = Long.parseLong(rep);
  }
    
  protected void doToText(StringBuffer sbuf) {
    sbuf.append("  Resource: " + resource + "\n");
    
    CertificateAuthorityInfo[] cais = getCertificateAuthorityInfos();
    if (cais.length != 0) {
      sbuf.append("  Trusted CAs: \n");
      
      for (int i=0; i < cais.length; i++) {
        sbuf.append("    Name: " + cais[i].getName() + "\n");
        
        Directory[] idirs = cais[i].getIdentityDirectories();
        if (idirs.length != 0) {
          for (int j=0; j < cais.length; j++) {
            sbuf.append("    Identity Directories: " + idirs[j] + "\n");
          }
        }

        Directory[] clrdirs = cais[i].getCRLDirectories();
        if (clrdirs.length != 0) {
          for (int j=0; j < cais.length; j++) {
            sbuf.append("    CRL Directories: " + clrdirs[j] + "\n");
          }
        }
      }
    }
    
    sbuf.append("  Stakeholders: \n");
    
    UseCondIssuerGroup[] ucig = getUseCondIssuerGroups();
    for (int i=0; i < ucig.length; i++) {
      sbuf.append("    Stakeholders Group: \n");
      
      AkentiPrincipal[] ap = ucig[i].getUCIssuers();
      for (int j=0; j < ap.length; j++) {
        sbuf.append("      Name: " + ap[j].getName() + "\n");
        sbuf.append("      CA: " + ap[j].getGuarantor() + "\n");
      }
      
      sbuf.append("      UseCondition Directories: \n");
      Directory[] dirs = ucig[i].getDirectories();
      for (int k=0; k < dirs.length; k++) {
        sbuf.append("        " + dirs[k] + "\n");
      }
    }
    
    Directory[] dirs = getAttributeDirectories();
    
    if (dirs.length != 0 ) {
      sbuf.append("  Attribute Directories: \n");
      for (int j=0; j < dirs.length; j++) {
        sbuf.append("    " + dirs[j] + "\n");
      }
    }
    
    long cacheTime = getCacheTime();
    
    if (cacheTime > -1) {
        sbuf.append("  Maximum cache time: " + cacheTime + " seconds\n");
    }
  }

}