package lbl.akenti.dirServices;

import java.net.URL;
import java.util.Map;
import java.security.cert.X509Certificate;
import javax.naming.NamingException;
import lbl.akenti.util.LDAPSearchUtil;

/**
  * Class <code>LdapCertificateRetrieval</code> loads certificates  
  * <i>X509</i> certificates from <i>LDAP </i> directories.
  * 
  * @author  Abdelilah Essiari
  * @see lbl.akenti.util.X509CertReader;
  * @see lbl.akenti.util.LDAPSearchUtil;
  */
final public 
class LdapCertificateRetrieval extends CertificateRetrieval {

  /**
    * Construct this <code>CertificateRetrieval</code>.
    */
  public LdapCertificateRetrieval() {
  }

  /**
    * Retrieves <code>X509Certificate</code>s.
    */
  public X509Certificate[] 
  retrieveX509Certificates(URL url, Map args) {
    try {
      String filter = (String) args.get("base");

      if (filter == null) {
        filter = "cn=*";
      }

      return LDAPSearchUtil.loadX509Certs(url, filter);
    } catch(NamingException ex) {
      return new X509Certificate[0];
    }
  }
}
