package lbl.akenti.disp;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import lbl.akenti.gen.controller.LBLFrame;
import lbl.akenti.gen.controller.WrapUtil;
import lbl.akenti.messaging.CertificateInfo;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.messaging.PolicyInfo;

/**
 * Allows the user to choose a resource and/or view its respective
 * UseConditions
 */

public class LBLPolUCChooseFrame
    extends LBLFrame implements ActionListener, ListSelectionListener {

    private JList polList;
    private JList useCondList;

    private JButton polButton;
    private JButton useCondButton;
    private CheckPolicyResponse resp;

    private Hashtable viewTable;

    /**
     * Constructor
     * Sets up the frame
     *
     * @param title title of the frame
     * @param instructions header/instruction to be displayed
     *                     at top of frame
     */
    public LBLPolUCChooseFrame(String title, String instructions) {
	super(title, instructions, LAST);

	viewTable = new Hashtable();

	polList = new JList(new DefaultListModel());
	polList.setSelectionMode(0);
	polList.addListSelectionListener(this);
	JLabel polLabel = new JLabel("Policy Certificates");

	polButton = new JButton("View");
	polButton.setEnabled(false);
	polButton.setActionCommand("ViewPol");
	polButton.addActionListener(this);
	JPanel polSet = WrapUtil.wrapComponents(new JScrollPane(polList), polButton);
	JPanel polPane = WrapUtil.wrapVertical(polLabel, polSet);

	useCondList = new JList(new DefaultListModel());
	useCondList.setSelectionMode(0);
	useCondList.addListSelectionListener(this);
	JLabel useCondLabel = new JLabel("Use Condition Certificates");

	useCondButton = new JButton("View");
	useCondButton.setEnabled(false);
	useCondButton.setActionCommand("ViewUC");
	useCondButton.addActionListener(this);
	JPanel useCondSet = WrapUtil.wrapComponents(new JScrollPane(useCondList), useCondButton);
	JPanel useCondPane = WrapUtil.wrapVertical(useCondLabel, useCondSet);

	JPanel listPane = WrapUtil.wrapVertical(polPane, useCondPane);
	
	JPanel contentPane = (JPanel) this.getContentPane();
	contentPane.add(listPane, "Center");

	doEnable();

    }

    /**
     * Returns button labels specific for this frame
     */
    protected String [] getButtonLabels(int type) {
	if (type == 2) {
	    return new String[] { "Help", "Back", "Exit"};
	} else {
	    return super.getButtonLabels(type);
	}
    }
    
    /**
     * Called whenever an item is chosen on a list
     */
    public void valueChanged(ListSelectionEvent l) {
	if (l.getSource().equals(polList)) {
	    updateUCList();
	} else {
	    doEnable();
	}
    }

    public void actionPerformed(ActionEvent event) {
	String cmd = event.getActionCommand();

	if (cmd.equals("ViewPol")) {
	    int index = polList.getSelectedIndex();
	    PolicyInfo[] polInfoArray = resp.toPolicyInfosArray();
	    PolicyInfo p = polInfoArray[index];
	    Iterator certInfos = p.getCertificateInfos();
	    CertificateInfo c = (CertificateInfo) certInfos.next();
	    CertInfoViewFrame frame = new CertInfoViewFrame(c);
	    if (viewTable.contains(frame)) {
		CertInfoViewFrame oldFrame = (CertInfoViewFrame) viewTable.get(c.getAkentiCertificate());
		oldFrame.setVisible(true);
		oldFrame.toFront();
	    } else {
		viewTable.put(c.getAkentiCertificate(), frame);
       		frame.pack();
		frame.setVisible(true);
	    }
	} else if (cmd.equals("ViewUC")) {
	    int polIndex = polList.getSelectedIndex();
	    PolicyInfo[] polInfoArray = resp.toPolicyInfosArray();
	    PolicyInfo p = polInfoArray[polIndex];
	    int useCondIndex = useCondList.getSelectedIndex() + 1;
	    CertificateInfo[] certInfoArray = p.toCertificateInfosArray();
	    CertificateInfo c = certInfoArray[useCondIndex];
	    CertInfoViewFrame frame = new CertInfoViewFrame(c);
	    if (viewTable.contains(frame)) {
		CertInfoViewFrame oldFrame = (CertInfoViewFrame) viewTable.get(c.getAkentiCertificate());
		oldFrame.setVisible(true);
		oldFrame.toFront();
	    } else {
		viewTable.put(c.getAkentiCertificate(), frame);
		frame.pack();
		frame.setVisible(true);
	    }
	} else {
	    throw new RuntimeException(cmd + " not supported");
	}
    }

    /**
     * Enables/disables buttons to view certificates
     */
    public void doEnable() {
	polButton.setEnabled(polList.getSelectedIndex() != -1);
	useCondButton.setEnabled(useCondList.getSelectedIndex() != -1);
    }


    /**
     * Updates useCondList to reflect UCs for currently chosen policy
     * Called upon changing the selected policy
     */
    public void updateUCList() {

	DefaultListModel model = (DefaultListModel) useCondList.getModel();
	model.clear();
	if (polList.getSelectedIndex() != -1) {
	    PolicyInfo[] polRespArray = resp.toPolicyInfosArray();
	    PolicyInfo p = polRespArray[polList.getSelectedIndex()];
	    Iterator certInfos = p.getCertificateInfos();
	    CertificateInfo c = (CertificateInfo) certInfos.next();
	    while (certInfos.hasNext()) {
		c = (CertificateInfo) certInfos.next();
		model.addElement(c);
	    }
	}
	doEnable();
    }

    /**
     * Load in proper CheckPolicyResponse, begin processing information
     */
    public void setCheckPolicyResponse(CheckPolicyResponse c) {
	if (c == null) {
	    throw new RuntimeException("No ResourcePolicy found");
	}
	resp = c;
	processLists();
    }

    /**
     * Populate the policy list based on the current CheckPolicyResponse
     */
    public void processLists() {
	DefaultListModel model = (DefaultListModel) polList.getModel();
	model.clear();
	Iterator polInfos = resp.getPolicyInfos();
	while (polInfos.hasNext()) {
	    PolicyInfo p = (PolicyInfo) polInfos.next();
	    Iterator certInfos = p.getCertificateInfos();
	    // first certificate in PolicyResponse must be policy
	    CertificateInfo certInfo = (CertificateInfo) certInfos.next();
	    if (certInfo.getAkentiCertificate().getType().equals("UseCondition")) {
		throw new RuntimeException("No Policy at beginning of PolicyResponse");
	    }
	    model.addElement(certInfo);
	}
    }

    /**
     * Closes all open frames for viewing certificate responses
     */
    public void closeAllViewFrames() {
	Enumeration e = viewTable.elements();
	while (e.hasMoreElements()) {
	    CertInfoViewFrame frame = (CertInfoViewFrame) e.nextElement();
	    frame.setVisible(false);
	}
    }

    /**
     * Clears the frame
     */
    public void clear() {
	DefaultListModel model = (DefaultListModel) polList.getModel();
	model.clear();
	model = (DefaultListModel) useCondList.getModel();
	model.clear();
	viewTable.clear();
	doEnable();
    }
}