package lbl.akenti.disp;

import com.sun.net.ssl.internal.ssl.Provider;
import java.io.FileInputStream;
import java.net.URL;
import java.security.Security;
import java.util.Properties;

import lbl.akenti.gen.controller.AbstractController;
import lbl.akenti.gen.controller.LBLFrame;
import lbl.akenti.gen.controller.LBLResourceFrame;
import lbl.akenti.messaging.CheckPolicyRequest;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.remotePolicyEngine.RemotePolicyEngine;

import java.io.IOException;





/**
 * Checks and displays <i>policy</i> and <i>use conditions</i>
 * for a given <i>resource</i>.
 *
 * @see AbstractController
 */
public class ResrcDispController
    extends AbstractController 
    implements ResrcDispConsts {

    /**
     * Constructor.
     *
     * @see AbstractController#doGetFrameAt(int)
     */
    public ResrcDispController() {
	// We will be using 2 primary frames.
	super(2);
    }

    /** 
     * Shows the first frame.
     *
     * @see AbstractController#start()
     */
    public void start() {
	super.start();
    }

    /**
     * Creates the frame associated with the given index.
     *
     * @param index of the frame to be created
     * @return an LBLFrame that corresponds to index.
     *
     * @see AbstractController#getFrameAt(index)
     */
    protected LBLFrame doGetFrameAt(int index) {
	switch(index) {
	case RESOURCE_INDEX:
	    return new LBLResourceFrame(TITLES[index], HEADERS[index], new String[] { "http" });
	case POL_UC_INDEX:
	    return new LBLPolUCChooseFrame(TITLES[index], HEADERS[index]);
	default:
	    throw new RuntimeException();
	}
    }

    /**
     * Loads inputs in the frame that corresponds to <i>index</i>.
     *
     * @param index index of frame for which inputs are to be loaded
     *
     * @return true if this method should be called when displaying this
     *              frame in the future.
     *         false otherwise.
     *
     * @see AbstractController#showNext()
     * @see AbstractController#loadInputsIntoFrameAt(int)
     * @see AbstractController#clearBits()
     */
    public boolean loadInputsIntoFrameAt(int index) throws Exception {
	switch (index) {
	case POL_UC_INDEX:
	    LBLResourceFrame resourceFrame = (LBLResourceFrame) getFrameAt(RESOURCE_INDEX);
	    URL base = new URL(resourceFrame.getBaseURL());
	    String resource = resourceFrame.getResource();
            
            if (!base.getProtocol().equals("http")) {
              throw new IOException("'" + base.getProtocol() + "' protocol not supported.\n " +
              "Must be talking to AkentiServer via http.");
            }
            
	    CheckPolicyRequest cpr = new CheckPolicyRequest(RemotePolicyEngine.getID(), resource);
	    RemotePolicyEngine eng = new RemotePolicyEngine();
	    CheckPolicyResponse resp = eng.process(base, cpr);

	    //ResourcePolicy resPol = new ResourcePolicy();
	    LBLPolUCChooseFrame chooseFrame = (LBLPolUCChooseFrame) getFrameAt(POL_UC_INDEX);
	    chooseFrame.clear();
	    chooseFrame.setCheckPolicyResponse(resp);
	    return true;
	default:
	    throw new RuntimeException();
	}
    }

    /**
     * Returns help message for frame
     *
     * @param index index of current frame
     * @return String containing help message for corresponding frame
     */
    protected String getHelpMessage(int index) {
	return helpMess[index];
    }

    /**
     * Load properties and start GUI
     */
    public static void main (String[] args) {
	try {
	    // load properties
	    if (args.length == 1) {
		Properties props = System.getProperties();
		props.load(new FileInputStream(args[0]));
		System.out.println(props);
	    }

	    Security.addProvider(new Provider());

	    new ResrcDispController().start();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
