package lbl.akenti.gen.controller;

import javax.swing.Icon;

import javax.swing.filechooser.FileSystemView;
import java.io.File;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import lbl.akenti.remotePolicyEngine.RemotePolicyEngine;
import lbl.akenti.messaging.ResourceBrowseRequest;
import lbl.akenti.messaging.ResourceBrowseResponse;
import org.apache.soap.SOAPException;

/**
 */
class DFileSystemView extends FileSystemView {
  
  final static File ROOT = new File("/");
  
  private String base;
  
  /**
   */
  DFileSystemView(String base) {
    this.base = base;
  }
  
  /**
   */
  String getBase() {
    return base;
  }
  
  /**
   */
  public boolean isRoot(File f) {
    if (f.equals(ROOT)) {
      return true;
    }
    
    return false;
  }
  
  /**
   */
  public Boolean isTraversable(File f) {
    return super.isTraversable(f);
  }
  
  /**
   */
  public String getSystemDisplayName(File f) {
    return super.getSystemDisplayName(f);
  }
  
  /**
   */
  public Icon getSystemIcon(File f) {
    return super.getSystemIcon(f);
  }
  
  /**
   */
  public boolean isParent(File folder, File file) {
    return super.isParent(folder, file);
  }
  
  /**
   */
  public File getChild(File parent, String fileName) {
    throw new RuntimeException();
  }
  
  /**
   */
  public boolean isFileSystem(File f) {
    return true;
  }
  
  /**
   */
  public boolean isHiddenFile(File f) {
    return super.isHiddenFile(f);
  }
  
  /**
   */
  public File createNewFolder(File containingDir) throws IOException {
    throw new IOException("CANNOT CREATE DIR");
  }
  
  /**
   */
  public File[] getRoots() {
    return new File[] { ROOT };
  }
  
  public File getHomeDirectory() {
    return ROOT;
  }
  
  public File getDefaultDirectory() {
    return ROOT;
  }
  
  static class DFile extends File {
    DFile(File dir, String filename) {
      super(dir, filename);
    }
    
    DFile(String path) {
      super(path);
    }
    
    public boolean isDirectory() {
      return true;
    }
    
    public boolean exists() {
      return true;
    }
    
    public boolean isFile() {
      return false;
    }
    
    public File getParentFile() {
      String path = getParent();
      
      if (path == null) { return null; }
      
      return new DFile(path);
    }
    
    public File getCanonicalFile() throws IOException {
      return this;
    }
    
  }
  
  public File createFileObject(File dir, String filename) {
    return new DFile(dir, filename);
  }
  
  /**
   */
  public File createFileObject(String path) {
    return new DFile(path);
  }
  
  /**
   */
  public File getParentDirectory(File dir) {
    if (dir.equals(ROOT)) {
      return null;
    }
    
    return createFileObject(dir.getParent());
  }
  
  int x = 0;
  
  /**
   */
  public synchronized File[]
  getFiles(File dir, boolean useFileHiding) {
    File[] ret = new File[0];
    
    // TODO: 
    // We get called three times during initialization (JFileChooser)

    if (x < 2) {
      x++;
      return ret;
    }

    try {
      URL url = new URL(base);
      
      if (url.getProtocol().equals("file")) {
        ret = listFileResources(url.getFile(), dir);
      } else if (dir.equals(ROOT)) {
        ret = listHttpResources(base, dir);
      } else {
        String temp = dir.toString();
        
        temp += "/";
        ret = listHttpResources(base + temp, dir);
      }
    } catch(RuntimeException ex) {
      LBLDialogFactory.showInfoDialog(null, 
                                      "ERROR:" + ex.getClass().getName(), 
                                      ex.getMessage());
    } catch(Exception ex) {
      LBLDialogFactory.showInfoDialog(null, 
                                      ex.getClass().getName(), 
                                      ex.getMessage());

    }
    
    return ret;
  }
  
  
  /**
   * Gets all available resources from the URL.
   *
   * @param String URL where to begin search
   * @return String[] array of all available resources
   */
  private File[] listHttpResources(String url, File dir)
  throws MalformedURLException, IOException, SOAPException{
    String resName = dir.toString();

    if(resName.startsWith("/") ) {
      resName = resName.substring(1);
    }
    
    RemotePolicyEngine eng = new RemotePolicyEngine();
    ResourceBrowseRequest req = new ResourceBrowseRequest(RemotePolicyEngine.getID(), resName);
    ResourceBrowseResponse resp = eng.process(new URL(url), req);
    
    String[] dirFiles = resp.getResources();
    ArrayList dirs = new ArrayList(dirFiles.length);
    
    for (int i=0; i < dirFiles.length; i++) {
      dirs.add(createFileObject(dir,dirFiles[i]));
    }
    
    return (File[]) dirs.toArray(new File[0]);
  }
  
  /**
   */
  private File[] listFileResources(String base, File dir) throws IOException {
    File file = new File(base + dir.getName());
    
    String []temp = file.list();
    
    if (temp == null) {
      throw new IOException("File not found:" + file.getAbsolutePath());
    }
    
    ArrayList dirs = new ArrayList(temp.length);
    
    for (int i = 0; i < temp.length; ++i) {
      
      if (new File(base + dir.toString() + "/" + temp[i]).isDirectory()) {
        dirs.add(createFileObject(dir, temp[i]));
      }
    }
    
    return (File[]) dirs.toArray(new File[0]);
  }
}
