package lbl.akenti.gen.controller;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;

import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.URL;
import java.net.MalformedURLException;

import lbl.akenti.gen.common.GenCa;
import lbl.akenti.certs.elements.Directory;

/**
  * This option pane allows the user to construct a directory.
  *
  * @see javax.swing.JPanel
  */
public class LBLBuildDirPane 
  extends JOptionPane implements ItemListener, ActionListener , DocumentListener  {
  public static final String[] PROTOCOLS = { "ldap", "http", "file" };

  private final static String BUILD_DIR_PANEL_MESSAGE =
    "<html><font size=+1 color=blue>Build Directory" +
    "<li><font color=blue>Choose a protocol and provide " +
         "the appropriate components of a url" +
    "<li><font color=blue>hostname, port, file or search base if ldap....";

  private final static String HELP_MESSAGE =
    "When protocol is \"file\" Host and Port are not applicable.";

  private JComboBox mComboBox = new JComboBox();

  private JLabel baseLabel = new JLabel("");
  private JTextField hostField;
  private JTextField portField;
  private JTextField fileField;

  private JButton helpButton;
  private JButton okButton;
  private JButton hintButton;
  private JButton browse;

  
  private GenCa[] cas;

  /** 
    * Constructor 
    */
  public LBLBuildDirPane() {
    this(null);
  }

  /**
    */
  public LBLBuildDirPane(String []prots) {
    this(null, prots);
  }

  /**
    *
    * North:
    *
    *   AKTxtArea or a Label containing <i>msg</i> 
    *
    * Center :
    *
    *   ComboBox containing <i>protocols</i> 
    *   3 TextFields  (host port, file/base)
    *
    * South:
    *
    *   Buttons (ok, cancel) 
    *
    */
  public LBLBuildDirPane(String msg, String []prots) {
    setLayout(new BorderLayout(5, 5));
    
    if (msg == null) {
      msg = BUILD_DIR_PANEL_MESSAGE;
    }

    if (msg.startsWith("<html>")) {
      add(LBLLabelFactory.createHtmlLabel(msg), "North");
    } else {
      add(new AKTxtCanvas(msg), "North");
    }

    hostField = LBLTextFieldFactory.createTextField(": /", this);
    portField = LBLTextFieldFactory.createNumberField(this);
    fileField = LBLTextFieldFactory.createTextField(":", this);

    browse = new JButton("Browse");
    browse.setActionCommand("browse");
    browse.addActionListener(this);

    mComboBox.addItemListener(this);

    if (prots == null || prots.length == 0) {
      prots = PROTOCOLS;
    }

    for (int i = 0; i < prots.length; i++) {
      mComboBox.addItem(prots[i]);
    }

    itemStateChanged(null);

    JLabel labels[] = {
       new JLabel("Protocol"), 
       new JLabel("Host"),
       new JLabel("Port"), 
       baseLabel
    };

    for (int i = 0; i < labels.length; ++i) {
      labels[i].setForeground(Color.black);
    }

    JComponent []components = new JComponent[labels.length];

    components[0] = mComboBox;
    components[1] = hostField;
    components[2] = portField;
    components[3] = fileField;
    add(WrapUtil.wrapComponents(labels, components), "Center");
    
    hintButton = new JButton("Use Hints");
    LBLButtonPanel.enableBlueButton(hintButton);
    hintButton.addActionListener(this);

    JPanel temp = new JPanel();

    temp.add(hintButton);
    add(temp, "East");

    JPanel buttonPanel = new JPanel();

    helpButton = new JButton("Help");
    helpButton.setActionCommand("help");
    helpButton.addActionListener(this);
    buttonPanel.add(helpButton);
    
    okButton = new JButton("OK");
    okButton.setActionCommand("OK");
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    
    JButton cancelButton = new JButton("CANCEL");

    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);

    // buttonPanel.add(browse);

    add(buttonPanel, "South");
    doEnable();
  }

  /**
    */
  public void setGenCas(GenCa []cas) {
    this.cas = cas;
    doEnable();
  }

  /**
    */
  public void setOkLabel(String label) {
    okButton.setText("Continue");
  }

  /**
    */
  public void changedUpdate(DocumentEvent e)  {
    doEnable();
  }
  
  /**
    */
  public void insertUpdate(DocumentEvent e)  {
    doEnable();
  }

  /**
    */
  public void removeUpdate(DocumentEvent e) {
    doEnable();
  }

  /**
    */
  private void doEnable() {
    hintButton.setVisible(cas != null && cas.length != 0);

    if ((((String) mComboBox.getSelectedItem())).equalsIgnoreCase("file")) {
      doEnable(getFile().length() != 0);
    } else if ((((String) mComboBox.getSelectedItem())).equalsIgnoreCase("ldap")) {
      doEnable(getFile().length() != 0 && getHost().length() != 0);
    } else {
      doEnable(getHost().length() != 0);
    }

  }
  
  /**
    */
  private void doEnable(boolean on) {
    if (on) {
      LBLButtonPanel.enableBlueButton(okButton);
    } else {
      LBLButtonPanel.disableBlueButton(okButton);
    }
  }
  
  /**
    */
  public void itemStateChanged(ItemEvent evt) {
    clearTextFields();

    String prot = (String) mComboBox.getSelectedItem();
    browse.setEnabled(getProtocol().equals("file"));

    if (prot.equalsIgnoreCase("file")) {
      hostField.setEditable(false);
      portField.setEditable(false);
      baseLabel.setText("File"); 
      return;
    } else if (prot.equalsIgnoreCase("ldap")) {
      hostField.setEditable(true);
      portField.setEditable(true);
      baseLabel.setText("Base"); 
      return;
    } else {
      hostField.setEditable(true);
      portField.setEditable(true);
      baseLabel.setText("File");
    }
  }

  /**
    */
  public void clear() {
    cas = null;
    clearTextFields();
    doEnable();
    setValue(null);
  }

  /**
    */
  private void clearTextFields() {
    hostField.setText("");
    portField.setText("");
    fileField.setText("");
  }

  /**
    */
  public String getDirectory() {
    StringBuffer sb = new StringBuffer(getProtocol());

    String temp = getHost();
    
    sb.append(':');

    if (!temp.equals("")) {
      sb.append('/');
      sb.append('/');
      sb.append(temp);

      temp = getPort();

      if (!temp.equals("")) {
        sb.append(':');
        sb.append(temp);
      }
    }

    sb.append(getFile());
    return sb.toString();
  }

  /**
    */
  private String getProtocol () {
    return (String) mComboBox.getSelectedItem();
  }

  /**
    */
  private String getHost() {
    return (String) hostField.getText();
  }

  /**
    */
  private String getPort() {
    return (String)portField.getText();
  } 
    
  /**
    */
  private String getFile() {
    String file = fileField.getText();
    
    if (file.length() == 0) {
      return "";
    }

    if (file.charAt(0) == '/') {
      return (String) file;
    }

    return "/" + file;
  }
   
  /**
    */
  public void actionPerformed(ActionEvent evt) {
    String cmd = evt.getActionCommand();

    if (cmd.equals("help")) {
      showDialog("Info", HELP_MESSAGE); 
    } else if (cmd.equals("OK")) {
      setValue(getDirectory());
    } else if (cmd.equals("Use Hints")) {
      useHints();
    } else if (cmd.equals("browse")) {
      System.out.println("BROWSE ACTION REQUESTED");
    } else {
      setValue(GenConsts.AK_CANCEL_VALUE);
    }
  }

  protected void showDialog(String title, String info) {
    LBLDialogFactory.createInfoDialog(
      this,
      title,
      (info == null? "" : info)
    ).setVisible(true);
  }

  /**
    */
  void useHints() {
    if (cas.length == 0) {
      showMessage("<html><font color=red>No Certificate Authority Infos");
      return;
    }

    GenCa ca = (GenCa) LBLDialogFactory.showListDialog(
      this,
      "Certificate Authorities",
      "<html><font color=blue> Select a Certificate Authority",
      cas,
      0
    );

    if (ca == null) {
      return;
    }

    Directory []dirs = ca.getDirs();

    if (dirs.length == 0) {
      showMessage("<html><font color=red> No Directories for selected CA");
      return;
    }

    Directory dir = (Directory) LBLDialogFactory.showListDialog(
      this,
      "Directories",
      "<html><font color=blue> Select a directory",
      dirs,
      0
    );

    if (dir == null) {
      return;
    }

    URL url;

    try {
      url = new URL(dir.toString());
    } catch(MalformedURLException ex) {
      throw new RuntimeException("hummmm");
    }

    mComboBox.setSelectedItem(url.getProtocol());
    hostField.setText(url.getHost());

    int port = url.getPort();

    if (port != -1) {
      portField.setText(String.valueOf(port));
    }

    String base = url.getFile();

    if (base.startsWith("/")) {
      base = base.substring(1);
    }

    fileField.setText(base);
  }
  
  private void showMessage(String message) {
    LBLDialogFactory.createInfoDialog(
      this,
      "Error Message",
      message
    ).setVisible(true);
  }
}
