package lbl.akenti.gen.controller;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.DefaultListModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import lbl.akenti.certs.elements.AkentiPrincipal;

/**
  * Allows a user to choose the certificate issuer.
  *
  * This sounds funny but the user and issuer are 
  * one and the same and that is why we have two flavors:
  *
  * Optional: when we dont really know the issuers.
  * Required: when we know exactly who the issuers can be.
  */
public class LBLIssuerFrame extends LBLFrame implements ListSelectionListener {
    private JList list;

    private boolean optional;

    /** 
      * Constructor.
      */
    public LBLIssuerFrame(String TITLE, String HEADER) {
	super(TITLE, HEADER);
	
	JPanel contentPane = (JPanel) this.getContentPane();
	
	JPanel temp = new JPanel(new BorderLayout(5, 5));
	JLabel label = new JLabel ("Issuer, Issuer's Certificate Authority");
	
	label.setForeground(Color.red);
	temp.add(label, "North");
	
	list = new JList(new DefaultListModel());
	list.addListSelectionListener(this);
	temp.add(new JScrollPane(list) , "Center");
	contentPane.add(temp, "Center");
        doEnable(false);	
    }
    
    /**
      */
    public void setOptional(boolean optional) {
      this.optional = optional;
      doEnable(false);	
    }

    /**
      */
    public void valueChanged(ListSelectionEvent e) {
	doEnable(list.getSelectedIndex() != -1);
    }
    
    /**
      * Called When Cancel is clicked
      */
    public void clear() {
	DefaultListModel model = (DefaultListModel) list.getModel();

	model.clear();
        doEnable(false);
    }

    /**
      */
    private void doEnable(boolean on) {
        getButton("Next").setVisible(optional || on);
    }
    
    /**
      */
    public void setIssuers(AkentiPrincipal []issuers) {
        clear();

	DefaultListModel model = (DefaultListModel) list.getModel();
	
	for (int i = 0; i < issuers.length; i++) {
	    model.addElement(issuers[i]);
	}
    }
    
    public AkentiPrincipal getIssuer() {
	return (AkentiPrincipal) list.getSelectedValue();
    }
}
