package lbl.akenti.gen.controller;

import javax.swing.JTextField;
import javax.swing.JPasswordField;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.AttributeSet;
import javax.swing.event.DocumentListener;
import java.awt.Color;

/**
  * This factory should be used when creating textfields.
  *
  * One can specify a filter which is basically a string 
  * of characters that should not be accepted by the textfield
  * created.
  *
  * Additionally one can register a <i>DocumentListener</i>
  * that can be used to catch <i>DocumentEvent</i>s.
  * 
  */
public class LBLTextFieldFactory { 
  
  public static final String URL_FILTER = "!@#$%^&*{}[]\\|()~`\'+=<>\t ";
  public static final String FILE_FILTER = URL_FILTER + ":";

  /**
    */
  private LBLTextFieldFactory() {
  }

  /**
    */
  public static JTextField createTextField() {
    JTextField textField = new JTextField();

    textField.setForeground(Color.blue);
    textField.setBackground(Color.white);
    return textField;
  }

  /**
    */
  public static JTextField createTextField(DocumentListener docListener) {
    JTextField textField = new JTextField();

    textField.setForeground(Color.blue);
    textField.getDocument().addDocumentListener(docListener);
    return textField;
  }

  /**
    */
  public static JTextField createURLTextField() {
    return createURLTextField(null);
  }

  /**
    */
  public static JTextField createURLTextField(DocumentListener docListener) {
    return createTextField(URL_FILTER, docListener);
  }

  /**
    */
  public static JTextField createFileTextField() {
    return createFileTextField(null);
  }

  /**
    */
  public static JTextField createFileTextField(DocumentListener docListener) {
    return createTextField(FILE_FILTER, docListener);
  }

  /**
    */
  public static JTextField createTextField(String filter) {
    return createTextField(filter, null);
  }

  /**
    */
  public static JTextField 
  createTextField(final String filter, DocumentListener docListener) {
    JTextField textField = createTextField();

    Document doc = new PlainDocument() {
      public void insertString(int offs, String str, AttributeSet a) 
        throws BadLocationException {
 
        if (str == null) {
          return;
        }

        for (int i = 0; i < str.length(); ++i) {
          if (filter.indexOf(str.charAt(i)) != -1) {
            return;
          }
        }

        super.insertString(offs, str, a);
      }
    };
   
    textField.setDocument(doc);
    
    if (docListener != null) {
      doc.addDocumentListener(docListener);
    }
    
    return textField;
  }

  /**
    */
  public static JPasswordField createPasswordField() {
    return createPasswordField(null);
  }

  /**
    */
  public static JPasswordField 
  createPasswordField(DocumentListener docListener) {
    JPasswordField passwordField = new JPasswordField();

    passwordField.setForeground(Color.blue);
    
    
    if (docListener != null) {
      passwordField.getDocument().addDocumentListener(docListener);
    }

    return passwordField;
  }

  /**
    */
  public static JTextField createNumberField() {
    return createNumberField(null);
  }

  public static JTextField createNumberField(DocumentListener docListener) {
    JTextField textField = createTextField();

    Document doc = new PlainDocument() {
      public void insertString(int offs, String str, AttributeSet a) 
        throws BadLocationException {
 
        if (str == null) {
          return;
        }

        for (int i = 0; i < str.length(); ++i) {
          if (!Character.isDigit(str.charAt(i))) {
            return;
          }
        }

        super.insertString(offs, str, a);
      }
    };
   
    textField.setDocument(doc);
    
    if (docListener != null) {
      doc.addDocumentListener(docListener);
    }

    return textField;
  }

}

