package lbl.akenti.gen.controller.uc;

/**
 * A class to contain the String constants used by the ucGenerator
 */
public interface UCConsts {

    // Indices for accessing the controllers frames 
  public static final int RESOURCE_INDEX = 0;
  public static final int ISSUER_INDEX = 1;
  public static final int EXPR_INDEX = 2;
  public static final int SCOPE_INDEX = 3;
  public static final int TIME_INDEX = 4;
  public static final int REVIEW_INDEX = 5;   
  public static final int SIGN_INDEX = 6;

  static final String[] TITLES =  {
    "Resource Information",
    "Use Condition Issuer Info",
    "Expression Builder",
    "Scope and Actions",
    "Certificate Lifetime",
    "Certificate Content",
    "Sign And Save Certificate"};

  // Resource Frame
  public static String RESOURCE_HEADER =
    "<html><font size=+1 color=blue>"+TITLES[RESOURCE_INDEX] +
    "<li><font color=blue>Enter the name of the resource.<i>" +
    "<li><font color=blue>Enter the base url."
  ; // end of header

  public static String ISSUER_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[ISSUER_INDEX] +
    "<li><font color=blue>Select Use Condition Issuer (yourself) from this list."
  ;
  
  // Expression Builder Frame
  public static String EXPR_HEADER =
    "<html><font size=+2 color=blue>" + TITLES[EXPR_INDEX] +
    "<li><font color=blue>Select an attribute type" +
    "<li><font color=blue>Specify an attribute name" +
    "<li><font color=blue>Specify a value" +
    "<li><font color=red>The Not operator can only used with X509 attributes"
  ;

  public static String SCOPE_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[SCOPE_INDEX] + 
    "<li><font color=blue>If <i><font color=red>scope</i> is local" +
    " the Use Condition will only apply<BR>to this resource." +
    "<li><font color=blue>If <i><font color=red>scope</i> is subtree" +
    " it will apply to the whole subtree<BR>of resouces." +
    "<li><font color=blue>Checking <i><font color=red>critical</i>" +
    " makes the Use Condition<BR>mandatory (users must satisfy it)." +
    "<li><font color=blue>Specify the <i><font color=red>actions</i>" +
    " to allow for this Use Condition.";
  ;
	
  // Time Frame
  public static String TIME_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[TIME_INDEX] +
    "<li><font color=blue>Choose how long the certificate will be valid for."
  ; // end of header

  // Review Frame
  public static String REVIEW_HEADER =
    "<html><font size=+1 color=blue>Review your Use Condition Certificate" +
    "<li><font color=blue>Click <i>next</i> when ready.";
  ;

  // Sign And Save Frame
  public static String SIGN_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[SIGN_INDEX] +
    "<li><font color=blue>Enter your <font color=red>private key</font> file path " +
      "and <font color=red>passphase</font> if <br> did not already provide." +
      "<li><font color=blue>Enter the full path of the file where" +
      " the newly <br>generated Attribute Certificate will be stored."
  ;

  static final String[] HEADERS = {
    RESOURCE_HEADER, 
    ISSUER_HEADER, 
    EXPR_HEADER,
    SCOPE_HEADER,
    TIME_HEADER,
    REVIEW_HEADER,
    SIGN_HEADER
  };

  static final String[] helpMessages = {
    // Resource Frame Help message
    "The resouce can be hierarchical or flat. " +
    "The relative resource name is appended to the base url " +
    "which the generator uses to find information about this resource. " +
    "\n\nA base url, different from the default one, " +
    "can be chosen using the Create button, or be typed in. " +
    "If you are not sure of the correct relative resource name " +
    "clicking on the Browse button will give you a file browse window  " +
    "that server controls. The Policy Definition Server returns the contents of the " +
    "authorization file (which lists the stakeholders for the resource) " +
    "and the resource attributes file which contains the attributes, " +
    "values, and actions that are defined for the resource.\n\n" +
    "The host and port constitute the address of the Policy Definition Server." +
    "\nAll default values are read from the AkentiGen.conf file.",

    // Issuer Help message
    "Only these users are recognized stakeholders for this resource. "+
    "If your name does not appear here, contact one of the other " +
    "stakeholders or the administrator for the resource tree." +
    " Stakeholders for a resource are defined in the authorization " +
    "file for the resource.",

    // Expression builder frame help message
    "Attributes and values that are meaningful are defined in the " +
    "resource attributes file for a specific resource. The following " +
    "attributes are defined by a user's X.509 identity certificate and "+
    "can be used for any resource:\n " +
    "    o - Organization to which the user belongs\n" +
    "    ou - Organizational Unit to which the user belongs\n" +
    "    cn - Common name of the user\n" +
    "For each attribute and value, you must select an attribute issuer " +
    "and its CA, that is trusted to certify the user's possession of this " +
    "attribute. In the case of the X509 attributes, the CA is both  " +
    "the issuer and CA.",

    // Scope and Access Frame help message
    "If the resource tree consists of hierarchal resources, Use " +
    "Condition certificates can apply to just one resource (local) " +
    "or to all the resources including and below a point in " +
    "the hierarchy (sub-tree). If the resource only contains a single " +
    "level, chose scope local.  Checking critical makes the UseCondition " +
    "mandatory (user must satisfy it).",

    // TimeFrame help message 
    "The certificate valdity period with begin with the current time " + 
    "and end after curent time plus the duration chosen here.",

    // Review Frame help message
    "At this point you can use the Back button to go back to any " +
    "previous window where you can re-enter any information that "+
    "you may want to correct.",

    // Sign and Save frame help message
    "Your keyfile has to be a pkcs12 (.p12) file. The UseCondition " +
    "certificate file must be stored in a directory that is " +
    "specified in the Policy Certificate for this resource."
  };
}

