package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiMsgType;

/**
  * Associates a code with a message.
  *
  * @author Abdelilah Essiari 
  */

final public 
class AkentiMessage {
  private int code;
  private String message;

  /**
    * Default constructor.
    */
  public AkentiMessage() {
    this(-1, "NO MESSAGE");
  }

  /**
    * Constructs this object.
    */
  public AkentiMessage(int code, String message) {
    if (message == null) {
      throw new IllegalArgumentException();
    }

    this.code = code;
    this.message = message;
  }

  /**
    * Returns the <i>code</i> associated with <i>AkentiMessage</i>
    */
  public int getCode() {
    return code;
  }

  /**
    * Returns the <i>message</i> associated with <i>AkentiMessage</i>
    */
  public String getMessage() {
    return message;
  }

  /**
    */
  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append(code);
    sb.append(' ');
    sb.append(message);
    return sb.toString();
  }

  /**
    * Used for XML.
    */
  AkentiMessage(AkentiMsgType impl) {
    this(impl.getCode(), impl.getStatus());
  }

  /**
    * Used for XML.
    */
  AkentiMsgType toJAXBImpl() throws JAXBException {
    AkentiMsgType impl = Util.getObjectFactory().createAkentiMsgType();

    impl.setCode(getCode());
    impl.setStatus(getMessage());
    return impl;
  }
}
