package lbl.akenti.messaging;

import lbl.akenti.jaxb.AkentiRequestType;
import javax.xml.bind.JAXBException;

/**
  * Encapsulates the various requests that are handed to the 
  * <i>Akenti Server</i>.                                        
  *
  * Each request contains a client-generated unique id that can be useful 
  * when clients are sending or keeping track of multiple requests.
  * 
  * @author Abdelilah Essiari.
  */

public abstract
class AkentiRequest {
  private int id;

  /**
    * Constructs a request.
    */
  protected  AkentiRequest(int id) {
    if (id < 0) {
      throw new IllegalArgumentException ();
    }

    this.id = id;
  }

  /**
    * Returns the client-generated unique id.
    */
  public int getID() {
    return id;
  }

  /**
    * JAXB STUFF
    */
  AkentiRequest(AkentiRequestType impl) {
    id = impl.getID();
  }
  
  /**
    * TODO:
    *
    * JAXB STUFF
    * HERE WE ARE BREAKING THE RULES TO MAKE 
    * JAXB/SOAP/C++ PARSING HAPPY.
    * TO BE REVISITED.
    *
    */
  public final AkentiRequestType toJAXBImpl(boolean type) throws JAXBException {
    AkentiRequestType impl = doToJAXBImpl(type);

    impl.setID(getID());
    return impl;
  }

  /**
    * JAXB STUFF
    */
  abstract protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException;
}
