package lbl.akenti.messaging;

import java.io.Writer;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;
import java.text.ParseException;
import org.w3c.dom.Element;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiResponsesGenerated;
import lbl.akenti.jaxb.AkentiResponseType;
import lbl.akenti.jaxb.CheckAccessResponseType;
import lbl.akenti.jaxb.CheckPolicyResponseType;
import lbl.akenti.jaxb.ResourceBrowseResponseType;
import lbl.akenti.jaxb.CreateResourceResponseType;
import lbl.akenti.jaxb.UploadPolicyCertificateResponseType;
import lbl.akenti.jaxb.GetResourceInfoResponseType;


/**
  */
final public
class AkentiResponses {
  private List responses;

  /**
    */
  public AkentiResponses(AkentiResponse ar) {
    this(new AkentiResponse[]{ar});
  }

  /**
    */
  public AkentiResponses(AkentiResponse[] ars) {
    responses = new ArrayList(ars.length);
    responses.add(ars[0]);
   
    for (int i = 1; i < ars.length; ++i) {
      responses.add(ars[i]);
    } 

    responses = Collections.unmodifiableList(responses);
  }

  /**
    */
  public AkentiResponses(InputStream in) throws IOException, ParseException {
    try {
      init((AkentiResponsesGenerated) Util.getUnmarshaller().unmarshal(in));
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }

  /**
    */
  public AkentiResponses(Element element) throws IOException, ParseException {
    try {
      init((AkentiResponsesGenerated) Util.getUnmarshaller().unmarshal(element));
    } catch(JAXBException ex) {
      throw new ParseException(ex.getMessage(), -1);
    }
  }

  /**
    * Returns the responses as an iterator.
    *
    * @see AkentiResponse
    */
  public Iterator getAkentiResponses() {
    return responses.listIterator();
  }
  
  /**
    * Returns the responses as an array.
    *
    * @see AkentiResponse
    */
  public AkentiResponse[] toAkentiResponsesArray() {
    AkentiResponse[] ret = new AkentiResponse[responses.size()];
 
    responses.toArray(ret);
    return ret;
  }

  /**
    */
  public void toStream(OutputStream out) throws IOException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), out);
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }

  /**
    */
  public void toWriter(Writer writer) throws IOException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), writer);
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }

  /**
    */
  public void toElement(Element element) throws ParseException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), element);
    } catch(JAXBException ex) {
      throw new ParseException(ex.getMessage(), -1);
    }
  }

  /**
    */
  private void init(AkentiResponsesGenerated impl) throws IOException, ParseException {
    List temp = impl.getAkentiResponse();
    Iterator iter = temp.listIterator();

    responses = new ArrayList(temp.size());

    do {
      AkentiResponseType resp = (AkentiResponseType) iter.next();
 
      if (resp instanceof CheckAccessResponseType) {
        responses.add(new CheckAccessResponse((CheckAccessResponseType) resp));
      } else if (resp instanceof CheckPolicyResponseType) {
        responses.add(new CheckPolicyResponse((CheckPolicyResponseType) resp));
      } else if (resp instanceof ResourceBrowseResponseType) {
        responses.add(new ResourceBrowseResponse((ResourceBrowseResponseType) resp));
      } else if (resp instanceof CreateResourceResponseType) {
        responses.add(new CreateResourceResponse((CreateResourceResponseType) resp));
      } else if (resp instanceof UploadPolicyCertificateResponseType) {
        responses.add(new UploadPolicyCertificateResponse((UploadPolicyCertificateResponseType) resp));
      } else if (resp instanceof GetResourceInfoResponseType) {
        responses.add(new GetResourceInfoResponse((GetResourceInfoResponseType) resp));
      } else {
        throw new RuntimeException();
      }
    } while (iter.hasNext());

    responses = Collections.unmodifiableList(responses);
  }

  /** 
    * FOR XML STUFF
    */
  private AkentiResponsesGenerated toJAXBImpl() throws JAXBException {
    AkentiResponsesGenerated impl = Util.getObjectFactory().createAkentiResponsesGenerated();
    Iterator iter = getAkentiResponses();
    List temp = impl.getAkentiResponse();

    do {
      AkentiResponse ar = (AkentiResponse) iter.next();

      temp.add(ar.toJAXBImpl());
    } while (iter.hasNext());

    return impl;
  }
}
