package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import java.text.ParseException;
import lbl.akenti.jaxb.CheckAccessRequestType;
import lbl.akenti.jaxb.PrincipalType;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.certs.elements.AkentiPrincipal;

/**
  * Holds the minimum inputs to the <i> Akenti Authorization Engine</i>.
  * These are an entity's DN/CADN pair and the name of the resource 
  * the entity is trying to access. Other inputs will be added in the
  * future.
  *
  * @author Abdelilah Essiari
  **/

final public 
class CheckAccessRequest extends AkentiRequest {
  private AkentiPrincipal ap;
  private String resource;

  /**
    * Constructs a <i>CheckAccessRequest</i>.
    *
    * @param ap the DN/CADN pair of an entity.
    * @param resource name of a resource
    * @see AkentiPrincipal
    */
  public CheckAccessRequest(int uid, AkentiPrincipal ap, String resource) {
    super(uid);

    if (ap == null || resource.length() == 0) {
      throw new IllegalArgumentException();
    }

    this.ap = ap;
    this.resource = resource;
  }

  /**
    * Returns the name of the resource.
    */
  public String getResource() {
    return resource;
  }

  /**
    * Returns the DN/CADN pair of the entity.
    *
    * @see AkentiPrincipal
    */
  public AkentiPrincipal getAkentiPrincipal() {
    return ap;
  }

  /**
    * JAXB STUFF.
    */
  CheckAccessRequest(CheckAccessRequestType impl) throws ParseException {
    super(impl);
    resource = impl.getResource(); 
    ap = fromJAXBImpl(impl.getPrincipal());
  }
  
  /**
    * JAXB STUFF. 
    */
  protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException {
    CheckAccessRequestType impl = type? Util.getObjectFactory().createCheckAccessRequestType()
                                        : Util.getObjectFactory().createCheckAccessRequestGenerated();

    impl.setPrincipal(toJAXBImpl(getAkentiPrincipal()));
    impl.setResource(getResource());
    return impl;
  }

  /**
    * JAXB STUFF. 
    * TODO REMOVE THIS FROM HERE WHEN JAXBING THE CERTS.ELEMENTS
    * PACKAGE
    */
  private static AkentiPrincipal fromJAXBImpl(PrincipalType impl) throws ParseException {
    return new AkentiPrincipal(impl.getUserDN(), impl.getCADN());
  }
 
  /**
    * JAXB STUFF.
    * TODO REMOVE THIS FROM HERE WHEN JAXBING THE CERTS.ELEMENTS
    * PACKAGE
    */
  private static PrincipalType toJAXBImpl(AkentiPrincipal ap) throws JAXBException {
    PrincipalType impl = Util.getObjectFactory().createPrincipalType();
 
    impl.setUserDN(ap.getName().to500String());
    impl.setCADN(ap.getGuarantor().to500String());
    return impl;
  }
}
