package lbl.akenti.messaging;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.ParseException;
import java.io.IOException;
import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.CheckAccessResponseType;
import lbl.akenti.jaxb.AkentiResponseType;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.util.Base64;

/**
  * Wraps a <i>CapabilityCertificate</i>. This certificate will  only
  * be present in response to a successful <i>CheckAccessRequest</i>.
  * 
  * @author Abdelilah Essiari
  */
final public 
class CheckAccessResponse extends AkentiResponse {

  private CapabilityCertificate cert;

  /**
    * Constructs this <i>CheckAccessResponse</i> with no 
    * <i> CapabilityCertificate</i>.
    */
  public CheckAccessResponse(int uid, AkentiMessage mesg) {
    this(uid, mesg, null);
  }

  /**
    * Constructs this <i>CheckAccessResponse</i>.
    *
    * @param cert a CapabilityCertificate
    * @see CapabilityCertificate
    */
  public CheckAccessResponse(int uid, AkentiMessage mesg, CapabilityCertificate cert) {
    super(uid, mesg);
    this.cert = cert;
  }

  /**
    * Returns <i>true</i> if a <i>CapabilityCertificate</i>
    * is present.
    *
    * @see getCapabilityCertificate
    */
  public boolean hasCapabilityCertificate() {
    return cert != null;
  }

  /**
    * Returns the <i>CapabilityCertificate</i>.
    *
    * @see hasCapabilityCertificate
    */
  public CapabilityCertificate getCapabilityCertificate() {
    return cert;
  }

  /**
    * JAXB STUFF
    * TODO: Efficiency
    */
  CheckAccessResponse(CheckAccessResponseType impl) 
    throws IOException, ParseException {
    super(impl);

    String base64= impl.getAkentiOpaqueCapability();

    if (base64 != null) {
      byte[] rep = Base64.decode(base64);
      ByteArrayInputStream bais = new ByteArrayInputStream(rep);
      cert = new CapabilityCertificate(new AkentiInputStream(bais));
    }
  }
  
  /**
    * JAXB STUFF.
    * TODO: Efficiency
    */
  protected AkentiResponseType doToJAXBImpl() throws JAXBException {
    try {
      CheckAccessResponseType impl = Util.getObjectFactory().createCheckAccessResponseGenerated();

      if (hasCapabilityCertificate()) {
        CapabilityCertificate cert = getCapabilityCertificate();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();

        cert.writeObject(new AkentiOutputStream(baos));

        String base64 = new String(Base64.encode(baos.toByteArray()));
        impl.setAkentiOpaqueCapability(base64);
      }

      return impl;
    } catch(IOException ex) {
      throw new JAXBException(ex.getMessage());
    }
  }
}
