package lbl.akenti.messaging;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;
import java.io.IOException;
import java.text.ParseException;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.CheckPolicyResponseType;
import lbl.akenti.jaxb.PolicyInfoType;
import lbl.akenti.jaxb.AkentiResponseType;

final public
class CheckPolicyResponse extends AkentiResponse {
  private List polInfosList;

  public CheckPolicyResponse(int uid, AkentiMessage mesg) {
    this(uid, mesg, new PolicyInfo[0]);
  }

  /**
    */
  public CheckPolicyResponse(int uid, AkentiMessage mesg, PolicyInfo pr) {
    this(uid, mesg, new PolicyInfo[] { pr });
  }

  /**
    */
  public CheckPolicyResponse(int uid, AkentiMessage mesg, PolicyInfo[] prs) {
    super(uid, mesg);
    polInfosList = new ArrayList(prs.length);

    for (int i = 0; i < prs.length; ++i) {
      polInfosList.add(prs[i]);
    } 

    polInfosList = Collections.unmodifiableList(polInfosList); 
  }

  /**
    */
  public PolicyInfo[] toPolicyInfosArray() {
    PolicyInfo[] ret = new PolicyInfo[polInfosList.size()];
	
    polInfosList.toArray(ret);
    return ret;
  }

  /**
    */
  public Iterator getPolicyInfos() {
    return polInfosList.listIterator();
  }
  
  /** 
    * FOR XML STUFF
    */
  CheckPolicyResponse(CheckPolicyResponseType impl) 
    throws IOException, ParseException {
    super(impl);

    List tempList = impl.getPolicyInfo();
    Iterator iter = tempList.listIterator();

    polInfosList = new ArrayList(tempList.size());

    while (iter.hasNext()) {
      PolicyInfoType prt = (PolicyInfoType) iter.next();

      polInfosList.add(new PolicyInfo(prt));
    }

    polInfosList = Collections.unmodifiableList(polInfosList); 
  }

  /** 
    * FOR XML STUFF
    */
  protected AkentiResponseType doToJAXBImpl() throws JAXBException {
    CheckPolicyResponseType impl = Util.getObjectFactory().createCheckPolicyResponseGenerated();
    Iterator iter = getPolicyInfos();
    List tempList = impl.getPolicyInfo();

    while (iter.hasNext()) {
      PolicyInfo ar = (PolicyInfo) iter.next();

      tempList.add(ar.toJAXBImpl());
    } 

    return impl;
  }
}
