package lbl.akenti.messaging;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.ParseException;
import java.io.IOException;
import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.OpaqueCertificateType;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.util.Base64;

/**
  * Convenients static methods to go from <i>AkentiCertificate</i>
  * to the JAXB opaque certificate implementation and vice-verca.
  *
  * @author Abdelilah Essiari 
  */
final 
class OpaqueCertificate {

  /**
    * Hides this constructor.
    */
  private OpaqueCertificate() {
  }

  /**
    * FOR XML STUFF.
    * TODO: Efficiency
    */
  static AkentiCertificate 
  fromJAXBImpl(OpaqueCertificateType impl) throws IOException, ParseException {
    byte[] rep = impl.getEncodedCertificate();
    ByteArrayInputStream bais = new ByteArrayInputStream(rep);

    if (impl.getType().equalsIgnoreCase("Policy")) {
      return new PolicyCertificate(new AkentiInputStream(bais));
    } else if (impl.getType().equalsIgnoreCase("UseCondition")) {
      return new UseCondCertificate(new AkentiInputStream(bais));
    } else if (impl.getType().equalsIgnoreCase("Attribute")) {
      return new AttributeCertificate(new AkentiInputStream(bais));
    } else if (impl.getType().equalsIgnoreCase("Capability")) {
      return new CapabilityCertificate(new AkentiInputStream(bais));
    } else {
      throw new RuntimeException();
    } 
  }
  
  /**
    * FOR XML STUFF.
    * TODO: Efficiency
    */
  static OpaqueCertificateType 
  toJAXBImpl(AkentiCertificate cert) throws JAXBException {
    try {
      OpaqueCertificateType impl = Util.getObjectFactory().createOpaqueCertificateType();
      ByteArrayOutputStream baos = new ByteArrayOutputStream();

      cert.writeObject(new AkentiOutputStream(baos));
      
      impl.setEncodedCertificate(baos.toByteArray());
      impl.setType(cert.getType());
      return impl;
    } catch(IOException ex) {
      throw new JAXBException(ex.getMessage());
    }
  }
}
