package lbl.akenti.messaging;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;
import java.io.IOException;
import java.text.ParseException;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.PolicyInfoType;
import lbl.akenti.jaxb.CertificateInfoType;

/**
  * Represents a per-resource immutable non-empty list of 
  * <i>CertificateInfo</i>s.
  *
  * @author Abdelilah Essiari
  */
final public
class PolicyInfo {
  private List certInfosList;
  private boolean policyCorrect;
  private String resource;

  /**
    * Constructs this <i>PolicyInfo</i> with a single
    * <i>CertificateInfo</i>.
    *
    * @see CertificateInfo
    */
  public PolicyInfo(CertificateInfo cr) {
    this(new CertificateInfo[] { cr });
  }

  /**
    * Constructs this <i>PolicyInfo</i> with array 
    * <i>CertificateInfo</i> whose size is at least 1.
    *
    * @see CertificateInfo
    */
  public PolicyInfo(CertificateInfo[] crs) {
    certInfosList = new ArrayList(crs.length);

    for (int i = 0; i < crs.length; ++i) {
      certInfosList.add(crs[i]);
    } 

    certInfosList = Collections.unmodifiableList(certInfosList);
  }

  /**
    */
  public CertificateInfo[] toCertificateInfosArray() {
    CertificateInfo[] ret = new CertificateInfo[certInfosList.size()];
  
    certInfosList.toArray(ret);
    return ret;
  }

  /**
    * Returns the <i>CertificateInfo</i>s as an iterator.
    */
  public Iterator getCertificateInfos() {
    return certInfosList.iterator();
  }
   
  /**
    * Returns the resource.
    */
  public String getResource() {
    return resource;
  }
    
  /**
    * Returns the <i>CertificateInfo</i>s as an iterator.
    */
  public boolean isPolicyCorrect() {
    return policyCorrect;
  }
  
  /** 
    * JAXB STUFF.
    */
  PolicyInfo(PolicyInfoType impl) 
    throws IOException, ParseException {
    List tempList = impl.getCertificateInfo();
    Iterator iter = tempList.iterator();

    certInfosList = new ArrayList(tempList.size());

    do {
      CertificateInfoType art = (CertificateInfoType) iter.next();

      certInfosList.add(new CertificateInfo(art));
    } while (iter.hasNext());

    certInfosList = Collections.unmodifiableList(certInfosList);
  }

  /** 
    * JAXB STUFF.
    */
  PolicyInfoType toJAXBImpl() throws JAXBException {
    PolicyInfoType impl = Util.getObjectFactory().createPolicyInfoType();
    Iterator iter = getCertificateInfos();
    List tempList = impl.getCertificateInfo();

    do {
      CertificateInfo ar = (CertificateInfo) iter.next();

      tempList.add(ar.toJAXBImpl());
    } while (iter.hasNext());

    return impl;
  }
}
