package lbl.akenti.messaging;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.ResourceBrowseResponseType;
import lbl.akenti.jaxb.AkentiResponseType;

/**
 * A response to a browse request type.
 * Holds a list of sub-resources.
 * @author Tommaso Sciortino
 */
final public
class ResourceBrowseResponse extends AkentiResponse {
  
  private List resources;
  
  /**
   * Constructs this <i>ResourceBrowseResponse</i>.
   *
   */
  public ResourceBrowseResponse(int uid, AkentiMessage mesg, String[] resourcesList) {
    super(uid, mesg);
    
    resources = new ArrayList(resourcesList.length);
    
    for (int i = 0; i < resourcesList.length; ++i) {
      resources.add(resourcesList[i]);
    }
    
    resources = Collections.unmodifiableList(resources);
  }
  
  /**
   * Returns an array of strings containing the subdirectories.
   *
   */
  public String[] getResources() {
    return (String[]) resources.toArray(new String[resources.size()]);
  }
  
  /**
   * JAXB STUFF
   * TODO: Efficiency
   */
  ResourceBrowseResponse(ResourceBrowseResponseType impl){
    super(impl);
    resources = Collections.unmodifiableList(impl.getResource());
  }
  
  /**
   * JAXB STUFF.
   * TODO: Efficiency
   */
  protected AkentiResponseType doToJAXBImpl() throws JAXBException {
    ResourceBrowseResponseType impl = Util.getObjectFactory().createResourceBrowseResponseGenerated();
    
    impl.getResource().addAll(resources);
    return impl;
  }
}
