package lbl.akenti.messaging;

import java.text.ParseException;
import java.io.IOException;
import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.UploadPolicyCertificateRequestType;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.AkentiCertificate;

/**
 * A request for uploading a certificate.
 * @author Tommaso Sciortino
 */
final public
class UploadPolicyCertificateRequest extends AkentiRequest {
  private PolicyCertificate cert;

  /**
   * Creates an <i>uploadCertificateRequest</i>.
   */
  public UploadPolicyCertificateRequest(int uid, PolicyCertificate cert) {
    super(uid);

    if (cert == null) {
      throw new IllegalArgumentException();
    }

    this.cert = cert;
  }

  /**
   * Returns the certificate we wish to upload.
   */
  public PolicyCertificate getCertificate() {
    return cert;
  }

  /**
    * USED FOR DOING XML STUFF 
    */
  UploadPolicyCertificateRequest(UploadPolicyCertificateRequestType impl)
    throws IOException, ParseException {
    super(impl);
    AkentiCertificate temp = OpaqueCertificate.fromJAXBImpl(impl.getOpaqueCertificate());

    if (temp instanceof PolicyCertificate == false) {
      throw new ParseException("Invalid cert type", -1);
    }

    cert = (PolicyCertificate) temp; 
  }
  
  /**
    * USED FOR DOING XML STUFF 
    */
  protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException {
    UploadPolicyCertificateRequestType impl = 
      type? Util.getObjectFactory().createUploadPolicyCertificateRequestType() 
          : Util.getObjectFactory().createUploadPolicyCertificateRequestGenerated();

    impl.setOpaqueCertificate(OpaqueCertificate.toJAXBImpl(getCertificate()));
    return impl;
  }
}
