package lbl.akenti.policyEngine;
 
import lbl.akenti.certs.Certificate;

/**
  * Wraps a <i>Certificate</i> and holds trust and cache
  * info used respectively by the <i>TrustVerifier</i>
  * and the <i>CertificateCache</i>.
  *
  * @see lbl.akenti.certs.Certificate
  * @see bl.akenti.policyEngine.TrustVerifier 
  * @see bl.akenti.policyEngine.CertificateCache 
  * @author Abdelilah Essiari
  */
public interface CertificateHolder {

  /**
    * Returns the certificate wrapped in this holder.
    */
  public Certificate getCertificate();

  /**
    * Returns if certificate is fresh. Fresh certificates
    * are stored externally at the end of a request.
    *
    * @see bl.akenti.policyEngine.CertificateCache 
    */
  public boolean isFresh();

  /**
    * Returns the level at wich this certificate was
    * was trusted.
    *
    * @return -1 if certificate is untrusted
    *          0 if certificate has not been verified.
    *          a positive number denoting the level 
    *          at which this certificate verified.
    *
    * @see bl.akenti.policyEngine.TrustVerifier 
    */
  public int getTrustLevel();

}
