package lbl.akenti.policyEngine;

import java.security.cert.X509Certificate;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.util.AkentiMessage;

/**
  * Holds information collected during CheckAccess/CheckPolicy requests.<BR>
  * 
  * @author Abdelilah Essiari
  */
public interface PolicyContext {

  /**
    * Returns the name of the resource.
    */
  String getResource();

  /**
    * Returns the resource alternative name.
    */
  String getResourceAltName();

  /**
    * Returns the subject.
    */
  AkentiPrincipal getSubject();

  /**
    * Returns the subject's <i>X509</i> certificate.
    */
  X509Certificate getSubjectCertificate();

  /**
    * Returns the subjects <i>Capability</i>.
    */
  CapabilityCertificate getCapabilityCertificate();

  /**
    * Returns the resource policies.
    *
    * @see ResourcePolicy
    */
  ResourcePolicy[] toResourcePolicies();

  /**
    * Returns <i>X509</i> certificates.
    *
    * @see java.security.certs.X509Certificate
    */
  X509Certificate[] toX509Certificates();
  
  /**
    * Returns the <i>Attribute</i> certificates.
    */
  AttributeCertificate[] toAttributeCertificates();

  /**
    * Returns the cache interval.
    */
  long getCacheInterval();

  /**
    * Returns certificates collected by the external
    * evaluator.
    *
    * @see ExternalEvaluator
    */
  Certificate[] toExternalCertificates();

  /**
    */
  String [] toDesiredActions();

  /**
    */
  AkentiMessage getAkentiMessage();

}
