package lbl.akenti.policyEngine.impl;
 
import lbl.akenti.certs.Certificate;

/**
  * Wraps a <i>Certificate</i> and holds trust and cache
  * info used respectively by the <i>TrustVerifier</i>
  * and the <i>CertificateCache</i>.
  *
  * @see lbl.akenti.certs.Certificate
  * @see bl.akenti.policyEngine.TrustVerifier 
  * @see bl.akenti.policyEngine.CertificateCache 
  * @author Abdelilah Essiari
  */
class AkentiCertificateHolder 
  implements lbl.akenti.policyEngine.CertificateHolder {
  private Certificate cert;
  private boolean fresh;
  private int trustLevel;
 
  /**
    */
  AkentiCertificateHolder(Certificate cert) {
    this(cert, 0);
  }

  AkentiCertificateHolder(Certificate cert, int trustLevel) {
    this(cert, trustLevel, true);
  }

  AkentiCertificateHolder(Certificate cert, int trustLevel, boolean fresh) {
    this.cert = cert;
    this.trustLevel = trustLevel;
    this.fresh = fresh;
  }

  /**
    * Returns the certificate wrapped in this holder.
    */
  public final Certificate getCertificate() {
    return cert;
  }

  /**
    */
  final void setFresh(boolean fresh) {
    this.fresh = fresh;
  }

  /**
    * Returns if certificate is fresh. Fresh certificates
    * are stored externally at the end of a request.
    *
    * @see bl.akenti.policyEngine.CertificateCache 
    */
  public final boolean isFresh() {
    return fresh;
  }

  /**
    * Returns the level at wich this certificate was
    * was trusted.
    *
    * @return -1 if certificate is untrusted
    *          0 if certificate has not been verified.
    *          a positive number denoting the level 
    *          at which this certificate verified.
    *
    * @see lbl.akenti.policyEngine.TrustVerifier 
    */
  public final int getTrustLevel() {
    return trustLevel;
  }

  /**
    */
  final void setTrustLevel(int trustLevel) {
    this.trustLevel = trustLevel;
  }

  /**
    */
  public boolean equals(Object o) {
    if (cert == o || cert.equals(o)) {
      return true;
    }
 
    return false;
  }
}
