package lbl.akenti.policyEngine.impl;

import java.util.Set;
import lbl.akenti.policyEngine.Runtime;
import lbl.akenti.policyEngine.PolicyContext;
import lbl.akenti.policyEngine.PolicyCollector;
import lbl.akenti.policyEngine.UseCondCollector;
import lbl.akenti.policyEngine.Authorizer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.util.AkentiMessage;

public class AkentiEngine {
  /**
    */ 
  private Runtime runtime; 

  /**
    */
  public AkentiEngine() {
    runtime = new AkentiRuntime(); 
  }

  public void setDesiredActions(Set desiredActions) {
    AkentiPolicyContext ctx = (AkentiPolicyContext) getPolicyContext();

    ctx.setDesiredActions(desiredActions);
  }

  public boolean checkAccess(String resource, AkentiPrincipal ap) {
    AkentiPolicyContext ctx = (AkentiPolicyContext) getPolicyContext();

    ctx.setResource(resource);
    ctx.setSubject(ap);

    PolicyCollector policyCollector = runtime.getPolicyCollector();

    if (!policyCollector.collect()) {
      return false;
    } 

    AkentiTrustVerifier trustVerifier = (AkentiTrustVerifier) runtime.getTrustVerifier();

    trustVerifier.loadTrustMaterial();

    if (!policyCollector.verify()) {
      return false;
    }

    UseCondCollector useCondCollector = runtime.getUseCondCollector();

    if (!useCondCollector.collectAndVerify()) {
      return false;
    }

    Authorizer authorizer = runtime.getAuthorizer();

    if (!authorizer.authorize()) {
      return false;
    }

    return true;
  }

  /**
    */
  public PolicyContext getPolicyContext() {
    return runtime.getPolicyContext();
  }

  /**
    */
  public AkentiMessage getAkentiMessage() {
    return getPolicyContext().getAkentiMessage();
  }

  public static boolean init(String fileName) {
    java.util.Properties props = System.getProperties();
    props.setProperty("TRANSPGRID", "/home/portnoy/u1/mrt/globusAkenti/");
    props.setProperty("TRANSPGRIDD", "home/g1/users/mrt/globusAkent");
    props.setProperty("R1", "/home/portnoy/u1/aes/akenti/build/linux2/testcase/resourceTree");
    props.setProperty("R2", "/home/portnoy/u1/aes/akenti/build/linux2/testcase/resourceTree");
    props.setProperty("R3", "/home/portnoy/u1/aes/akenti/build/linux2/testcase/resourceTree");
    return true;
  }

  public static String getInitializationMessage() {
    return null;
  }
}
