package lbl.akenti.policyEngine.impl;
 
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;
import java.net.URL;
import java.net.MalformedURLException;
import lbl.akenti.policyEngine.Runtime;
import lbl.akenti.policyEngine.CertificateCache;
import lbl.akenti.policyEngine.CertificateHolder;
import lbl.akenti.policyEngine.TrustVerifier;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.dirServices.CertificateRetrievalFactory;
import lbl.akenti.parsers.Result;
import lbl.akenti.util.Util;
import lbl.akenti.util.UTCTime;

/**
  * @author Abdelilah Essiari
  **/
final
class AkentiInternalEvaluator extends DefaultInternalEvaluator {
  private List newInfos = new ArrayList();
  private int index = 0;

  protected AkentiInternalEvaluator(Runtime runtime) {
    super(runtime);
  }
  
  public void setAttributeInfos(AttributeInfo[] infos) {
    index = 0;
    super.setAttributeInfos(infos);
  }

  public AttributeInfo[] getAttributeInfos() {
    return (AttributeInfo[]) newInfos.toArray(new AttributeInfo[0]);
  }

  public Result reduce(int op, String attribute, String value) {
    AttributeInfo info = infos[index++];

    switch(info.getType()) {
      case AttributeInfo.X509:
        AkentiPrincipal ap = runtime.getPolicyContext().getSubject();
        newInfos.add(info);
        return Result.NULL;
      case AttributeInfo.AKENTI:
        ap = runtime.getPolicyContext().getSubject();
        StringBuffer sb = new StringBuffer();
 
        sb.append(ap.getName().to500String());
        sb.append(ap.getGuarantor().to500String()); 
        sb.append(attribute); 
        sb.append(value); 

        CertificateHolder[] holders = getCertificateHolders(sb.toString(), info);
        TrustVerifier trustVerifier = runtime.getTrustVerifier();
        UTCTime now = new UTCTime();

        for (int i = 0; i < holders.length; ++i) {
          Certificate cert = holders[i].getCertificate();

          if (!cert.checkValidity(now)) {
          } else if (!info.hasIssuer(cert.getIssuer().getIssuer())) {
          } else if (!trustVerifier.verify(holders[i])) {
          } else {
            return Result.TRUE;
          }
        }

        return Result.FALSE;
      case AttributeInfo.SYSTEM:
        newInfos.add(info);
        return Result.NULL;
      case AttributeInfo.EXT_AUTH:
        newInfos.add(info);
        return Result.NULL;
      default:
        throw new RuntimeException();
    }
  }

  CertificateHolder[]
  getCertificateHolders(String key, AttributeInfo info) {
    CertificateCache certCache = runtime.getCertificateCache();
    CertificateHolder[] holders = certCache.getCertificateHolders(key);
 
    System.out.println("CHECKING CACHE");

    if (holders.length != 0) {
      System.out.println("CHECKING CACHE OK");
      return holders;
    }

    System.out.println("CHECKING CACHE FAILED");

    Map args = new HashMap();
 
    args.put("type", "Attribute");
    args.put("hash", Util.createHashName(key));

    Directory[] dirs = info.getDirectories(); 
   
    if (dirs.length == 0) {
      dirs = runtime.getAttributeDirectories();
    }

    if (dirs.length == 0) {
      return new CertificateHolder[0];
    }

    List holderList = new ArrayList(1);
    int i = 0;

    do {
      Certificate[] certs = retrieveCertificates(dirs[i], args);
      int j = 0;

      if (certs.length == 0) {
      } else {
        do {
          if (certs[j].getHashString().equals(key)) {
            holderList.add(certCache.addCertificate(key, certs[j]));
          }
        } while (++j < certs.length);
      }
    } while (holderList.size() == 0 && ++i < dirs.length);

    return (CertificateHolder[]) holderList.toArray(new CertificateHolder[0]);
  }

  /**
    */
  private Certificate[] retrieveCertificates(Directory dir, Map args) {
    try {
      URL url = dir.parseDirectory();
 
      return CertificateRetrievalFactory.retrieveAkentiCertificates(url, args);
    } catch(MalformedURLException ex) {
      return new Certificate[0];
    }
  }
}
