package lbl.akenti.policyEngine.impl;

import java.util.Iterator;
import java.util.Collections;
import java.util.List;
import java.util.ArrayList;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.Directory;

/**
  * Groups the <i>PolicyCertificate</i> and the <i>UseConditionCertificate</i>s
  * that apply to the same resource.<BR>
  *
  * @author Abdelilah Essiari.
  * @see lbl.akenti.PolicyCertificate
  * @see lbl.akenti.UseCondCertificate
  */

final
class AkentiResourcePolicy 
  implements lbl.akenti.policyEngine.ResourcePolicy {

  /**
    * Policy certificate.
    */
  private PolicyCertificate policyCert;

  /**
    * UseCondition certificates.
    */
  private List ucCerts = new ArrayList();

  /**
    */
  AkentiResourcePolicy(PolicyCertificate policyCert) {
    this.policyCert = policyCert;
  }

  /**
    */
  public String getResource() {
    return getPolicyCertificate().getResource();
  }

  /**
    */
  public PolicyCertificate getPolicyCertificate()  {
    return policyCert;
  }

  /**
    * Adds a <i>UseCondCertificate</i> to this <i>ResourcePolicy</i>
    */
  void addUseCondCertificate(UseCondCertificate ucCert) {
    ucCerts.add(ucCert);
  }

  public UseCondCertificate[] toUseCondCertificates() {
    return (UseCondCertificate[]) ucCerts.toArray(new UseCondCertificate[0]);
  }

  /**
    */
  Iterator getUseCondCertificates()  {
    return ucCerts.iterator();
  }

  /**
    * Returns critical UCs.
    */
  List getCriticalUseCondCertificates()  {
    Iterator iter = getUseCondCertificates();
    List ret = new ArrayList();

    while (iter.hasNext()) {
      UseCondCertificate tempCert = (UseCondCertificate) iter.next();

      if (tempCert.isCritical()) {
        ret.add(tempCert);
      }
    }
    
    return ret;
  }

  /**
    * Returns non critical UCs.
    */
  List getNonCriticalUseCondCertificates()  {
    Iterator iter = getUseCondCertificates();
    List ret = new ArrayList();

    while (iter.hasNext()) {
      UseCondCertificate tempCert = (UseCondCertificate) iter.next();

      if (!tempCert.isCritical()) {
        ret.add(tempCert);
      }
    }

    return ret;
  }
}
