package lbl.akenti.policyEngine.impl;

import lbl.akenti.policyEngine.Runtime;
import lbl.akenti.policyEngine.PolicyContext;
import lbl.akenti.policyEngine.PolicyCollector;
import lbl.akenti.policyEngine.UseCondCollector;
import lbl.akenti.policyEngine.InternalEvaluator;
import lbl.akenti.policyEngine.ExternalEvaluator;
import lbl.akenti.policyEngine.CertificateCache;
import lbl.akenti.policyEngine.TrustVerifier;
import lbl.akenti.policyEngine.Authorizer;
import lbl.akenti.certs.elements.Directory;

public class AkentiRuntime implements lbl.akenti.policyEngine.Runtime {
  private PolicyContext policyContext;
  private TrustVerifier trustVerifier;
  private PolicyCollector policyCollector;
  private CertificateCache certCache;
  private UseCondCollector useCondCollector;
  private Authorizer authorizer;
  private InternalEvaluator internalEvaluator;
  private int policyLevel;

  /**
    */
  public AkentiRuntime() {
  }

  /**
    */
  public PolicyContext getPolicyContext() {
    if (policyContext == null) {
      policyContext = new AkentiPolicyContext();
    }

    return policyContext;
  }

  /**
    */
  public TrustVerifier getTrustVerifier() {
    if (trustVerifier == null) {
      trustVerifier = new AkentiTrustVerifier(this);
    }

    return trustVerifier;
  }

  /**
    */
  public PolicyCollector getPolicyCollector() {
    if (policyCollector == null) {
      policyCollector = new AkentiPolicyCollector(this);
    }
    
    return policyCollector;
  }

  /**
    */
  public UseCondCollector getUseCondCollector() {
    if (useCondCollector == null) {
      useCondCollector = new AkentiUseCondCollector(this);
    }

    return useCondCollector;
  }

  /**
    */
  public ExternalEvaluator getExternalEvaluator() {
    return null;
  }

  /**
    */
  public InternalEvaluator getInternalEvaluator() {
    if (internalEvaluator == null) {
      internalEvaluator = new AkentiInternalEvaluator(this);
    }

    return internalEvaluator;
  }

  /**
    */
  public Authorizer getAuthorizer() {
    if (authorizer == null) {
      authorizer = new AkentiAuthorizer(this);
    }

    return authorizer; 
  }

  /**
    */
  public CertificateCache getCertificateCache() {
    if (certCache == null) {
      certCache = new AkentiCertificateCache(this);
    }

    return certCache;
  }

  /**
    */
  public void setPolicyLevel(int policyLevel) {
    this.policyLevel = policyLevel;
  }
  
  /**
    */
  public  int getPolicyLevel() {
    return policyLevel;
  }

  public Directory[] getAttributeDirectories() {
    AkentiPolicyContext ctx = (AkentiPolicyContext) getPolicyContext();
    int i = getPolicyLevel() - 1;
    Directory []dirs;
    
    do {
      lbl.akenti.policyEngine.ResourcePolicy rp = ctx.getResourcePolicy(i);
      lbl.akenti.certs.PolicyCertificate cert = rp.getPolicyCertificate();

      dirs = cert.getAttributeDirectories(); 
    } while (dirs.length == 0 && --i >= 0); 
    
    return dirs;
  }
}
