package lbl.akenti.policyEngine.impl;

import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;
import lbl.akenti.policyEngine.CertificateHolder;
import lbl.akenti.policyEngine.CertificateCache;
import lbl.akenti.policyEngine.ResourcePolicy;
import lbl.akenti.policyEngine.Runtime;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;

/**
  * TrustVerifier is responsible for deciding whether certificates/subject
  * should be trusted or not. 
  *
  * @author Abdelilah Essiari
  **/
final class 
AkentiTrustVerifier 
  implements lbl.akenti.policyEngine.TrustVerifier {
  private HashMap listedCAS = new HashMap();
  private HashMap listedDirs = new HashMap();
  private Runtime runtime;

  /**
    */
  AkentiTrustVerifier(Runtime runtime) {
    this.runtime = runtime;
  }

  /**
    */
  void loadTrustMaterial() {
    AkentiPolicyContext ctx = (AkentiPolicyContext) runtime.getPolicyContext(); 
    Iterator iter = ctx.getResourcePolicies();
    int caLevel = 1;

    while (iter.hasNext()) {
      ResourcePolicy resourcePolicy = (ResourcePolicy) iter.next();
      PolicyCertificate policyCert = resourcePolicy.getPolicyCertificate();
      CertificateAuthorityInfo[] infos = policyCert.getCertificateAuthorityInfos();
      HashMap cas = new HashMap();

      for (int i = 0; i < infos.length; ++i) {
        X509Certificate[] chain = infos[i].getChain();

        cas.put(infos[i].getDistinguishedName(), 
                new CACertHolderHolder(new AkentiX509Certificate(chain[0]), 
                                 caLevel == 1? 1 : 0,    // ONLY ROOT ARE TRUSTED 
                                 caLevel, 
                                 infos[i].getIdentityDirectories()));  

        HashMap previous = (HashMap) listedCAS.get(new Integer(caLevel - 1));

        for (int j = 1; caLevel > 1 && j < chain.length; ++j) {
          AkentiX509Certificate cert = new AkentiX509Certificate(chain[j]);
          DistinguishedName dn = cert.getSubjectDN();

          if (previous.get(dn) == null) {  // DO NOT OVERWRITE 
            previous.put(dn, new CACertHolderHolder(cert, 0, caLevel - 1, new Directory[0]));
          }
        }
      }

      listedCAS.put(new Integer(caLevel++), cas);
    }
  }

  public CertificateHolder verifySubject(AkentiPrincipal ap) {
    AkentiCertificateHolder h = findX509Certificate(ap.getName(),
                                                    ap.getGuarantor(),
                                                    new Directory[0],
                                                    runtime.getPolicyLevel()); 

    if (h == null) {
      return null;
    } 

    verify(h);
    return h;
  }

  public CertificateHolder verify(Certificate cert) {
    AkentiCertificateHolder h = new AkentiCertificateHolder(cert);
    
    verify(h);
    return h;
  }

  public boolean verify(CertificateHolder h) {
    Certificate cert = h.getCertificate();
    AkentiIssuer issuer = cert.getIssuer();
    int startLevel = getCALevelStartingAt(runtime.getPolicyLevel());
    int caLevel = startLevel;

    ((AkentiCertificateHolder) h).setTrustLevel(runtime.getPolicyLevel());
    return true;
/*
    AkentiCertificateHolder holder = findX509Certificate(issuer.getName(),
                                            issuer.getGuarantor(),
                                            issuer.getIdentityDirs(),
                                            caLevel); 

    if (holder == null) {
      return false;
    } else if (holder.getTrustLevel() < 0) {
      return false;
    } else if (holder.getTrustLevel() >= caLevel) {
      return true;
    }

    ArrayList chain = new ArrayList(10);
    int depth = 10;
    
    do {
      chain.add(holder);
      // issuer = holder.cert.getAkentiIssuer();
      issuer = null;
      
      if (holder instanceof CACertHolderHolder) {
        caLevel = ((CACertHolderHolder) holder).caLevel;
      }

      holder = findX509Certificate(issuer.getName(),
                                   issuer.getGuarantor(),
                                   issuer.getIdentityDirs(),
                                   caLevel); 
        
    } while ((holder != null)
          && (holder.getTrustLevel() > 0)
          && (holder.getTrustLevel() < caLevel)
          && (--depth > 0));

    if (depth == 0) {
      return false;
    } else if (holder == null) {
      Iterator iter = chain.iterator();
      
      while (iter.hasNext()) {
        holder = (AkentiCertificateHolder) iter.next();
        holder.setTrustLevel(-1);
      }
      
      return false;
    }

    Iterator iter = chain.iterator();
    
    caLevel = startLevel;

    while (iter.hasNext()) {
      holder = (AkentiCertificateHolder) iter.next();
      
      if (holder instanceof CACertHolderHolder) {
        caLevel = ((CACertHolderHolder) holder).caLevel;
      }

      holder.setTrustLevel(caLevel);
    }

    return true;
*/
  }

  private AkentiCertificateHolder 
  findX509Certificate(DistinguishedName dn,
                      DistinguishedName caDN,
                      Directory[] idDirs,
                      int startLevel) {
    if (caDN == null) {
      // CHECKING FOR A CA'S CERTIFICATE
      CACertHolderHolder holder = (CACertHolderHolder) getCAStartingAtLevel(dn, startLevel);
       
      if (holder != null) {
        return holder;
      } 
    }

    if (idDirs.length == 0) {
      idDirs = getIDDirsStartingAtLevel(startLevel);

      if (idDirs.length == 0) {
        // NO IDDIRS  POLICY NEED TO BE FIXED
        return null;
      }
    }

    CertificateCache certCache = runtime.getCertificateCache();

    if (false) {
      // CONSTRUCT THEY KEY
      // CHECK THE CACHE 
      return null;
    }

    // USE DIRS TO FIND CA
    // ADD TO CACHE
    return null;
  }

  private Directory[] 
  getIDDirsStartingAtLevel(int startLevel) {
    int caLevel = getCALevelStartingAt(startLevel);
    Integer key = new Integer(caLevel);
    Directory[] idDirs = (Directory []) listedDirs.get(key);
    
    if (idDirs == null) {
      HashMap cas = (HashMap) listedCAS.get(new Integer(caLevel));
      Iterator iter = cas.values().iterator();
      ArrayList temp = new ArrayList();

      while (iter.hasNext()) {
        CACertHolderHolder holder = (CACertHolderHolder) iter.next();
 
        for (int i = 0; i < holder.idDirs.length; ++i) {
          temp.add(holder.idDirs[i]);
        }
      }

      idDirs = (Directory[]) temp.toArray(new Directory[0]);
      listedDirs.put(key, idDirs);
    }

    return idDirs;
  }

  private CACertHolderHolder 
  getCAStartingAtLevel(DistinguishedName dn, int startLevel) {
    int caLevel = getCALevelStartingAt(startLevel);
    HashMap cas = (HashMap) listedCAS.get(new Integer(caLevel));

    return (CACertHolderHolder) cas.get(dn);
  }

  private int getCALevelStartingAt(int startLevel) {
    HashMap cas = (HashMap) listedCAS.get(new Integer(startLevel));

    while (cas.size() == 0) {
      cas = (HashMap) listedCAS.get(new Integer(--startLevel));
    }

    return startLevel;
  }

  class CACertHolderHolder extends AkentiCertificateHolder {
    int caLevel;
    Directory[] idDirs;

    CACertHolderHolder(Certificate cert, 
                 int trustLevel, 
                 int caLevel, 
                 Directory[] idDirs) {
      super(cert, trustLevel);
      this.caLevel = caLevel;
      this.idDirs = idDirs;
    }
  }
}
