package lbl.akenti.policyEngine.impl;
 
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;
import java.net.URL;
import java.net.MalformedURLException;
import lbl.akenti.policyEngine.Runtime;
import lbl.akenti.policyEngine.CertificateHolder;
import lbl.akenti.policyEngine.TrustVerifier;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.dirServices.CertificateRetrievalFactory;
import lbl.akenti.util.UTCTime;
import lbl.akenti.util.Util;

/**
  * @author Abdelilah Essiari
  **/
class AkentiUseCondCollector 
  implements lbl.akenti.policyEngine.UseCondCollector {
  private Runtime runtime;

  AkentiUseCondCollector(Runtime runtime) {
    this.runtime = runtime;
  }

  /**
    * Collects and verifies UseCondition certificates.
    */
  public boolean collectAndVerify() {
    TrustVerifier trustVerifier = runtime.getTrustVerifier();
    AkentiPolicyContext ctx = (AkentiPolicyContext) runtime.getPolicyContext();
    Iterator resourcePolicies = ctx.getResourcePolicies();
    int policyLevel = 1;
 
    do { 
      runtime.setPolicyLevel(policyLevel++);

      AkentiResourcePolicy resourcePolicy = (AkentiResourcePolicy) resourcePolicies.next();
      String resource = resourcePolicy.getResource();
      List holders = collectUseCondCertificates(resourcePolicy);

      if (holders.size() == 0) {
        ctx.setMessage(501, "failed to find uc certs for " + resource);
        return false;
      }

      Iterator iter = holders.iterator();

      do {
        CertificateHolder holder = (CertificateHolder) iter.next();
        UseCondCertificate ucCert = (UseCondCertificate) holder.getCertificate();

        if (!ucCert.isScopeLocal() || !resourcePolicies.hasNext()) {
          if (trustVerifier.verify(holder)) {
            resourcePolicy.addUseCondCertificate(ucCert);
          }
        }
      } while (iter.hasNext());
    } while (resourcePolicies.hasNext());
 
    ctx.setMessage(103, "collecting and verifying UseCondCertficate ok");
    return true;
  }

  /**
    * Collect UCs per resource.
    */
  List collectUseCondCertificates(AkentiResourcePolicy resourcePolicy) {
    PolicyCertificate policyCert = resourcePolicy.getPolicyCertificate();
    String resource = policyCert.getResource();
    AkentiCertificateCache certCache = (AkentiCertificateCache) runtime.getCertificateCache();

    // CHECK CACHE
    Map args = new HashMap();
    
    args.put("type", "UseCondition");
    args.put("hash", Util.createHashName(resource));
    
    UTCTime now = new UTCTime();
    TrustVerifier trustVerifier = runtime.getTrustVerifier();
    UseCondIssuerGroup[] groups = policyCert.getUseCondIssuerGroups();
    List holders = new ArrayList();
    List rest = new ArrayList();
    int policyLevel = runtime.getPolicyLevel();
    boolean foundValidUC;
    int i = 0;

    do {
      Directory[] dirs = groups[i].getDirectories();
      
      foundValidUC = false;

      for (int j = 0; !foundValidUC && j < dirs.length; ++j) {
        System.out.println("DIRECTORY = " + dirs[j]);

        Certificate[] certs = retrieveCertificates(dirs[j], args);
        int k;

        for (k = 0; !foundValidUC && k < certs.length; ++k) {
          AkentiIssuer issuer = certs[k].getIssuer();
 
          if (!certs[k].getHashString().equals(resource)) {
          } else if (!certs[k].checkValidity(now)) {
          } else if (!groups[i].containsUCIssuer(issuer.getIssuer())) {
          } else {
            CertificateHolder holder = trustVerifier.verify(certs[k]);

            if (holder.getTrustLevel() != policyLevel) {
            } else {
              foundValidUC = true;
              holders.add(holder);
            }
          }
        }

        for (int l = k; l < certs.length; ++l) {
          AkentiIssuer issuer = certs[l].getIssuer();
 
          if (!certs[l].getHashString().equals(resource)) {
          } else if (!certs[l].checkValidity(now)) {
          } else if (!groups[i].containsUCIssuer(issuer.getIssuer())) {
          } else {
            rest.add(certs[l]);
          }
        }
      }
    } while (foundValidUC && ++i < groups.length);

    if (foundValidUC) {
      Iterator iter = holders.iterator();
     
      do {
        certCache.addCertificateHolder(resource, (CertificateHolder) iter.next());
      } while (iter.hasNext());
 
      iter = rest.iterator();

      while (iter.hasNext()) {
        CertificateHolder holder = certCache.addCertificate(resource, (Certificate) iter.next());

        holders.add(holder);
      }
    }
    
    return holders;
  }

  private Certificate[] retrieveCertificates(Directory dir, Map args) {
    try {
      URL url = dir.parseDirectory();

      return CertificateRetrievalFactory.retrieveAkentiCertificates(url, args); 
    } catch(MalformedURLException ex) {
      return new Certificate[0];
    }
  }
}
