package lbl.akenti.policyEngine.impl;

import java.security.GeneralSecurityException;
import java.security.SignatureException;
import java.security.Principal;
import java.security.cert.X509Certificate;
import java.security.PublicKey;
import java.text.ParseException;
import java.io.IOException;
import java.util.Date;
import org.w3c.dom.Element;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.Certificate;
import lbl.akenti.util.UTCTime;

/**
  * Adapts X509 certificates to an AkentiCertificate.
  */
class AkentiX509Certificate extends Certificate {

  private X509Certificate cert;


  AkentiX509Certificate() {
    super("X509");
  }

  AkentiX509Certificate(X509Certificate cert) {
    super("X509");
    this.cert = cert;
  }

  /**
    **/
  public String getHashString() {
    throw new UnsupportedOperationException();
  }

  /**
    * Returns the version number.
    */
  public int getVersion() {
    return cert.getVersion();
  }

  /**
    * Returns this <i>AkentiCertificate</i>'s unique id.
    */
  public String getUniqueID() {
    throw new UnsupportedOperationException();
  }

  /**
    */
  public DistinguishedName getSubjectDN() {
    throw new UnsupportedOperationException();
  }

  /**
    * Returns this <i>AkentiCertificate</i>'s issuer.
    * Principal getIssuerDN();
    */
  public AkentiIssuer getIssuer() {
    throw new UnsupportedOperationException();
  }

  /**
    * Returns the start validity date.
    */
  public UTCTime getNotBefore() {
    throw new UnsupportedOperationException();
  }

  /**
    * Returns the end validity date.
    */
  public UTCTime getNotAfter() {
    throw new UnsupportedOperationException();
  }

  /**
    * Returns the name of the signature algorithm.
    */
  public String getSignatureAlg() {
    return cert.getSigAlgName();
  }

  /**
    * Returns the name of the canonalization algorithm.
    */
  public String getCanonAlg() {
    return "ASN.1";
  }

  /**
    * Returns the signature.
    */
  public byte[] getSignature() {
    throw new UnsupportedOperationException();
  }

  /**
    * Returns the signable part of this certificate.
    *
    * @see writeObject(AkentiOutputStream)
    * @see doWriteObject(AkentiOutputStream)
    */
  public byte[] getSignable() throws IOException {
    throw new UnsupportedOperationException();
  }

  /**
    */
  public boolean verify(PublicKey publicKey) 
    throws GeneralSecurityException, IOException {
    try {
      cert.verify(publicKey);
      return true;
    } catch(SignatureException ex) {
      return false;
    }
  }

  /**
    */
  protected Certificate newInstance()  {
    return new AkentiX509Certificate(); 
  }

  /**
    */
  public void writeObject(AkentiOutputStream drain) 
    throws IOException {
    throw new UnsupportedOperationException();
  }

  protected void readObject(AkentiInputStream source) 
    throws IOException , ParseException {
    throw new UnsupportedOperationException();
  }

  protected void fromXML(Element cert) throws ParseException {
    throw new UnsupportedOperationException();
  }
 
  public void toXML(Element cert) {
    throw new UnsupportedOperationException();
  }
}
