package lbl.akenti.policyManager.base;

import lbl.akenti.util.P12Reader;

import java.security.Principal;
import java.security.PrivateKey;
import java.io.IOException;
import java.security.GeneralSecurityException;

/**
  * Stores an entity's credential info.
  */
final public 
class CredentialInfo {

  private static CredentialInfo credentialInfo; 

  private P12Reader p12Reader = new P12Reader();

  private boolean isInit = false;

  private String fileName = "";

  private CredentialInfo() {
  }

  /**
    */
  public static CredentialInfo getInstance() {
    if (credentialInfo == null) {
      synchronized(CredentialInfo.class) {
        if (credentialInfo == null) {
          credentialInfo = new CredentialInfo();
        }
      }
    }

    return credentialInfo;
  }

  /**
    * Reads the pkcs12 file. 
    */
  public void readP12File(String fileName, char[] password)
    throws IOException, GeneralSecurityException { 
    p12Reader.readP12File(fileName, password);
    isInit = true;
    this.fileName = fileName;
  }

  /**
    * Returns whether or not credential info has been initialized.
    */
  public boolean isInitCredential() {
    return isInit;
  }

  /**
    * Gets the subject's distinguished name.
    *
    * @return subject's distinguished name. 
    */
  public Principal getSubjectDN() {
    return p12Reader.getSubjectDN(); 
  }

  /** 
    * Returns the issuer's distinguished name.
    */
  public Principal getIssuerDN() {
    return p12Reader.getIssuerDN();
  }

  /**
    * Return the pkcs12 private key.
    */
  public PrivateKey getPrivateKey() {
    return p12Reader.getPrivateKey();
  }

  /**
    * Returns the pkcs12 file name.
    */ 
  public String getFileName() {
    return fileName; 
  }

}
