package lbl.akenti.policyManager.base;

import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.util.UTCTime;

/** 
  * Writes akenti certificates in Html.
  */
public class HtmlCertWriter {

    String headerColor;

    public HtmlCertWriter() {
      this("blue");
    }

    /** Constructor: the color of headers is set to 'color' */
    public HtmlCertWriter(String color) {
	if (color == null) {
	    throw new IllegalArgumentException();
        }
	
	headerColor = color;
    }

    /** setHeaderColor: sets color used in headers to 'color' */
    public void setHeaderColor(String color) {
	if (color == null) {
	    throw new IllegalArgumentException();
        }

	headerColor = color;
    }    

    /** beginHtml: returns the <html> tag */
    public String beginHtml() {
	return "<html>";
    }

    /** endHtml: return the </html> tag */
    public String endHtml() {
	return "</html>";
    }
    
    /** formatHeader: wraps 'text' into a header format */
    public String formatHeader(Object text) {
	StringBuffer sb = new StringBuffer();

	sb.append("<font color=" + headerColor + ">");
	sb.append("<i><b>");
	sb.append(text);
	sb.append("</b></i>");
	sb.append("</font>");
	return sb.toString();
    }

    /** formatHeader: wraps 'text' into a header format */
    // Added 07/19/01
    public String formatSubHeader(Object text) {
	StringBuffer sb = new StringBuffer();

	sb.append("<font color=" + headerColor + ">");
	sb.append("<font size=-2>");
	sb.append("&nbsp;&nbsp;&nbsp;&nbsp;");
	sb.append("<i><b>");
	sb.append(text);
	sb.append("</b></i>");
	sb.append("</font>");
	sb.append("</font>");
	return sb.toString();
    }
    
    /** formatValues: wraps a group of values in smaller print 
     *  Puts one value per line
     */
    public String formatValues(Object[] values) {
	StringBuffer sb = new StringBuffer();

	sb.append("<p>");
	sb.append("<font size=-1>");

	for(int i = 0; i< values.length; i++) {
	    sb.append("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"); 
	    sb.append(values[i]);
	    sb.append("<br>");
	}

	sb.append("</font>");
	return sb.toString();
    }
    
    /** formatValues: wraps a single value in smaller print */
    public String formatValue(Object value) {
	StringBuffer sb = new StringBuffer();

	sb.append("<p>");
	sb.append("<font size=-1>");
	sb.append("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"); 
	sb.append(value);
	sb.append("<br>");
	sb.append("</font>");
	return sb.toString();
    } 
    
    /** formatString: formats a piece of text with the given color and size */
    public String 
    formatString(Object text, String color, String size, boolean bold) {
	StringBuffer sb = new StringBuffer();

	sb.append("<font size=");
	sb.append(size);
	sb.append(" color=");
	sb.append(color);
	sb.append(">");

	if (bold) {
	    sb.append("<b>");
        }

	sb.append(text);

	if (bold) {
	    sb.append("</b>");
        }

	sb.append("</font>");
	return sb.toString();
    }

    /** stringSized: return a string of size equal to 'size' */
    public String 
    stringSized(Object text, String size) {
	StringBuffer sb = new StringBuffer();

	sb.append("<font size=");
	sb.append(size);
	sb.append(">");
	sb.append(text);
	sb.append("</font>");
	return sb.toString();
    }
    
    /** newLine: return a single <br> tag */
    public String newLine() {
	return "<br>";
    }

    /** newLines: return a number of <br> tabs equal to numLines */
    public String newLines(int numLines) {
	StringBuffer sb = new StringBuffer();
	
	for (int i = 0; i < numLines; i++) {
	    sb.append("<br>");
        }

	return sb.toString();
    }

    /** spaces: return a number of spaces equal to numSpaces */
    public String spaces(int numSpaces) {
	StringBuffer sb = new StringBuffer();
	
	for (int i = 0; i < numSpaces; i++) {
	    sb.append("&nbsp;");
        }

	return sb.toString();
    }
    
    private StringBuffer doToHtml(AkentiCertificate cert) {
      StringBuffer sb = new StringBuffer();
    
      sb.append(beginHtml());
      
      AkentiIssuer issuer = cert.getIssuer();
      
      if (issuer != null) {
        Object dn = issuer.getName();
        Object caDn = issuer.getGuarantor();
     
        sb.append(formatHeader("Issuer"));
        sb.append(formatValues(new Object[] { dn, caDn }));
      }
      
      sb.append(formatHeader("Validity period"));

      UTCTime notBefore = cert.getNotBefore();
      UTCTime notAfter = cert.getNotAfter();

      sb.append(
        formatValues(
          new String[] { 
            notBefore.toFriendlyString(), 
            notAfter.toFriendlyString() 
          }
        )
      );

      return sb;
    }

    public String toHtml(AkentiCertificate cert) {
      if (cert instanceof PolicyCertificate) {
        return toHtml((PolicyCertificate) cert);
      } else if (cert instanceof AttributeCertificate) {
        return toHtml((AttributeCertificate) cert);
      } else if (cert instanceof UseCondCertificate) {
        return toHtml((UseCondCertificate) cert);
      }
      
      return null;
    }

    public String toHtml(PolicyCertificate cert) {
      StringBuffer sb = doToHtml(cert);
        
      sb.append(formatHeader("Resource"));
      sb.append(formatValue(cert.getResource()));
      
      String temp  = "Maxium allowed certificate caching time";

      sb.append(formatHeader(temp));
      sb.append(formatValue(String.valueOf(cert.getCacheTime())));
      
      sb.append(formatHeader("Attribute Directories"));
      sb.append(formatValues(cert.getAttributeDirectories()));

      CertificateAuthorityInfo[] infos = cert.getCertificateAuthorityInfos();
      
      if (infos.length != 0) {
        sb.append(formatHeader("Certificate Authority Infos"));
        sb.append(newLine());

        // XXX we nedd to the id Dirs and crl Dirs
        for (int i = 0; i < infos.length; i++) {
          sb.append(formatSubHeader("Certificate Authority"));
          sb.append(formatValue(infos[i].getDistinguishedName()));
        }
      }

      sb.append(formatHeader("UseCondition Issuer Groups"));
      sb.append(newLine());
      
      UseCondIssuerGroup []groups = cert.getUseCondIssuerGroups();

      for(int i = 0; i < groups.length; i++) {
          Object []issuers = groups[i].getUCIssuers();

          sb.append(formatSubHeader("Issuers"));
          sb.append(formatValues(issuers));

          Object []ucDirs = groups[i].getDirectories();

          sb.append(formatSubHeader("UseCondition Directories"));
          sb.append(formatValues(ucDirs));
          sb.append(newLine());
      }

      sb.append(endHtml());
      return sb.toString(); 
    }
    
    public String toHtml(AttributeCertificate cert) {
      StringBuffer sb = doToHtml(cert);

      sb.append(formatHeader("User"));

      AkentiPrincipal user = cert.getPrincipal();
      Object dn = user.getName();
      Object caDn = user.getGuarantor();

      sb.append(formatValues(new Object[] { dn, caDn }));

      sb.append(formatHeader("AttributeValuePair"));
      sb.append(formatValues(new Object[] { cert.getAttribute(), cert.getValue() }));
      return sb.toString();
    }

    public String toHtml(UseCondCertificate cert) {
      StringBuffer sb = doToHtml(cert);

      sb.append(formatHeader("Resource"));
      sb.append(formatValue(cert.getResource()));

      sb.append(formatHeader("Scope"));

      if (cert.isScopeLocal()) {
        sb.append(formatValue("local"));
      } else {
        sb.append(formatValue("sub-tree"));
      }

      sb.append(formatHeader("Access"));

      if (cert.isCritical()) {
        sb.append(formatValue("Critical"));
      } else {
        sb.append(formatValue("Not critical"));
      }

      sb.append(formatHeader("Constraint"));

      String constraint = cert.getConstraint();

      if (constraint.length() == 0) {
        sb.append(formatValue("(  )"));
      } else {
        sb.append(formatValue(constraint));
      }

      AttributeInfo[] infos = cert.getAttributeInfos();

      if (infos.length != 0) {
        sb.append(formatHeader("AttributeInfos"));
      }
     
      for (int i = 0; i < infos.length; ++i) {
        int type = infos[i].getType();

        StringBuffer temp = new StringBuffer();
        
        temp.append(AttributeInfo.TYPES[type]);
        temp.append(',');
        temp.append(infos[i].getAttribute());
        temp.append(',');
        temp.append(infos[i].getValue());
        sb.append(formatValue(temp)); 
        
        switch(type) {
          case AttributeInfo.SYSTEM:
            break;
          case AttributeInfo.X509:
            sb.append(formatValues(infos[i].getCAs()));
            break;
          case AttributeInfo.AKENTI:
            sb.append(formatValues(infos[i].getIssuers()));
            sb.append(formatValues(infos[i].getDirectories()));
            break;
          case AttributeInfo.EXT_AUTH:
          default:
            throw new RuntimeException();
        }
      }

      String[] actions = cert.getActions();

      if (actions.length == 0) {
        return sb.toString();
      }

      sb.append(formatHeader("Actions"));
      StringBuffer temp = new StringBuffer(actions[0]);

      for (int i = 1; i < actions.length; ++i) {
        temp.append(',');
        temp.append(actions[i]);
      }

      sb.append(formatValue(temp));
      return sb.toString();
    }
}
 
