package lbl.akenti.policyManager.manager;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Color;

import java.beans.PropertyVetoException;
import java.io.IOException;
import java.net.URL;

import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.net.URLStreamHandlerFactory;

import javax.swing.JFrame;
import javax.swing.JDesktopPane;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JDialog;
import javax.swing.KeyStroke;
import javax.swing.ImageIcon;
import javax.swing.SwingUtilities;
import javax.swing.JInternalFrame;
import javax.swing.JScrollBar;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import lbl.akenti.policyManager.widgets.ACMHelpMenu;
import lbl.akenti.policyManager.widgets.HTMLMenuMaker;
import lbl.akenti.policyManager.widgets.LBLTopFrame;
import lbl.akenti.policyManager.widgets.LBLDesktopMaker;
import lbl.akenti.policyManager.widgets.VersionChecker;

/**
 * SUN MICROSYSTEMS RECOMMENDS JAVA 1.4.x OR ABOVE!!
 */
public class AkentiCertificateManager extends JFrame implements ActionListener {
    private JDesktopPane desktop;
    
    private static final int HTMLMENUOFFSET_X = 1000;
    private static final int HTMLMENUOFFSET_Y = 800;
    
    private static final String OFFCIAL_VERSION = "0.3c";
    
    private ClassLoader loader = Thread.currentThread().getContextClassLoader();
    
    public AkentiCertificateManager() {
        super("Akenti Certificate Manager");
        
        //Make the big window be indented 150 pixels from each edge of the screen.
        int inset = 150;
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds(inset, inset,
        screenSize.width  - inset*2,
        screenSize.height - inset*2);
        
        
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new
        WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                quit(0);
            }
        });
        
        //Set up the GUI.
        desktop = LBLDesktopMaker.getInstance();
        
        setContentPane(desktop);
        setJMenuBar(createMenuBar());
        
        //Make dragging a little faster but perhaps uglier.
        desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
        
    }
    
    protected JMenuBar createMenuBar() {
        JMenuBar menuBar = new JMenuBar();
        
        JMenu menu = new JMenu("File");
        menu.setMnemonic(KeyEvent.VK_F);
        menuBar.add(menu);
        
        JMenu helpMenu = new JMenu("Help");
        helpMenu.setMnemonic(KeyEvent.VK_H);
        menuBar.add(helpMenu);
        
        //a submenu
        menu.addSeparator();
        JMenu newMenu = new JMenu("New");
        newMenu.setMnemonic(KeyEvent.VK_N);
        
        JMenuItem polyCertController = new JMenuItem("Policy Certificate Generator");
        polyCertController.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.ALT_MASK));
        polyCertController.setActionCommand("polyCertController");
        polyCertController.addActionListener(this);
        
        JMenuItem useCertController = new JMenuItem("User Certificate Generator");
        useCertController.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, ActionEvent.ALT_MASK));
        useCertController.setActionCommand("useCertController");
        useCertController.addActionListener(this);
        
        JMenuItem attrCertController = new JMenuItem("Attribute Certificate Generator");
        attrCertController.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, ActionEvent.ALT_MASK));
        attrCertController.setActionCommand("attrCertController");
        attrCertController.addActionListener(this);
        
        newMenu.add(polyCertController);
        newMenu.add(useCertController);
        newMenu.add(attrCertController);
        
        menu.add(newMenu);
        
        JMenuItem quitItem = new JMenuItem("Quit");
        quitItem.setMnemonic(KeyEvent.VK_Q);
        quitItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_Q, ActionEvent.ALT_MASK));
        quitItem.setActionCommand("quit");
        quitItem.addActionListener(this);
        menu.add(quitItem);
        
        JMenuItem helpItem = new JMenuItem("Help Contents");
        helpItem.setMnemonic(KeyEvent.VK_C);
        helpItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_C, ActionEvent.ALT_MASK));
        helpItem.setActionCommand("help");
        helpItem.addActionListener(this);
        helpMenu.add(helpItem);
        
        JMenuItem releaseItem = new JMenuItem("Release Notes");
        releaseItem.setMnemonic(KeyEvent.VK_R);
        releaseItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_R, ActionEvent.ALT_MASK));
        releaseItem.setActionCommand("release");
        releaseItem.addActionListener(this);
        helpMenu.add(releaseItem);
        
        JMenuItem licenseItem = new JMenuItem("License");
        licenseItem.setMnemonic(KeyEvent.VK_L);
        licenseItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_L, ActionEvent.ALT_MASK));
        licenseItem.setActionCommand("license");
        licenseItem.addActionListener(this);
        helpMenu.add(licenseItem);
        
        JMenuItem aboutItem = new JMenuItem("About Akenti Certificate Manager");
        aboutItem.setMnemonic(KeyEvent.VK_A);
        aboutItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_A, ActionEvent.ALT_MASK));
        aboutItem.setActionCommand("about");
        aboutItem.addActionListener(this);
        helpMenu.add(aboutItem);
        
        return menuBar;
    }
    
    // React to menu selections.
    public void actionPerformed(ActionEvent e) {
        if ("polyCertController".equals(e.getActionCommand())) {
            createPolyCertControllerFrame();
        } else if ("attrCertController".equals(e.getActionCommand())) {
            createAttrCertControllerFrame();
        } else if ("useCertController".equals(e.getActionCommand())) {
            createUCCertControllerFrame();
        } else if ("quit".equals(e.getActionCommand())) {
            quit(0);
        } else if ("help".equals(e.getActionCommand())) {
            showHelpPane();
        } else if ("release".equals(e.getActionCommand())) {
            showInformationPane("Akenti Release Notes", loader.getResource("lbl/akenti/policyManager/docs/release_notes/ReleaseNotes.html").toString());
        } else if ("about".equals(e.getActionCommand())) {
            showAboutDialog();
        } else if ("license".equals(e.getActionCommand())) {
            if (loader == null) { // there may not be a context class loader
                loader = AkentiCertificateManager.class.getClassLoader();
            }
            showInformationPane("Akenti License", loader.getResource("lbl/akenti/policyManager/docs/license/license.txt").toString());
        } else {
            throw new RuntimeException("No such feature exists.");
        }
    }
    
    public void showHelpPane() {
        ACMHelpMenu frame = new ACMHelpMenu();
        frame.setSize(HTMLMENUOFFSET_X, HTMLMENUOFFSET_Y);
        frame.setVisible(true); // necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
    }
    public void showAboutDialog() {
        if (loader == null) { // there may not be a context class loader
            loader = AkentiCertificateManager.class.getClassLoader();
        }
        JOptionPane.showInternalMessageDialog(desktop,
        "Akenti Certificate Generator " + OFFCIAL_VERSION + " Unofficial build",
        "About Akenti Certificate Generator",
        JOptionPane.INFORMATION_MESSAGE,
        new ImageIcon(loader.getResource("lbl/akenti/policyManager/docs/pics/AkentiKeyS.png"))
        );
        
    }
    public void showInformationPane(String title, String url) {
        HTMLMenuMaker frame = new HTMLMenuMaker(title, url);
        frame.setSize(HTMLMENUOFFSET_X, HTMLMENUOFFSET_Y);
        frame.setVisible(true); // necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
    }
    protected void createPolyCertControllerFrame() {
        PolCertController polCertController = new PolCertController();
        polCertController.start();
        LBLTopFrame frame = polCertController.getFrame();
        frame.initBitSet();
        
        frame.pack();
        frame.setVisible(true); //necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
    }
    
    protected void createAttrCertControllerFrame() {
        AttrCertController attrCertController = new AttrCertController();
        attrCertController.start();
        LBLTopFrame frame = attrCertController.getFrame();
        frame.initBitSet();
        
        frame.pack();
        frame.setVisible(true); //necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
    }
    
    protected void createUCCertControllerFrame() {
        UCCertController ucCertController = new UCCertController();
        ucCertController.start();
        LBLTopFrame frame = ucCertController.getFrame();
        frame.initBitSet();
        
        frame.pack();
        frame.setVisible(true); //necessary as of 1.3
        desktop.add(frame);
        try {
            frame.setSelected(true);
        } catch (PropertyVetoException e) {}
    }
    
    protected static void quit(int status) {
        System.exit(status);
    }
    
    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    private static void createAndShowGUI() {
        AkentiCertificateManager frame = new AkentiCertificateManager();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        frame.setVisible(true);
        
    }
    public static JOptionPane createConfDialog(Component parent, Object[] components, String message) {
        final JOptionPane optionPane = new JOptionPane(components, JOptionPane.QUESTION_MESSAGE,
        JOptionPane.OK_CANCEL_OPTION);
        
        JDialog jd = optionPane.createDialog(parent, message);
        
        jd.setResizable(true);
        //Handle window closing correctly.
        jd.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                optionPane.setValue(new Integer(JOptionPane.CLOSED_OPTION));
            }
        });
        jd.show();
        
        return optionPane;
    }
    
    public static void main(String[] args) {
        
        // check JVM version, must be 1.4.x or higher
        boolean versionOK = VersionChecker.checkJavaVersion();
        
        if(!versionOK) {
            quit(1);
        }
        
        String osName = System.getProperty("os.name");
        if (!osName.equals("SunOS")) {
            try {
                javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
            }
            catch (Throwable t) {
                throw new RuntimeException(t.getMessage());
            }
        }
        /**
         * Schedule a job for the event-dispatching thread:
         * creating and showing this application's GUI.
         * Allow ldap
         */
        
        URL.setURLStreamHandlerFactory(new URLStreamHandlerFactory() {
            public URLStreamHandler
            createURLStreamHandler(String protocol) {
                
                if (protocol.equalsIgnoreCase("ldap")) {
                    return new URLStreamHandler() {
                        protected URLConnection
                        openConnection(URL url) throws IOException {
                            throw new IOException("open ldap connection unsupported");
                        }
                    };
                }
                
                return null;
            }
        });
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }
}
