package lbl.akenti.policyManager.manager;

import lbl.akenti.policyManager.widgets.LBLTopFrame;
import lbl.akenti.policyManager.widgets.LBLResourcePanel;
import lbl.akenti.policyManager.widgets.LBLIssuerPanel;
import lbl.akenti.policyManager.widgets.LBLTimePanel;
import lbl.akenti.policyManager.widgets.LBLCachePanel;
import lbl.akenti.policyManager.widgets.LBLReviewPanel;
import lbl.akenti.policyManager.widgets.LBLSavePanel;
import lbl.akenti.policyManager.widgets.AbstractController;

import lbl.akenti.policyManager.widgets.pol.PolConsts;
import lbl.akenti.policyManager.widgets.pol.PolAttrDirsPanel;
import lbl.akenti.policyManager.widgets.pol.PolCAInfosPanel;
import lbl.akenti.policyManager.widgets.pol.PolUCGroupsPanel;

import lbl.akenti.policyManager.base.GenInfo;

import lbl.akenti.util.UTCTime;

import lbl.akenti.certs.PolicyCertGenerator;

import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.Directory;

import java.net.URL;

import java.util.HashSet;

public class PolCertController
extends AbstractController
implements PolConsts {
    
    private LBLTopFrame frame;
    
    // panels used by AttrCertController
    private LBLResourcePanel lblResourcePanel;
    private PolCAInfosPanel polCAInfosPanel;
    private PolUCGroupsPanel polUCGroupsPanel;
    private LBLIssuerPanel lblIssuerPanel;
    private PolAttrDirsPanel polAttrDirsPanel;
    private LBLTimePanel lblTimePanel;
    private LBLCachePanel lblCachePanel;
    private LBLReviewPanel lblReviewPanel;
    private LBLSavePanel lblSavePanel;
    
    public PolCertController() {
        super(9); // We will be using 9 frames.
    }
    
    public void start() {
        
        frame = LBLTopFrame.getInstance("Policy Certificate Generator");
        
        frame.setController(this);
        
        // initialize panels
        lblResourcePanel = new LBLResourcePanel(frame, HEADERS[RESOURCE_INDEX]);
        
        polCAInfosPanel = new PolCAInfosPanel(frame, HEADERS[CA_INFOS_INDEX]);
        
        polUCGroupsPanel = new PolUCGroupsPanel(frame, HEADERS[UC_GROUPS_INDEX]);
        
        lblIssuerPanel = new LBLIssuerPanel(frame, HEADERS[ISSUER_INDEX]);
        
        polAttrDirsPanel = new PolAttrDirsPanel(frame, HEADERS[ATTR_DIRS_INDEX]);
        
        lblTimePanel = new LBLTimePanel(frame, HEADERS[TIME_INDEX]);
        
        lblCachePanel = new LBLCachePanel(frame, HEADERS[CACHE_INDEX]);
        
        lblReviewPanel = new LBLReviewPanel(frame, HEADERS[REVIEW_INDEX]);
        
        lblSavePanel = new LBLSavePanel(frame, HEADERS[SIGN_INDEX]);
        
        // add panels in this specific order
        frame.addPanel(lblResourcePanel, "LBLResourcePanel");
        frame.addPanel(polCAInfosPanel, "PolCAInfosPanel");
        frame.addPanel(polUCGroupsPanel, "PolUCGroupPanel");
        frame.addPanel(lblIssuerPanel, "LBLIssuerPanel");
        frame.addPanel(polAttrDirsPanel, "PolAttrDirsPanel");
        frame.addPanel(lblTimePanel, "LBLTimePanel");
        frame.addPanel(lblCachePanel, "LBLCachePanel");
        frame.addPanel(lblReviewPanel, "LBLReviewPanel");
        frame.addPanel(lblSavePanel, "LBLSavePanel");
    }
    
    public LBLTopFrame getFrame() {
        return frame;
    }
    public void clearLBLResourcePane() {
        lblResourcePanel.clear();
    }
    
    /**
     * Loads inputs in the panel that corresponds to <i>index</i>.
     *
     * @param index index of panel that needs to be loaded
     *
     * @return true if this method should be called when displaying this
     *              frame in the future.
     *         false otherwise.
     */
    public boolean loadInputsIntoPanel(int index) throws Exception {
        switch(index) {
            case CA_INFOS_INDEX:
                URL base = new URL(lblResourcePanel.getBaseURL());
                String resource = lblResourcePanel.getResource();
                if (info == null || !(info.hasResource(resource) && info.hasBaseURL(base))) {
                    // Make sure we load the GenInfo first.
                    GenInfo tempInfo = new GenInfo(resource, base);
                    tempInfo.loadResFileInfo();
                    tempInfo.loadPolicies();
                    this.info = tempInfo;
                    
                    polCAInfosPanel.setOptional(!info.isRoot());
                    polCAInfosPanel.setCAInfos(info.getCertificateAuthorityInfos());
                    frame.clearBits();
                }
                return true;
                
            case UC_GROUPS_INDEX:
                polUCGroupsPanel.setUCGroups(info.getUseCondIssuerGroups());
                return true;
                
            case ISSUER_INDEX:
                UseCondIssuerGroup[] ucGroups = polUCGroupsPanel.getUCGroups();
                HashSet ucIssuers = new HashSet();
                
                for (int i = 0; i < ucGroups.length; ++i) {
                    AkentiPrincipal[] aps = ucGroups[i].getUCIssuers();
                    for (int j = 0; j < aps.length; ++j) {
                        ucIssuers.add(aps[j]);
                    }
                }
                
                AkentiPrincipal []temp = new AkentiPrincipal[ucIssuers.size()];
                ucIssuers.toArray(temp);
                lblIssuerPanel.setIssuers(temp);
                return false;
                
            case ATTR_DIRS_INDEX:
                polAttrDirsPanel.clear();
                polAttrDirsPanel.addAttrDirs(info.getAttributeDirectories());
                return true;
                
            case TIME_INDEX:
                return true;
                
            case CACHE_INDEX:
                lblCachePanel.setCacheTime(info.getCacheTime());
                return true;
                
            case REVIEW_INDEX:
                
                PolicyCertGenerator gen = new PolicyCertGenerator();
                
                gen.setVersionNumber(2);
                gen.setUniqueID("UID");
                
                UTCTime utc = new UTCTime();
                
                gen.setNotBefore(utc);
                gen.setNotAfter(new UTCTime(utc, lblTimePanel.getLifeTime()));
                gen.setIssuer(new AkentiIssuer(lblIssuerPanel.getIssuer()));
                
                CertificateAuthorityInfo []infos = polCAInfosPanel.getCAInfos();
                
                if (infos.length != 0) {
                    gen.setCertificateAuthorityInfos(infos);
                }
                
                gen.setResource(lblResourcePanel.getResource());
                gen.setUseCondIssuerGroups(polUCGroupsPanel.getUCGroups());
                
                Directory [] dirs = polAttrDirsPanel.getAttrDirs();
                
                if (dirs.length != 0) {
                    gen.setAttributeDirectories(dirs);
                }
                
                long cacheTime = lblCachePanel.getCacheTime();
                
                if (cacheTime != -1) {
                    gen.setCacheTime(cacheTime);
                }
                
                lblReviewPanel.setCertificate(gen.generate());
                this.gen = gen;
                return false;
                
            case SIGN_INDEX:
                lblSavePanel.clear();
                return false;
                
            default:
                throw new RuntimeException();
        }
    }
    
    protected String getHelpMessage(int index) {
        return helpMess[index];
    }
    
    public void showDialog(String title, String info) {
        frame.showDialog(title, info);
    }
    
    public char[] getPassword() {
        return lblSavePanel.getPassword();
    }
    public String getP12Filename() {
        return lblSavePanel.getKeyFileName();
    }
    public String getSaveLocation() {
        return lblSavePanel.getCertFileName();
    }
    public void cancel() {
        frame.cancel();
    }
    /**
     *   Used to call doEnable() on panels
     *
     *   param int index the index of the panel
     */
    public void checkEnableButton(int index) {
        switch(index) {
            case CA_INFOS_INDEX:
                break;
                
            case UC_GROUPS_INDEX:
                break;
                
            case ISSUER_INDEX:
                lblIssuerPanel.doEnable();
                break;
                
            case ATTR_DIRS_INDEX:
                break;
            case TIME_INDEX:
                
                break;
                
            case REVIEW_INDEX:
                
                break;
                
            case CACHE_INDEX:
                
                break;
            case SIGN_INDEX:
                lblSavePanel.doEnable();
                break;
                
            default:
                
                throw new RuntimeException("Error in PolCertController.checkEnable");
        }
    }
    
}
