package lbl.akenti.policyManager.manager;

import java.security.Security;

import java.util.Properties;
import java.util.BitSet;

import lbl.akenti.certs.elements.Condition;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.UseCondCertGenerator;

import lbl.akenti.policyManager.base.GenInfo;

import lbl.akenti.policyManager.widgets.LBLTopFrame;
import lbl.akenti.policyManager.widgets.LBLResourcePanel;
import lbl.akenti.policyManager.widgets.LBLIssuerPanel;
import lbl.akenti.policyManager.widgets.LBLTimePanel;
import lbl.akenti.policyManager.widgets.LBLCachePanel;
import lbl.akenti.policyManager.widgets.LBLReviewPanel;
import lbl.akenti.policyManager.widgets.LBLSavePanel;
import lbl.akenti.policyManager.widgets.AbstractController;

import lbl.akenti.policyManager.widgets.uc.UCConsts;
import lbl.akenti.policyManager.widgets.uc.UCExpressionPanel;
import lbl.akenti.policyManager.widgets.uc.UCScopePanel;

import lbl.akenti.util.UTCTime;

import java.net.URL;


public class UCCertController extends AbstractController implements UCConsts {
    
    private LBLTopFrame frame;
    
    // panels used by AttrCertController
    private LBLResourcePanel lblResourcePanel;
    private UCExpressionPanel ucExpressionPanel;
    private UCScopePanel ucScopePanel;
    private LBLIssuerPanel lblIssuerPanel;
    private LBLTimePanel lblTimePanel;
    private LBLReviewPanel lblReviewPanel;
    private LBLSavePanel lblSavePanel;
    
    public UCCertController() {
        super(7);   // We will be using 7 panels.
    }
    
    public void start() {
        
        frame = LBLTopFrame.getInstance("User Condition Certificate Generator");
        
        frame.setController(this);
        
        // initialize panels
        lblResourcePanel = new LBLResourcePanel(frame, HEADERS[RESOURCE_INDEX]);
        lblResourcePanel.setController(this);
        
        lblIssuerPanel = new LBLIssuerPanel(frame, HEADERS[ISSUER_INDEX]);
        lblIssuerPanel.setController(this);
        
        ucExpressionPanel = new UCExpressionPanel(frame, HEADERS[EXPR_INDEX]);
        ucExpressionPanel.setController(this);
        
        ucScopePanel = new UCScopePanel(frame, HEADERS[SCOPE_INDEX]);
        ucScopePanel.setController(this);
        
        lblTimePanel = new LBLTimePanel(frame, HEADERS[TIME_INDEX]);
        lblTimePanel.setController(this);
        
        lblReviewPanel = new LBLReviewPanel(frame, HEADERS[REVIEW_INDEX]);
        lblReviewPanel.setController(this);
        
        lblSavePanel = new LBLSavePanel(frame, HEADERS[SIGN_INDEX]);
        lblSavePanel.setController(this);
        
        // add panels in this specific order
        frame.addPanel(lblResourcePanel, "LBLResourcePanel");
        frame.addPanel(lblIssuerPanel, "LBLIssuerPanel");
        frame.addPanel(ucExpressionPanel, "UCExpressionPanel");
        frame.addPanel(ucScopePanel, "UCScopePanel");
        frame.addPanel(lblTimePanel, "LBLTimePanel");
        frame.addPanel(lblReviewPanel, "LBLReviewPanel");
        frame.addPanel(lblSavePanel, "LBLSavePanel");
        
    }
    
    
    public void clearLBLResourcePane() {
        lblResourcePanel.clear();
    }
    public LBLTopFrame getFrame() {
        return frame;
    }
    public char[] getPassword() {
        return lblSavePanel.getPassword();
    }
    public String getP12Filename() {
        return lblSavePanel.getKeyFileName();
    }
    public String getSaveLocation() {
        return lblSavePanel.getCertFileName();
    }
    
    protected boolean
    loadInputsIntoPanel(int index) throws Exception {
        switch(index) {
            case ISSUER_INDEX:
                URL base = new URL(lblResourcePanel.getBaseURL());
                String resource = lblResourcePanel.getResource();
                if (info == null || !(info.hasResource(resource) && info.hasBaseURL(base))) {
                    GenInfo tempInfo = new GenInfo(resource, base);
                    
                    tempInfo.loadResFileInfo();
                    tempInfo.loadPolicies();
                    
                    if (!tempInfo.getTopResource().equals(resource)) {
                        throw new Exception("No policy cert corresponding to " + resource);
                    }
                    
                    this.info = tempInfo;
                    
                    lblIssuerPanel.clear();
                    lblIssuerPanel.setIssuers(info.getUCIssuers());
                    frame.clearBits();
                }
                
                return true;
                
            case EXPR_INDEX:
                ucExpressionPanel.clear();
                ucExpressionPanel.setHintAttributes(info.getGenAttributes());
                return true;
                
            case SCOPE_INDEX:
                ucScopePanel.clear();
                ucScopePanel.setActions(info.getActions());
                return true;
                
            case TIME_INDEX:
                return true;
                
            case REVIEW_INDEX:
                UseCondCertGenerator gen = new UseCondCertGenerator();
                
                gen.setVersionNumber(2);
                gen.setUniqueID("UID");
                
                UTCTime utc = new UTCTime();
                
                gen.setNotBefore(utc);
                gen.setNotAfter(new UTCTime(utc, lblTimePanel.getLifeTime()));
                
                gen.setIssuer(new AkentiIssuer(lblIssuerPanel.getIssuer()));
                gen.setResource(lblResourcePanel.getResource());
                gen.setLocalScope(ucScopePanel.isScopeLocal());
                gen.setCritical(ucScopePanel.getCritical());
                
                Condition cond = new Condition(
                ucExpressionPanel.getExpression(),
                ucExpressionPanel.getAttributeInfos()
                );
                
                gen.setCondition(cond);
                gen.setActions(ucScopePanel.getActions());
                
                lblReviewPanel.setCertificate(gen.generate());
                this.gen = gen;
                return false;
                
            case SIGN_INDEX:
                lblSavePanel.clear();
                return false;
                
            default:
                throw new RuntimeException();
        }
        
    }
    
    public void showDialog(String title, String info) {
        frame.showDialog(title, info);
    }
    /**
     *
     */
    public LBLSavePanel getSavePanel() {
        return lblSavePanel;
    }
    /**
     */
    protected String getHelpMessage(int currentFrameIndex) {
        return helpMessages[currentFrameIndex];
    }
    
    public void cancel() {
        frame.cancel();
    }
    /**
     *   Used to call doEnable() on panels
     *
     *   param int index the index of the panel
     */
    public void checkEnableButton(int index) {
        switch(index) {
            case ISSUER_INDEX:
                lblIssuerPanel.doEnable();
                break;
                
            case EXPR_INDEX:
                break;
                
            case SCOPE_INDEX:
                break;
                
            case TIME_INDEX:
                break;
                
            case REVIEW_INDEX:
                break;
                
            case SIGN_INDEX:
                lblSavePanel.doEnable();
                break;
                
            default:
                
                throw new RuntimeException("Error in UCCertController.checkEnable");
        }
    }
}
