package lbl.akenti.policyManager.widgets;

import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import java.awt.event.ActionListener;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.util.NoSuchElementException;

/**
 * A sub-class of JPanel that represents a row of buttons that
 * can be accessed by their label/command string.
 */
public class LBLButtonPanel extends JPanel implements ActionListener {
    ActionListener listener;
    
    /**
     * Constructor that creates a panel but without any buttons
     */
    public LBLButtonPanel() {
        this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        this.add(Box.createHorizontalGlue());
    }
    
    /**
     * Constructor that creates a row of buttons.
     *
     * @param cmds an array of strings for labeling the buttons
     */
    public LBLButtonPanel(String []cmds) {
        this();
        
        if (cmds == null) {
            throw new IllegalArgumentException();
        }
        
        for (int i = 0; i < cmds.length; i++) {
            addButton(cmds[i]);
        }
    }
    
    public void setActionListener(ActionListener listener) {
        if (listener == null)
            throw new RuntimeException();
        
        this.listener = listener;
    }
    
    /**
     * Adds a single button to the panel
     *
     * @param  cmd String to label the button with
     * @return JButton that was requested
     */
    public JButton addButton(String cmd) {
        JButton button = new JButton(cmd);
        
        button.addActionListener(this);
        this.add(button);
        this.add(Box.createHorizontalGlue());
        return button;
    }
    
    /**
     * returns one of the buttons in the set of buttons
     *
     * @param cmd Return the button with this cmd
     */
    public JButton getButton(String cmd) {
        
        Object [] components = getComponents();
        
        for (int i = 1; i < components.length; i += 2) {
            JButton button = (JButton) components[i];
            
            if (button.getText().equals(cmd)) {
                return button;
            }
        }
        
        throw new NoSuchElementException();
    }
    
    /**
     * Called when a button belonging to this panel is pushed
     */
    public void actionPerformed(ActionEvent evt) {
        ActionListener listener = this.listener;
        
        if (listener != null) {
            listener.actionPerformed(evt);
        }
    }
    
    /**
     */
    public void setButtonEnabled(String cmd, boolean enable) {
        getButton(cmd).setEnabled(enable);
    }
    
    /**
     */
    public void setButtonVisible(String cmd, boolean visible) {
        getButton(cmd).setVisible(visible);
    }
    
    /**
     */
    public void enableBlueButton(String cmd) {
        enableBlueButton(getButton(cmd));
    }
    
    /**
     */
    public void disableBlueButton(String cmd) {
        disableBlueButton(getButton(cmd));
    }
    
    /**
     */
    public void enableRedButton(String cmd) {
        enableRedButton(getButton(cmd));
    }
    
    /**
     */
    public void disableRedButton(String cmd) {
        disableRedButton(getButton(cmd));
    }
    
    /**
     */
    public static void enableBlueButton(JButton button) {
        button.setEnabled(true);
        button.setBackground(Color.blue);
        button.setForeground(Color.white);
    }
    
    /**
     */
    public static void disableBlueButton(JButton button) {
        button.setEnabled(false);
        button.setBackground(Color.blue);
        button.setForeground(Color.white);
    }
    
    /**
     */
    public static void enableRedButton(JButton button) {
        button.setEnabled(true);
        button.setBackground(Color.red);
        button.setForeground(Color.white);
    }
    
    /**
     */
    public static void disableRedButton(JButton button) {
        button.setEnabled(false);
        button.setBackground(Color.pink);
        button.setForeground(Color.gray);
    }
    
}
