package lbl.akenti.policyManager.widgets;

import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.Color;

import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.Directory;

/**
 *
 */
public
class LBLCaListPane
extends JOptionPane
implements ActionListener, ListDataListener, ListSelectionListener {
    
    private static String PANEL_HEADER  =
    "<html><font size=+1 color=blue>Certificate Authorities" +
    "<li><font color=blue>Specify Certificate Authorities";
    
    private JList caList;
    
    private JButton addCaButton = new JButton("Add");
    private JButton remCaButton = new JButton("Remove");
    
    private JButton okButton = new JButton("OK");
    private JButton cancelButton = new JButton("CANCEL");;
    
    private LBLCaPane caPanel = null;
    
    /**
     * Builds this component.
     *
     */
    public LBLCaListPane() {
        setLayout(new BorderLayout(5, 5));
        
        Border border = BorderFactory.createEmptyBorder(5, 5, 5, 5);
        
        setBorder(border);
        
        add(LBLLabelFactory.createHtmlLabel(PANEL_HEADER), "North");
        
        DefaultListModel model = new DefaultListModel();
        
        model.addListDataListener(this);
        
        caList = new JList(model);
        caList.setSelectionMode(2);
        caList.addListSelectionListener(this);
        
        addCaButton.addActionListener(this);
        addCaButton.setActionCommand("addca");
        enableBlueButton(addCaButton);
        
        remCaButton.addActionListener(this);
        remCaButton.setActionCommand("remca");
        disableRedButton(remCaButton);
        
        add(createListPanel(), "Center");
        
        JPanel buttonPanel = new JPanel();
        
        okButton.setActionCommand("ok");
        okButton.addActionListener(this);
        disableRedButton(okButton);
        buttonPanel.add(okButton);
        
        cancelButton.addActionListener(this);
        cancelButton.setActionCommand("cancel");
        buttonPanel.add(cancelButton);
        add(buttonPanel, "South");
    }
    
    /**
     */
    private JPanel
    createListPanel() {
        JPanel panel = new JPanel(new BorderLayout(5, 5));
        JLabel label = new JLabel("Certificate Authorities");
        
        label.setForeground(Color.red);
        panel.add(label, "North");
        caList.setForeground(Color.blue);
        panel.add(new JScrollPane(caList), "Center");
        
        return WrapUtil.wrapComponents(
        panel,
        WrapUtil.wrapVertical(addCaButton, remCaButton)
        );
    }
    
    /**
     */
    public void setOKLabel(String label) {
        okButton.setLabel(label);
    }
    
    /**
     */
    private LBLCaPane getCaPane() {
        if (caPanel == null) {
            caPanel = new LBLCaPane();
        }
        
        return caPanel;
    }
    
    /**
     */
    public void actionPerformed(ActionEvent e) {
        String cmd = e.getActionCommand();
        
        if (cmd.equals("addca")) {
            LBLCaPane caPanel = getCaPane();
            
            caPanel.createDialog(this, "AddCaDialog").setVisible(true);
            
            Object test = caPanel.getValue();
            
            if ((test != null) && !test.equals(GenConsts.AK_CANCEL_VALUE)) {
                Object[] cas = (Object[]) test;
                addCaDns((DistinguishedName[]) cas);
            }
        } else if (cmd.equals("remca")) {
            int []indices = caList.getSelectedIndices();
            DefaultListModel model = (DefaultListModel) caList.getModel();
            
            for (int i = indices.length - 1; i > -1; --i) {
                model.removeElementAt(indices[i]);
            }
        } else if (cmd.equals("ok")) {
            DefaultListModel model = (DefaultListModel) caList.getModel();
            DistinguishedName[] cas = new DistinguishedName[model.getSize()];
            
            model.copyInto(cas);
            setValue(cas);
        } else if (cmd.equals("cancel")) {
            setValue(GenConsts.AK_CANCEL_VALUE);
        } else {
            throw new RuntimeException(cmd + " not supported");
        }
    }
    
    /**
     */
    public void valueChanged(ListSelectionEvent e) {
        doListEnable(caList.getSelectedIndex() != -1);
    }
    
    /**
     */
    public void contentsChanged(ListDataEvent e) {
    }
    
    /**
     */
    public void intervalAdded(ListDataEvent e) {
        doOkEnable(numOfCas() != 0);
    }
    
    /**
     */
    public void intervalRemoved(ListDataEvent e)  {
        doOkEnable(numOfCas() != 0);
        doListEnable(caList.getSelectedIndex() != -1);
    }
    
    /**
     */
    public void clear() {
        ((DefaultListModel) caList.getModel()).clear();
    }
    
    /**
     */
    private void doListEnable(boolean on) {
        if (on) {
            enableRedButton(remCaButton);
        } else {
            disableRedButton(remCaButton);
        }
    }
    
    /**
     */
    private void doOkEnable(boolean on) {
        if (on) {
            enableRedButton(okButton);
        } else {
            disableRedButton(okButton);
        }
    }
    
    /**
     */
    private void enableBlueButton(JButton button) {
        LBLButtonPanel.enableBlueButton(button);
    }
    
    /**
     */
    private void disableBlueButton(JButton button) {
        LBLButtonPanel.disableBlueButton(button);
    }
    
    /**
     */
    private void enableRedButton(JButton button) {
        LBLButtonPanel.enableRedButton(button);
    }
    
    /**
     */
    private void disableRedButton(JButton button) {
        LBLButtonPanel.disableRedButton(button);
    }
    
    /**
     */
    public int numOfCas() {
        return caList.getModel().getSize();
    }
    
    /**
     */
    public DistinguishedName[] getCaDns() {
        DefaultListModel model = (DefaultListModel) caList.getModel();
        DistinguishedName[] cas = new DistinguishedName[model.getSize()];
        
        model.copyInto(cas);
        return cas;
    }
    
    /**
     */
    public boolean addCa(DistinguishedName ca) {
        return addCaDns(new DistinguishedName[] { ca }) != 0;
    }
    
    /**
     */
    public int addCaDns(DistinguishedName[] cas) {
        DefaultListModel model = (DefaultListModel) caList.getModel();
        int added = 0;
        int size = model.getSize();
        
        caList.clearSelection();
        
        for (int i = 0; i < cas.length; ++i) {
            int index = model.indexOf(cas[i]);
            
            if (index != -1) {
                caList.addSelectionInterval(index, index);
            } else {
                model.addElement(cas[i]);
                caList.addSelectionInterval(size, size++);
                ++added;
            }
        }
        
        return added;
    }
}
