package lbl.akenti.policyManager.widgets;

import javax.swing.JFileChooser;

import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Color;

import java.util.StringTokenizer;

import lbl.akenti.policyManager.widgets.LBLPanel;
import lbl.akenti.policyManager.widgets.LBLDirField;
/**
 *  Allows the user to provide the resource and the base url.
 */
public class LBLResourcePanel extends LBLPanel implements ActionListener, DocumentListener {
    
    private JTextField resourceField;
    private LBLDirField dirField;
    
    private JButton browse = new JButton("Browse");
    
    /**
     * Constructor
     * Sets up the frame with default host and and port
     *
     * @param title title of the frame
     * @param instructions header/instruction to be displayed at top of frame
     */
    public LBLResourcePanel(LBLTopFrame parent, String header) {
        super(parent, header);
        
        JLabel[] labels = { new JLabel("Resource"), new JLabel("base URL") };
        
        labels[0].setForeground(Color.red);
        labels[1].setForeground(Color.red);
        
        resourceField = LBLTextFieldFactory.createTextField(": ", this);
        
        browse.setActionCommand("browse");
        browse.addActionListener(this);
        
        JComponent resourceComponent = WrapUtil.wrapHorizontal(
        resourceField,
        browse
        );
        
        // dirField = new LBLDirField(null, new String[] { "http", "https", "file" });
        dirField = new LBLDirField(null, new String[] { "http", "file" });
        dirField.setDocumentListener(this);
        
        add(
        wrapComponents(labels, new JComponent[] { resourceComponent, dirField}),
        "Center"
        );
        
        String temp = System.getProperty("RootResourceName");
        
        if (temp != null) {
            resourceField.setText(temp);
        }
        
        temp = System.getProperty("BaseURL");
        
        if (temp != null) {
            dirField.setDir(temp);
        }
        
        doEnable();
    }
    
    private JFileChooser fileChooser;
    
    JFileChooser createChooser(String url) {
        return new JFileChooser(new DFileSystemView(url));
    }
    
    /**
     */
    JFileChooser getFileChooser() {
        String url = getBaseURL();
        
        if (fileChooser == null) {
            fileChooser = createChooser(url);
        } else {
            DFileSystemView view = (DFileSystemView) fileChooser.getFileSystemView();
            
            if (!url.equals(view.getBase())) {
                fileChooser = createChooser(url);
            }
        }
        
        fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        return fileChooser;
    }
    
    
    
    public void actionPerformed(ActionEvent event) {
        String cmd = event.getActionCommand();
        if (cmd.equals("browse")) {
            if (getBaseURL().toLowerCase().startsWith("https")) {
                if (!getController().initSSLInfo()) {
                    return;
                }
            }
            
            JFileChooser chooser = getFileChooser();
            
            chooser.showOpenDialog(this);
            
            Object file = chooser.getSelectedFile();
            
            if (file != null) {
                resourceField.setText(file.toString().substring(1));
            }
            
        } else {
            throw new RuntimeException();
        }
    }
    
    /**
     */
    public void changedUpdate(DocumentEvent e)  {
        doEnable();
    }
    
    /**
     */
    public void insertUpdate(DocumentEvent e)  {
        doEnable();
    }
    
    /**
     */
    public void removeUpdate(DocumentEvent e) {
        doEnable();
    }
    
    /**
          If Base URL textfield is empty then disable
          "Browse" button.
   
          Else, enable "Browse" button
   
          If BOTH Resource and Base URL textfields are
          NOT empty, then ENABLE both "Browse" and "Next" buttons.
   
     */
    private void doEnable() {
        // if Base URL is not empty then enable Browse
        if (!getBaseURL().equals("")) {
            enableBlueButton(browse);
            // if Resource field is not equal to 0 length then enable Next
            if (resourceField.getText().length() != 0) {
                enableBlueButton("Next");
                return;
            }
            
        } else {
            // Base URL is empty so disable Browse button
            disableBlueButton(browse);
        }
        // base url was empty so disable next
        disableBlueButton("Next");
    }
    
    /**
     */
    public void clear() {
        resourceField.setText("");
        disableBlueButton("Next");
    }
    
    
    /**
     * Resource cannot begin with a slash and cannot end with a slash
     * It also cannot contina blank spaces.
     */
    public String getResource() {
        StringTokenizer st = new StringTokenizer(
        resourceField.getText(),
        "/"
        );
        
        if (!st.hasMoreTokens()) {
            return "";
        }
        
        StringBuffer sb = new StringBuffer(st.nextToken());
        
        while (st.hasMoreTokens()) {
            sb.append('/');
            sb.append(st.nextToken());
        }
        
        String resource = sb.toString();
        
        resourceField.setText(resource);
        return resource;
    }
    
    /**
     */
    public String getBaseURL() {
        String url = dirField.getDir();
        
        if (url.equals("")) {
            return "";
        } else if (!url.endsWith("/")) {
            url += "/";
        }
        
        return url;
    }
}
