package lbl.akenti.policyManager.widgets;

import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JPasswordField;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.JComponent;
import javax.swing.JCheckBox;
import javax.swing.BorderFactory;
import java.awt.BorderLayout;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

/**
 * Prompts the user to enter the name of their p12 file
 * the password, and the trusted CA file.
 */
public class LBLSSLPane
extends JOptionPane
implements ActionListener, DocumentListener {
    private static final String [] LABELS = {
        "Pkcs12 file Name",
        "Pkcs12 file Password",
        "Trusted CA File Name",
    };
    
    private static final ExtensionFileFilter []FORMAT_FILTERS = {
        new ExtensionFileFilter("Netscape P12 File", "p12")
    };
    
    private JTextField pkcs12FileField;
    private JPasswordField passwordField;
    private JTextField caFileField;
    
    private JFileChooser chooser;
    private File pkcs12Dir;
    private File caDir;
    private JButton okButton;
    
    /**
     * Builds this frame.
     *
     * @param title this frame's title
     * @param instructions displayed on the top portion of this frame
     */
    public LBLSSLPane(String header) {
        JOptionPane contentPane = this;
        
        contentPane.setLayout(new BorderLayout(5, 5));
        contentPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        
        if (header != null) {
            contentPane.add(LBLLabelFactory.createHtmlLabel(header), "North");
        }
        
        JLabel []labels = new JLabel[LABELS.length];
        
        for (int i = 0; i < labels.length; i++) {
            labels[i] = new JLabel(LABELS[i]);
            labels[i].setForeground(Color.red);
        }
        
        JComponent []components  = new JComponent[LABELS.length];
        JButton browse = new JButton("Browse");
        
        browse.setActionCommand("Browse for Pkcs12File");
        browse.addActionListener(this);
        pkcs12FileField = LBLTextFieldFactory.createFileTextField(this);
        components[0] = wrapComponents(pkcs12FileField, browse);
        components[1] = passwordField = LBLTextFieldFactory.createPasswordField(this);
        
        browse = new JButton("Browse");
        browse.setActionCommand("Browse for CAFile");
        browse.addActionListener(this);
        caFileField = LBLTextFieldFactory.createFileTextField(this);
        
        components[2] = wrapComponents(caFileField, browse);
        
        contentPane.add(wrapComponents(labels, components), "Center");
        
        LBLButtonPanel buttonPanel = new LBLButtonPanel();
        JButton cancelButton = buttonPanel.addButton("Cancel");
        
        cancelButton.setActionCommand("Cancel");
        cancelButton.addActionListener(this);
        okButton = buttonPanel.addButton("OK");
        okButton.setForeground(Color.white);
        okButton.setBackground(Color.blue);
        okButton.setActionCommand("OK");
        okButton.addActionListener(this);
        okButton.setEnabled(false);
        contentPane.add(buttonPanel, "South");
    }
    
    /**
     * Create a file chooser dialog box for selecting the pkcs12 file.
     */
    private JFileChooser getFileChooser() {
        if (chooser == null) {
            chooser = new JFileChooser();
            chooser.setFileHidingEnabled(false);
        }
        
        FileFilter filters[] =  chooser.getChoosableFileFilters();
        
        for (int i = 0; i < filters.length; i++) {
            chooser.removeChoosableFileFilter(filters[i]);
        }
        
        return chooser;
    }
    
    /**
     * Handles button clicking events (the browse buttons)
     * Opens up file choosers.
     */
    public void actionPerformed(ActionEvent evt) {
        JFileChooser chooser = getFileChooser();
        String cmd = evt.getActionCommand();
        
        if (cmd.equals("Browse for Pkcs12File")) {
            for (int i = 0; i < FORMAT_FILTERS.length; i++) {
                chooser.addChoosableFileFilter(FORMAT_FILTERS[i]);
            }
            
            chooser.setDialogTitle("Select the file that contains your private key");
            chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            
            if (pkcs12Dir != null) {
                chooser.setCurrentDirectory(pkcs12Dir);
            }
            
            if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                File keyFile = chooser.getSelectedFile();
                
                if (keyFile != null) {
                    pkcs12FileField.setText(keyFile.toString());
                }
            }
            
            pkcs12Dir = chooser.getCurrentDirectory();
            chooser.setSelectedFile(null);
        } else if (cmd.equals("Browse for CAFile")) {
            chooser.setDialogTitle("Select the file that contains the trusted CAs");
            chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
            
            if (caDir != null) {
                chooser.setCurrentDirectory(caDir);
            }
            
            if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                File certFile = chooser.getSelectedFile();
                
                if (certFile != null) {
                    caFileField.setText(certFile.toString());
                }
            }
            
            caDir = chooser.getCurrentDirectory();
            chooser.setSelectedFile(null);
        } else if (cmd.equals("OK")) {
            setValue("OK");
        } else if (cmd.equals("Cancel")) {
            System.out.println("CANCELLING");
            setValue("CANCEL");
        } else {
            throw new RuntimeException();
        }
    }
    
    /**
     */
    public void changedUpdate(DocumentEvent e)  {
        doEnable();
    }
    
    /**
     */
    public void insertUpdate(DocumentEvent e)  {
        doEnable();
    }
    
    /**
     */
    public void removeUpdate(DocumentEvent e) {
        doEnable();
    }
    
    /**
     */
    private void doEnable() {
        boolean on =
        pkcs12FileField.getText().length() != 0
        &&
        passwordField.getText().length() != 0
        &&
        caFileField.getText().length() != 0
        ;
        
        if (on) {
            okButton.setEnabled(true);
        } else {
            okButton.setEnabled(false);
        }
    }
    
    /**
     */
    public char[] getPassword() {
        return passwordField.getPassword();
    }
    
    /**
     */
    public void setPassword(String password) {
        passwordField.setText(password);
    }
    
    /**
     */
    public String getFormat() {
        return "P12";
    }
    
    /**
     */
    public String getPkcs12FileName() {
        return pkcs12FileField.getText();
    }
    
    /**
     */
    public void setPkcs12FileName(String pkcs12FileName) {
        pkcs12FileField.setText(pkcs12FileName);
    }
    
    /**
     */
    public String getCAFileName() {
        return caFileField.getText();
    }
    
    /**
     */
    public void setCAFileName(String caFileName) {
        caFileField.setText(caFileName);
    }
    
    /**
     */
    public void clear() {
        pkcs12FileField.setText("");
        passwordField.setText("");
        caFileField.setText("");
    }
    
    /**
     * Private class used with the file choosers to allow
     * selection of only certain types of files
     */
    private static class ExtensionFileFilter extends FileFilter {
        private String description;
        private String extension;
        
        ExtensionFileFilter(String description, String extension) {
            this.description = description;
            this.extension = extension;
        }
        
        public boolean accept(File file) {
            return file.isDirectory() || file.getName().endsWith("." + extension);
        }
        
        public String getDescription() {
            StringBuffer sb = new StringBuffer();
            
            sb.append(description);
            sb.append("(*.");
            sb.append(extension);
            sb.append(")");
            return sb.toString();
        }
    }
    
    protected JPanel wrapComponents(JLabel []labels, JComponent []components) {
        return WrapUtil.wrapComponents(labels, components);
    }
    
    /**
     * Wraps two components horizontally next to each other
     */
    protected JPanel wrapComponents(JComponent comp1, JComponent comp2) {
        return WrapUtil.wrapComponents(comp1, comp2);
    }
}
