package lbl.akenti.policyManager.widgets.pol;

/**
 * A class to contain the constants used by the attrGen class
 */
public interface PolConsts {
    
    // Indices for accessing the controllers frames
    public static final int RESOURCE_INDEX  = 0;
    public static final int CA_INFOS_INDEX  = 1;
    public static final int UC_GROUPS_INDEX = 2;
    public static final int ISSUER_INDEX    = 3;
    public static final int ATTR_DIRS_INDEX = 4;
    public static final int TIME_INDEX      = 5;
    public static final int CACHE_INDEX     = 6;
    public static final int REVIEW_INDEX    = 7;
    public static final int SIGN_INDEX      = 8;
    
    static final String[] TITLES =  {
        "Resource Info",
        "Certificate Authority Infos",
        "UseCondition Issuer Groups",
        "Policy Issuers",
        "Attribute Directories",
        "Certificate Lifetime",
        "Cache Time",
        "Certificate Content",
        "Information for Signing And Saving the Certificate"
    } ; // end of header
    
    // Resource Frame
    public static String RESOURCE_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[RESOURCE_INDEX] +
    "<li><font color=blue>Enter the name of the resource." +
    "<li><font color=blue>Enter the base url."
    ; // end of header
    
    // PolCAInfosFrame
    public static String CA_INFOS_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[CA_INFOS_INDEX] +
    "<li><font color=blue>Provide information about trusted CAs." +
    "<li><font color=blue>These infos consist of CAs and ID dirs." +
    "<li><font color=blue>Note this is only required for root resources."
    ; // end of header
    
    // PolUCGroupsFrame
    public static String UC_GROUPS_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[UC_GROUPS_INDEX] +
    "<li><font color=blue>Must provide the UseCondition Issuers." +
    "<li><font color=blue>These groups consist of UC issuers and the <br>" +
    "directories in which the UCs are stored." +
    "<li><font color=blue>Note that you need to belong to one of these groups."
    ; // end of header
    
    // Issuer Frame
    public static String ISSUER_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[ISSUER_INDEX] +
    "<li><font color=blue>Select Policy Issuer (yourself) from this list"
    ; // end of header
    
    // PolDirFrame
    public static String ATTR_DIRS_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[ATTR_DIRS_INDEX] +
    "<li><font color=blue>List directories where <i>Attribute " +
    "Certificates</i> can be found." +
    "<li><font color=blue>Note this is only required for root resources."
    ; // end of header
    
    // Sign And Save Frame
    public static String SIGN_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[SIGN_INDEX] +
    "<li><font color=blue>Enter your <font color=red>private key</font> file path " +
    "and <font color=red>passphase</font> if <br> did not already provide." +
    "<li><font color=blue>Enter the full path of the file where" +
    " the newly <br>generated Attribute Certificate will be stored."
    
    ;
    
    // Time Frame
    public static String TIME_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[TIME_INDEX] +
    "<li><font color=blue>Choose how long the certificate will be valid for."
    ; // end of header
    
    // Cache Frame
    public static String CACHE_HEADER =
    "<html><font size=+1 color=blue>" + TITLES[CACHE_INDEX] +
    "<li><font color=blue>Enter time in seconds for the Policy to" +
    " be stored in cache memory."
    ; // end of header
    
    // Review Frame
    public static String REVIEW_HEADER =
    "<html><font size=+1 color=blue>Review your Policy Certificate" +
    "<li><font color=blue>Click <i>next</i> when ready.";
    ;
    
    // CONVENIENT ARRAY OF HEADERS
    static final String[] HEADERS = {
        RESOURCE_HEADER,
        CA_INFOS_HEADER,
        UC_GROUPS_HEADER,
        ISSUER_HEADER,
        ATTR_DIRS_HEADER,
        TIME_HEADER,
        CACHE_HEADER,
        REVIEW_HEADER,
        SIGN_HEADER
    };
    
    /*
     * Help messages. Messages are indexed in the following order:
     * resourc, ca infos, uc groups, issuer, attribute dirs, time,
     * cache, review, and sign message.
     */
    static final String [] helpMess = {
        // Resource Frame Help message
        "The resouce can be hierarchical or flat. " +
        "The relative resource name is appended to the base url " +
        "which the generator uses to find information about this resource. " +
        "\n\nA base url, different from the default one, " +
        "can be chosen using the Create button, or be typed in. " +
        "If you are not sure of the correct relative resource name " +
        "clicking on the Browse button will give you a file browse window  " +
        "that server controls. The Policy Definition Server returns the contents of the " +
        "authorization file (which lists the stakeholders for the resource) " +
        "and the resource attributes file which contains the attributes, " +
        "values, and actions that are defined for the resource.\n\n" +
        "The host and port constitute the address of the Policy Definition Server." +
        "\nAll default values are read from the AkentiGen.conf file.",
        
        // CA Info Frame Help message
        "Provide trusted certificate authority information including an X509 " +
        "chain of the trusted CAs, at least one ID directory " +
        "An x509 chain consists of one or more x509 certificates. " +
        "An ID directory is the directory in which the CA " +
        "publishes the certificates that it issues.",
        
        // UC groups help message
        "If there was a Policy Certificate specified previously, the list " +
        "of UseCondition Issuer Groups from it, is listed in this frame. " +
        "You can select any groups and press the Remove Group button to remove it " +
        "or edit it by press the View Group button.  If you wish to add more " +
        "UseCondition Issuers, press the Create Group button.\n\nThe Policy " +
        "Certificate you are creating has to contain at least one " +
        "UseCondition Issuer Group.",
        
        // Issuer help message
        "If you are not in this list go back and add yourself as an " +
        "UseCondition issuer.",
        
        // Attribute Directory help message
        "For subordinate resources these directories are added to the list " +
        "of directories that are searched for attribute certificates",
        
        // TimeFrame help message
        "The certificate valdity period with begin with the current time" +
        "and end after curent time plus the duration chosen here.",
        
        // CacheFrame help message
        "Enter the maximum time in seconds that any certificate used by " +
        "this Policy should be cached.",
        
        // ReviewFrame help message
        "At this point you can use the Back button to go back to any " +
        "previous window where you can re-enter any information that "+
        "you may want to correct.",
        
        // Sign and Save frame help message
        "Your keyfile has to be a pkcs (.p12) file. The Certificate file " +
        "should be located in a directory that is accessible by the akenti server."
    };
}
