package lbl.akenti.policyManager.widgets.uc;

import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JCheckBox;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JScrollPane;
import javax.swing.DefaultListModel;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.Color;

import lbl.akenti.policyManager.widgets.LBLTopFrame;
import lbl.akenti.policyManager.widgets.LBLPanel;
import lbl.akenti.policyManager.widgets.LBLDialogFactory;

/**
 * Displays three main components:
 *
 *   1) JComboBox with two values "local" and "sub-tree"
 *
 *   2) JCheckBox  to set <i>Enable Access</i>
 *
 *   3) JList to hold actions and two buttons
 *      to add and remove actions.
 */
public class UCScopePanel
extends LBLPanel
implements ListSelectionListener, ListDataListener, ActionListener {
    
    private JComboBox scopeBox;
    private JCheckBox checkBox;
    private JList list;
    private JButton remButton;
    
    /**
     * Constructor
     * Sets up the frame with an empty list of actions
     *
     * @param title title of the frame
     * @param instructions header/instructions to be displayed at top of frame
     */
    public UCScopePanel(LBLTopFrame parent, String header) {
        super(parent, header);
        
        
        JPanel panel = new JPanel(new BorderLayout(5, 5));
        JPanel temp = new JPanel();
        
        temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
        temp.add(Box.createHorizontalGlue());
        
        JLabel label = new JLabel("Scope");
        
        label.setForeground(Color.red);
        temp.add(label);
        temp.add(Box.createHorizontalStrut(5));
        scopeBox = new JComboBox();
        scopeBox.addItem("local");
        scopeBox.addItem("sub-tree");
        temp.add(scopeBox);
        temp.add(Box.createHorizontalStrut(40));
        checkBox = new JCheckBox("Critical");
        checkBox.setForeground(Color.red);
        temp.add(checkBox);
        temp.add(Box.createHorizontalGlue());
        panel.add(temp, "North");
        
        temp = new JPanel(new BorderLayout(5, 5));
        
        label = new JLabel("Actions");
        label.setForeground(Color.red);
        temp.add(label, "North");
        
        DefaultListModel model = new DefaultListModel();
        
        model.addListDataListener(this);
        
        list = new JList(model);
        list.setSelectionMode(2);
        list.addListSelectionListener(this);
        temp.add(new JScrollPane(list), "Center");
        
        JButton addButton = new JButton("Add");
        addButton.setActionCommand("add");
        addButton.addActionListener(this);
        enableBlueButton(addButton);
        
        remButton = new JButton("Remove");
        remButton.setActionCommand("remove");
        remButton.addActionListener(this);
        panel.add(temp, "Center");
        
        add(
        wrapComponents(
        panel,
        wrapVertical(addButton, remButton)
        ),
        "Center"
        );
        
        parent.getButton("Next").setVisible(true);
        doEnable();
    }
    
    /**
     */
    public void setActions(String []actions) {
        DefaultListModel model = (DefaultListModel) list.getModel();
        
        model.clear();
        
        addActions(actions);
    }
    
    /**
     */
    public int addActions(String[] actions) {
        DefaultListModel model = (DefaultListModel) list.getModel();
        int added = 0;
        int size = model.getSize();
        
        list.clearSelection();
        
        for (int i = 0; i < actions.length; ++i) {
            int index = model.indexOf(actions[i]);
            
            if (index != -1) {
                list.addSelectionInterval(index, index);
            } else {
                model.addElement(actions[i]);
                list.addSelectionInterval(size, size++);
                ++added;
            }
        }
        
        return added;
    }
    
    /**
     */
    public int addAction(String action) {
        return addActions(new String[] { action });
    }
    
    /**
     */
    public boolean isScopeLocal() {
        return scopeBox.getSelectedItem().equals("local");
    }
    
    /**
     */
    public String[] getActions() {
        DefaultListModel model = (DefaultListModel) list.getModel();
        String[] actions = new String[model.size()];
        
        model.copyInto(actions);
        return actions;
    }
    
    /**
     */
    public void actionPerformed(ActionEvent event) {
        if (event.getActionCommand().equals("add")) {
            String action = LBLDialogFactory.showInputDialog(
            this,
            "Action Panel",
            "<html><font color=blue>Specify an action"
            );
            
            if (action != null && action.length() != 0) {
                addAction(action);
            }
        } else { // REMOVE
            int []indices = list.getSelectedIndices();
            DefaultListModel model = (DefaultListModel) list.getModel();
            
            for (int i = indices.length - 1; i > -1; --i) {
                model.removeElementAt(indices[i]);
            }
        }
    }
    
    /**
     */
    public void valueChanged(ListSelectionEvent e) {
        doEnable();
    }
    
    /**
     */
    public void contentsChanged(ListDataEvent e) {
        doEnable();
    }
    
    /**
     */
    public void intervalAdded(ListDataEvent e) {
        doEnable();
    }
    
    /**
     */
    public void intervalRemoved(ListDataEvent e)  {
        //    list.clearSelection();
        doEnable();
    }
    
    private void doEnable() {
        if (list.getSelectedIndex() != -1) {
            enableRedButton(remButton);
        } else {
            disableRedButton(remButton);
        }
    }
    
    /**
     */
    public boolean getCritical() {
        return checkBox.isSelected();
    }
    
    /**
     * Clears all components of the frame.
     * Called when Cancel is clicked.
     */
    public void clear(){
        DefaultListModel model = (DefaultListModel) list.getModel();
        
        model.clear();
        checkBox.setSelected(false);
        scopeBox.setSelectedIndex(0);
        doEnable();
    }
}
