package lbl.akenti.remotePolicyEngine;

import java.io.IOException;
import java.net.URL;
import java.util.Vector;
import lbl.akenti.messaging.AkentiRequest;
import lbl.akenti.messaging.AkentiResponse;
import lbl.akenti.messaging.AkentiResponses;
import lbl.akenti.messaging.CheckAccessRequest;
import lbl.akenti.messaging.CheckAccessResponse;
import lbl.akenti.messaging.CheckPolicyRequest;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.messaging.CreateResourceRequest;
import lbl.akenti.messaging.CreateResourceResponse;
import lbl.akenti.messaging.ResourceBrowseRequest;
import lbl.akenti.messaging.ResourceBrowseResponse;
import lbl.akenti.messaging.UploadPolicyCertificateRequest;
import lbl.akenti.messaging.UploadPolicyCertificateResponse;
import org.apache.soap.Constants;
import org.apache.soap.SOAPException;
import org.apache.soap.encoding.SOAPMappingRegistry;
import org.apache.soap.rpc.Call;
import org.apache.soap.rpc.Parameter;
import org.apache.soap.rpc.Response;
import org.apache.soap.util.xml.QName;

public class 
RemotePolicyEngine {

  /**
    */
  public RemotePolicyEngine() {
  }

  /**
    */
  public CheckAccessResponse 
  process(URL url, CheckAccessRequest req) throws IOException {
    return (CheckAccessResponse) process(url, (AkentiRequest) req);
  }

  /**
    */
  public CheckPolicyResponse 
  process(URL url, CheckPolicyRequest req) throws IOException {
    return (CheckPolicyResponse) process(url, (AkentiRequest) req);
  }

  /**
    */
  public ResourceBrowseResponse 
  process(URL url, ResourceBrowseRequest req) throws IOException {
    return (ResourceBrowseResponse) process(url, (AkentiRequest) req);
  }

  /**
    */
  public CreateResourceResponse 
  process(URL url, CreateResourceRequest req) throws IOException {
    return (CreateResourceResponse) process(url, (AkentiRequest) req);
  }

  /**
    */
  public UploadPolicyCertificateResponse 
  process(URL url, UploadPolicyCertificateRequest req) throws IOException {
    return (UploadPolicyCertificateResponse) process(url, (AkentiRequest) req);
  }

  /**
    */
  public AkentiResponse 
  process(URL url, AkentiRequest req) throws IOException {
    return process(url, new AkentiRequest[] { req })[0]; 
  }

  /**
    */
  public AkentiResponse[] 
  process(URL url, AkentiRequest[] requests) throws IOException {
try {
    if (requests.length == 0) {
       throw new RuntimeException();
    }

    Call call = new Call();

    call.setTargetObjectURI("http://dsd.lbl.gov/Akenti/2004/05/AkentiProtocol");
    call.setMethodName("AkentiRequests");
    call.setSOAPMappingRegistry(getRegistry());

    Vector params = new Vector();

    for (int i = 0; i < requests.length; ++i) {
       Parameter param = new Parameter("", AkentiRequest.class , requests[i], Constants.NS_URI_SOAP_ENC);

       params.addElement(param);
    }

    call.setParams(params);

    Response resp = call.invoke(url, "");

    if (resp.generatedFault()) {
      throw new IOException(resp.getFault().toString());
    } 

    Parameter result = resp.getReturnValue();
    AkentiResponses responsesList = (AkentiResponses) result.getValue();
    return responsesList.toAkentiResponsesArray();
} catch(SOAPException ex) {
    throw new IOException(ex.toString());
}
  }

  private static int id = 0;

  public static int getID() {
    int temp;

    synchronized(RemotePolicyEngine.class) {
      temp = id++;
    }
    
    return temp;
  }
  
  private static SOAPMappingRegistry smr;

  protected SOAPMappingRegistry getRegistry() {
    if (smr == null) {
      synchronized(RemotePolicyEngine.class) {
        if (smr == null) {
          smr = new SOAPMappingRegistry();

          AkentiMessagingSerializer srl = new AkentiMessagingSerializer();
          QName qn1 = new QName("", "");
 
          smr.mapTypes(Constants.NS_URI_SOAP_ENC, qn1, AkentiRequest.class, srl, null);

          QName qn2 = new QName("http://dsd.lbl.gov/Akenti/2004/05/AkentiProtocol", "akp:AkentiResponses");

          smr.mapTypes(Constants.NS_URI_SOAP_ENC, qn2, AkentiResponses.class, srl, srl);

        } 
      }
    }

    return smr;
  }
}
