package lbl.akenti.servers.AKMonitor.gui;

import java.util.EventObject;

/*
 * This event is delivered to a <code>ConnectListener</code>
 * to report the status of a connection.
 */
public class ConnectionEvent extends EventObject {
  /* error strings */
  public static final String errors[] = {
    "Server not responding",
    "Connection timed out",
    "Fatal I/O Error",
    "Unexpected Error"
  };

  /* codes */
  public static final int 
    NO_RESPONSE = 0,           /* Server not responding */
    TIMED_OUT = 1,             /* Connection timed out  */
    IO_ERROR = 2,              /* Input/Output error    */
    RUNTIME_ERROR = 3,         /* Unexcpected Error     */
    CONNECTED = 4,             /* Connection succeded   */
    DISCONNECTED = 5;          /* Disconnected          */
  
  /*
   * state of the connection 
   * connected if code == CONNECTED.
   * disconnected otherwise.
   */
  private int code;

  private String description;

  /*
   * Conteructor.
   *
   **/
  public ConnectionEvent(Object source, int code) {
    super(source);
    
    if (code < NO_RESPONSE || code > DISCONNECTED) {
      throw new IllegalArgumentException("code not supported");
    }

    this.code = code;
    this.description = "";
  }
  
  public ConnectionEvent(Object source, int code, String description) {
    this(source, code);
    this.description = description;
  }

  /*
   * @return true if connected.
   *         false otherwise
   **/
  public boolean isConnected() {
    return code == CONNECTED;
  }

  /*
   * @return code;
   */ 
  public int getCode() {
    return code;
  }
  
  /*
   * return an empty string if code == OK
   *        the error corresponding to the code otherwise.
   */
  public String getError() {
    if (code >= CONNECTED) {
      return "";
    }
    
    return description.equals("") ? errors[code] : errors[code] + description;
  }
}
