package lbl.akenti.servers.AKMonitor.gui;

/*
 * These are events that correspond to logs generated.
 * by the access control mechanism. 
 *
 * Here we lazily build the internl structure since
 * most events may not be seen by the user.
 * 
 * Note: no synchronization.
 */
public class StatusEvent {
  static final int OK = 0, PENDING = 1, SUCCESS = 2, FAILURE = 3, DENIED = 4;

  private int status;
  private int level;
  private String name;
  private String log;
  private String rep;
  
  public StatusEvent(String rep) { 
    if (rep == null) {
      throw new IllegalArgumentException();
    }
    
    this.rep = rep;
    status = -1;
  }
  
  /*
   * Since this is the first item in the string representation
   * We lazily read the status and leave the rest.
   * 
   * This works well since the progress panel uses this 
   * method to show progress.
   *
   * @return status  boolean
   */
  public int getStatus() {
    if (status == -1) {
      int index = rep.indexOf("#");
      String temp = rep.substring(0, index);
      status = Integer.parseInt(temp);
      rep = rep.substring(index + 2);
    }

    return status;
  }

  /*
   * Here we give up and just restore the internal state. 
   *
   * @return level 
   */
  public int getLevel() {
    if (rep != null) {
      toInternalForm();
      rep = null;
    }

    return level;
  }
  
  /*
   * Here we give up and just restore the internal state. 
   * 
   * @return name 
   */
  public String getName() {
    if (rep != null) {
      toInternalForm();
      rep = null;
    }

    return name;
  }

  /*
   * Here we give up and just restore the internal state. 
   * 
   * @return log. 
   */
  public String toString() {
    if (rep != null) {
      toInternalForm();
      rep = null;
    }

    return log;
  }
  
  /*
   * build internal structure.
   */
  protected void toInternalForm() {
    String temp;
    int index;

    index = rep.indexOf("#");

    if (status == -1) {
      temp = rep.substring(0, index);
      status = Integer.parseInt(temp);
      rep = rep.substring(index + 2);
    }

    temp = rep.substring(0, index);
    level = Integer.parseInt(temp);
    
    /* while  waiting to settle this issue */
    if (level == 3 && status == FAILURE) {
      status = DENIED;
    }
 
    rep = rep.substring(index + 2);
    index = rep.indexOf("#");
    name = rep.substring(0, index);
    log = rep.substring(index + 2);
  }
}
