package lbl.akenti.servers.AKMonitor.server;

import lbl.akenti.servers.AKMonitor.util.KeyFile;
import lbl.akenti.servers.AKMonitor.util.AppletConnection;

import java.util.Properties;
import java.util.StringTokenizer;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.io.DataInputStream;
import java.io.IOException;

import java.net.Socket;
import java.net.ServerSocket;


/**
  * Main class to start the Monitor Server
  * @author Guillaume Farret
  * @version August 2001
  */
public class Monitor extends Thread {

  private ServerSocket appletSSocket;
  private ServerSocket logSSocket;

/**
  * Constructor
  */
  private Monitor(int logPort, int appletPort) throws IOException {
    appletSSocket = new ServerSocket(appletPort);
    logSSocket = new ServerSocket(logPort); 
  }

/**
  * main method;
  * use: java Monitor configFile
  */  
  public static void main (String[] args) {
    Monitor monitor;
    try {
      
      Properties akentiProp = new Properties();

      if (args.length == 1) {
/*
        System.err.println("need a configuration file");
        System.err.println("use: java Monitor configFileName");
        System.exit(1);
*/

        System.err.println("Setting default Parameters....");
        akentiProp.load(new FileInputStream(args[0]));
      }

      String logFileName = akentiProp.getProperty("MonitorLogFile","Akenti.log");
      System.err.println("logging to file: "+logFileName);

      String keyFileName = akentiProp.getProperty("KeyFile","MonitorKeyWords");
      System.err.println("using keyFile: " + keyFileName);

      int appletPort = Integer.parseInt(akentiProp.getProperty("MonitorAppletPort","11111"));
      System.err.println("applet connection port: "+appletPort);

      int logPort = Integer.parseInt(akentiProp.getProperty("MonitorPort","11112"));
      System.err.println("log connection port: "+logPort);

      System.setOut(new PrintStream( new FileOutputStream(logFileName)));
      
      //initialization of the keyfile

      System.err.print("initializing KeyFile from: " + keyFileName + " ... ");
      System.err.print("initializing KeyFile from: " + keyFileName + " ... ");

      KeyFile.initKeyFile(keyFileName);

      System.err.println("success");
      System.err.println();

      monitor = new Monitor(logPort,appletPort);
      monitor.startAppletServer();
      monitor.startLogServer();

      //garbage collector for the AppletConnection Hashtable
      // XXX ConnectionHandler.startGarbageCollector();

    } catch (Exception e) {
      System.err.println("Initialization failed: " + e.getMessage());
      System.exit(1);
    }
  }

/**
  * start the server that listen to Applet connection
  */
  private void startAppletServer() {
    new Thread() {
      public void run() {
        System.err.println("listening to APPLET connection ...");
        while (true) {
          try { 
            Socket so = appletSSocket.accept();
            
            AppletConnection ac = new AppletConnection(so);
            DataInputStream in = ac.getDataInputStream();
            byte[] buffer = new byte[in.readInt()];
            in.readFully(buffer);
            StringTokenizer st = new StringTokenizer(new String(buffer),"=");
            st.nextToken();
            String key = st.nextToken().trim();

            System.err.println("new APPLET connection with UID:" + key);
            if (!ConnectionHandler.storeAppletConnection(key,ac)) {
              System.err.println("max user limit exceeded for this LOG");
            }
          } catch (IOException e) {
            System.err.println("applet server failure\n" + e.getMessage());
            e.printStackTrace(System.err);//debug
          }
        }
      }
    }.start();
  }

/**
  * start the server that listen to Log connection
  */
  private void startLogServer() {
    new Thread() {
      public void run() {
        System.err.println("listening to LOG connection ...");
        while (true) {
          try {
            Socket so = logSSocket.accept();
            System.err.println("new LOG connection");
            ConnectionHandler co = new ConnectionHandler(so.getInputStream());
            co.start();
          } catch (IOException e) {
            System.err.println(e.getMessage());
          } catch (Exception e) {
            System.err.println("log server accept failure\n" + e.getMessage());
            e.printStackTrace(System.err);//debug
          }
        }
      }
    }.start();
  }
}
