package lbl.akenti.servers.AKMonitor.util;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;

import java.net.Socket;

/**
  * Store the information on a applet connection
  * @author Guillaume Farret
  * @version August 2001
  */
public class AppletConnection {
  private Socket sock;
  private long lastTimeUsed;
  private DataOutputStream out;
  private int used = 0;

/**
  * Constructor
  */  
  public AppletConnection (Socket so) throws IOException {
    sock = so;
    out = new DataOutputStream(new BufferedOutputStream(sock.getOutputStream()));
    lastTimeUsed = System.currentTimeMillis();
  }

/**
  * sock getter
  */
  public Socket getSocket () {
    return sock;
  }

/**
  * out getter
  */
  public DataOutputStream getDataOutputStream() {
    return out;
  }

/**
  * return a DataInputStream attached with the Socket sock
  */
  public DataInputStream getDataInputStream() throws IOException {
    DataInputStream ret = new DataInputStream(sock.getInputStream());
    return ret;
  }

/**
  * lastTimeUsed getter
  */
  public long getLastTimeUsed () {
    return lastTimeUsed;
  }

/**
  * used getter
  */
  public int getUsed () {
    return used;
  }
  
/**
  * sort of semaphore to know if this Object is currently used
  * each time someone want to use an instance of this object, he has
  * to do a P() on it, then when he has finish to use it, he does;
  * a V(), this way 'used' give us how many thread are currently using
  * this instance
  */
  public synchronized void P() {
    used++;
  }

  public synchronized void V() {
    if (used > 0) used--;
    if (used == 0) lastTimeUsed = System.currentTimeMillis();
  }
}
