package lbl.akenti.servers.CacheManager;

import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;

/**
  *
  **/
public class CacheCertificate {
  
  String cert;

  UtcTime notAfter;

  public CacheCertificate() {
    this(new String(), new UtcTime());
  }

  public CacheCertificate(String cert, UtcTime notAfter) {
    if (cert == null || notAfter == null) {
      throw new IllegalArgumentException();
    }
    
    this.cert = cert;
    this.notAfter = notAfter;
  }
  
  /**
    */
  public UtcTime getNotAfter() {
    return notAfter;
  }

  /**
    */
  public String getCertificate() {
    return cert;
  }
  
  /**
    */
  public boolean checkValidity() {
    return checkValidity(new UtcTime());
  }

  /**
    */
  public boolean checkValidity(UtcTime utc) {
    return notAfter.compareTo(utc) >= 0;
  }
  
  /**
    */
  public void writeToFile(String fileName) throws IOException {
    writeToFile(new File(fileName));
  }
  
  public void writeToFile(File file) throws IOException {
    FileOutputStream fos = new FileOutputStream(file);
    BufferedOutputStream bos = new BufferedOutputStream(fos);
     
    bos.write(notAfter.toString().getBytes());
    bos.write('\n');
    bos.write(cert.getBytes());
    bos.close();
  }
  
  /**
    */
  public void readFromFile(String fileName) throws IOException , ParseException {
    readFromFile(new File(fileName));
  }

  public void readFromFile(File file) throws IOException, ParseException  {
    FileInputStream fis = new FileInputStream(file);
    BufferedReader reader = new BufferedReader(new InputStreamReader(fis));
    String temp = reader.readLine();

    notAfter = new UtcTime(temp);

    int len = (int) (file.length() - temp.length() - 1);
    char []chars = new char[len];

    reader.read(chars, 0, len);
    cert = new String(chars);
    reader.close();
  }
}
