
package lbl.akenti.test.certs;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

import java.security.PrivateKey; 
import java.security.cert.X509Certificate;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.text.ParseException;
import java.security.cert.Certificate;
import java.security.PublicKey;
import java.security.GeneralSecurityException;

import lbl.akenti.util.UTCTime;
import lbl.akenti.util.P12Reader;
import lbl.akenti.util.X509CertReader;
import lbl.akenti.certs.PolicyCertGenerator;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.UseCondIssuerGroup;

public class PolicyCertGeneratorTestCase extends TestCase {

  /**
    * constructor
    */
  public PolicyCertGeneratorTestCase(String name) {
    super(name);
  }

  /**
    * Test Suite.
    */
  public static Test suite() {
    return new TestSuite(PolicyCertGeneratorTestCase.class);
  }

  public void 
  testGeneratePolicyCertificate() 
    throws ParseException, IOException, GeneralSecurityException {
    UTCTime now = new UTCTime();
    UTCTime after = new UTCTime(now, 365 * 24 * 60 * 60);
    DistinguishedName issuer = new DistinguishedName("/C=US/OU=LBL/CN=Stake Holder I");
    DistinguishedName ca = new DistinguishedName("/C=US/OU=LBL/CN=TEST CA I");
    AkentiPrincipal guarantor = new AkentiPrincipal(issuer, ca);
    Directory attrDirectory = new Directory("file:/test1/ATTRDIRS");
    Directory[] attrDirectories = { attrDirectory };
    AkentiPrincipal[] ucIssuers = { guarantor };
    Directory ucDirectory = new Directory("file:/test1/UCDIRS");
    Directory[] ucDirectories = { ucDirectory };
    UseCondIssuerGroup group = new UseCondIssuerGroup(ucIssuers, ucDirectories);
    UseCondIssuerGroup[] groups = { group };
    X509CertReader xcr = new X509CertReader();
    X509Certificate[] caCerts = xcr.readX509Certs("testCAI.pem");
    Directory idDirectory = new Directory("file:/test1/IDDIRS");
    Directory[] idDirectories = { idDirectory };
    CertificateAuthorityInfo caInfo = new CertificateAuthorityInfo(caCerts, idDirectories);
    CertificateAuthorityInfo[] caInfos = { caInfo }; 
    PolicyCertGenerator gen = new PolicyCertGenerator();

    gen.setVersionNumber(2);
    gen.setUniqueID("SomeUniqueId");
    gen.setNotBefore(now);
    gen.setNotAfter(after);
    gen.setIssuer(new AkentiIssuer(guarantor));
    gen.setResource("RESOURCE/SUB_RESOURCE_I");
    gen.setCertificateAuthorityInfos(caInfos);
    gen.setUseCondIssuerGroups(groups);
    gen.setAttributeDirectories(attrDirectories);
    gen.setCacheTime(3600); 

    P12Reader p12 = new P12Reader();
    p12.readP12File("testCAI.p12", "testing");
    PrivateKey privateKey = p12.getPrivateKey();

    gen.sign("MD5withRSA", privateKey);

    PolicyCertificate polCert = (PolicyCertificate) gen.generate();
    Certificate cert = p12.getX509Certificate();
    PublicKey publicKey = cert.getPublicKey();

    assertTrue(polCert.verify(publicKey));

    FileOutputStream fos = new FileOutputStream("policyCert.xml");
    AkentiCertificateFactory.writeCertToStream("XML", polCert, fos);
    fos.close();

    FileInputStream fis = new FileInputStream("policyCert.xml");
    AkentiCertificate tempPolCert = AkentiCertificateFactory.initCertFromStream(
      "XML", fis
    );

    fis.close();     
    assertTrue(tempPolCert.verify(publicKey));
    fos = new FileOutputStream("policyCert.pem");
    AkentiCertificateFactory.writeCertToStream("PEM", tempPolCert, fos);
    fis = new FileInputStream("policyCert.pem");   
    tempPolCert = AkentiCertificateFactory.initCertFromStream(
      "PEM", fis
    );
    assertTrue(tempPolCert.verify(publicKey));
  }
}
