package lbl.akenti.test.certs;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.PublicKey;
import java.security.cert.Certificate;
import java.security.PrivateKey; 
import java.security.GeneralSecurityException; 
import java.text.ParseException;

import lbl.akenti.util.UTCTime;
import lbl.akenti.util.P12Reader;
import lbl.akenti.certs.UseCondCertGenerator;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.Condition;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;

public class UseCondCertGeneratorTestCase extends TestCase {

  /**
    * constructor
    */
  public UseCondCertGeneratorTestCase(String name) {
    super(name);
  }

  /**
    * Test Suite.
    */
  public static Test suite() {
    return new TestSuite(UseCondCertGeneratorTestCase.class);
  }

  /**
    *
    */
  public void 
  testGenerateUseCondCertGenerator() 
    throws ParseException, IOException, GeneralSecurityException {

    UTCTime now = new UTCTime();
    UTCTime after = new UTCTime(now, 365 * 24 * 60 * 60);
    DistinguishedName issuer = new DistinguishedName(
      "/C=US/OU=LBL/CN=Stake Holder I"
    );
    DistinguishedName ca = new DistinguishedName("/C=US/OU=LBL/CN=TEST CA I");
    AkentiPrincipal guarantor = new AkentiPrincipal(issuer, ca);
    DistinguishedName[] cas = { ca };
    AkentiPrincipal[] attrIssuers = { guarantor };
    AttributeInfo at1 = new AttributeInfo("OU", "LBL", cas);
    AttributeInfo at2 = new AttributeInfo(
      "JOB", 
      "SUPERVISOR", 
      attrIssuers, 
      new Directory[]{ new Directory("file:/home/g1/users") }
    );

    AttributeInfo[] attrInfos = { at1, at2 };
    Condition condition = new Condition("OU = LBL && JOB = SUPERVISOR", attrInfos);
    String[] actions = { "READ" };

    UseCondCertGenerator gen = new UseCondCertGenerator();

    gen.setVersionNumber(2);
    gen.setUniqueID("I");
    gen.setNotBefore(now);
    gen.setNotAfter(after);
    gen.setIssuer(new AkentiIssuer(guarantor));
    gen.setResource("RESOURCE");
    gen.setCondition(condition);
    gen.setLocalScope(false);
    gen.setCritical(true);
    gen.setActions(actions); 

    P12Reader p12 = new P12Reader();
    p12.readP12File("testCAI.p12", "testing");

    PrivateKey privateKey = p12.getPrivateKey();

    gen.sign("MD5withRSA", privateKey);

    UseCondCertificate ucCert = (UseCondCertificate) gen.generate();

    Certificate cert = p12.getX509Certificate();
    PublicKey publicKey = cert.getPublicKey();

    assertTrue(ucCert.verify(publicKey));

    FileOutputStream fos = new FileOutputStream("ucCert.xml");

    AkentiCertificateFactory.writeCertToStream("XML", ucCert, fos);
    fos.close();

    FileInputStream fis = new FileInputStream("ucCert.xml");
    AkentiCertificate tempUcCert = AkentiCertificateFactory.initCertFromStream("XML", fis);

    fis.close();     

    assertTrue(tempUcCert.verify(publicKey));

    fos = new FileOutputStream("ucCert.pem");
    AkentiCertificateFactory.writeCertToStream("PEM", tempUcCert, fos);
    fos.close();
    fis = new FileInputStream("ucCert.pem");
    tempUcCert = AkentiCertificateFactory.initCertFromStream("PEM", fis);
    fis.close();
    assertTrue(tempUcCert.verify(publicKey));
  }
}    
