package lbl.akenti.test.certs.elements;


import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;

import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.ParseException;

import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.Directory;

public class AttributeInfoTestCase extends TestCase {

  /**
    *
    */
  public AttributeInfoTestCase(String name) {
    super(name);
  }  

  /**
    *
    */
  public static Test suite() {
    return new TestSuite(AttributeInfoTestCase.class);
  }

  /**
    *  Tests the AttributeInfo object that is instantiated with the Attribute,
    *  and value.
    */
  public void testAttributeInfo() 
    throws IOException, ParseException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    AkentiOutputStream aos = new AkentiOutputStream(baos);
    AttributeInfo ai1 = new AttributeInfo("MEM", "!0 MG");
    DistinguishedName ca = new DistinguishedName(
      "/C=US/O=LBNL/OU=DSD/CN=CA I"
    );      

    DistinguishedName dn = new DistinguishedName(
      "/C=US/O=LBNL/OU=DSD/CN=User I"
    );      

    DistinguishedName[] cas = { ca };
    AttributeInfo ai2 = new AttributeInfo("OU", "LBL", cas);

    AkentiPrincipal ap = new AkentiPrincipal(dn, ca);
    AkentiPrincipal[] aps = { ap };
    Directory dir = new Directory("file:/home/joe"); 
    Directory[] dirs = { dir };

    AttributeInfo ai3 = new AttributeInfo("JOB", "SUPERVISOR", aps, dirs);

    ai1.writeObject(aos);
    baos.write(' ');
    ai2.writeObject(aos);
    baos.write(' ');
    ai3.writeObject(aos);
    aos.close(); 

    ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
    AkentiInputStream ais = new AkentiInputStream(bais);
    AttributeInfo temp = new AttributeInfo(ais);

    assertTrue(temp.equals(ai1));
    temp = new AttributeInfo(ais);
    assertTrue(temp.equals(ai2));
    temp = new AttributeInfo(ais);
    assertTrue(temp.equals(ai3));
  }   
}
