package lbl.akenti.test.messaging;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.messaging.AkentiMessage;
import lbl.akenti.messaging.AkentiRequest;
import lbl.akenti.messaging.AkentiRequests;
import lbl.akenti.messaging.AkentiResponse;
import lbl.akenti.messaging.AkentiResponses;
import lbl.akenti.messaging.CheckAccessRequest;
import lbl.akenti.messaging.CheckAccessResponse;
import lbl.akenti.messaging.CheckPolicyRequest;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.messaging.CreateResourceRequest;
import lbl.akenti.messaging.CreateResourceResponse;
import lbl.akenti.messaging.ResourceBrowseRequest;
import lbl.akenti.messaging.ResourceBrowseResponse;



/**
  * Test the messaging elements.
  */
public class MessagingTestCase extends TestCase {

  private static boolean VERBOSE = false;

  /**
    */
  public MessagingTestCase(String name) {
    super(name);
  }  

  /**
    *
    */
  public static Test suite() {
    return new TestSuite(MessagingTestCase.class);
  }

  /**
    */
  public void test1() throws Exception {
    AkentiPrincipal ap = new AkentiPrincipal("/CN=Joe", "/CN=Ca");
    CheckAccessRequest careq = new CheckAccessRequest(1, ap, "R1");
    AkentiRequests areqs = new AkentiRequests(careq);
    ByteArrayOutputStream baos = new ByteArrayOutputStream();

    areqs.toStream(baos);

    String out1 = new String(baos.toByteArray());
    ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());

    areqs = new AkentiRequests(bais);
    baos = new ByteArrayOutputStream();
    areqs.toStream(baos);

    String out2 = new String(baos.toByteArray());

    assertTrue(out1.equals(out2));

    if (VERBOSE) {
      areqs.toStream(System.out);
    }
  }

  public void test2() throws Exception {
    AkentiPrincipal ap = new AkentiPrincipal("/CN=Joe", "/CN=Ca");

    CheckAccessRequest req1 = new CheckAccessRequest(2, ap, "R1/S1");
    CheckPolicyRequest req2 = new CheckPolicyRequest(3, "R1");
    ResourceBrowseRequest req3 = new ResourceBrowseRequest(4, "R1");
    CreateResourceRequest req4 = new CreateResourceRequest(5, "R1/S1");

    AkentiRequests areqs = new AkentiRequests(new AkentiRequest[] {req1, req2, req3, req4});
    ByteArrayOutputStream baos = new ByteArrayOutputStream();

    areqs.toStream(baos);

    String out1 = new String(baos.toByteArray());
    ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());

    areqs = new AkentiRequests(bais);
    baos = new ByteArrayOutputStream();
    areqs.toStream(baos);

    String out2 = new String(baos.toByteArray());

    assertTrue(out1.equals(out2));

    if (VERBOSE) {
      areqs.toStream(System.out);
    }
  }

  public void test3() throws Exception {
    AkentiMessage mesg1 = new AkentiMessage(301, "access denied");
    CheckAccessResponse resp1 = new CheckAccessResponse(1, mesg1);

    AkentiMessage mesg2 = new AkentiMessage(505, "no policy found");
    CheckPolicyResponse resp2 = new CheckPolicyResponse(2, mesg2);

    AkentiMessage mesg3 = new AkentiMessage(505, "no policy found");
    ResourceBrowseResponse resp3 = new ResourceBrowseResponse(3, mesg3, new String[] { "R1" } );

    AkentiMessage mesg4 = new AkentiMessage(505, "no policy found");
    CreateResourceResponse resp4 = new CreateResourceResponse(4, mesg4);

    AkentiResponses aresps = new AkentiResponses(new AkentiResponse[] {resp1, resp2, resp3, resp4});
    ByteArrayOutputStream baos = new ByteArrayOutputStream();

    aresps.toStream(baos);

    String out1 = new String(baos.toByteArray());
    ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());

    aresps = new AkentiResponses(bais);
    baos = new ByteArrayOutputStream();
    aresps.toStream(baos);

    String out2 = new String(baos.toByteArray());

    assertTrue(out1.equals(out2));

    if (VERBOSE) {
      aresps.toStream(System.out);
    }
  }
}
