/*
 * PrettyPrintAkentiCert.java
 *
 * Created on June 22, 2004, 1:03 PM
 */

package lbl.akenti.tools;

import java.util.Properties;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import util.Arguments;  // from xerces samples

/**
 * This command line program prints a human-readable version of a certificate.
 *
 * @author  Tommaso Sciortino
 */
public class PrettyPrintAkentiCert {
  
  /** Creates a new instance of PrettyPrintAkentiCert */
  public PrettyPrintAkentiCert() {
  }
  
  /**
   * @param args the command line arguments
   */
  public static void main(String[] args) {
    
    try {
      Properties props = System.getProperties();
      
      Arguments argopt = new Arguments();
      
      argopt.setUsage(
      new String[] {
        "\nusage: prettyPrintAkentiCert.sh -a akentiCert\n" +
        "options:\n" +
        "  -a   Akenti Certificate\n" +
        "  [-v] verbose\n" +
        "  [-h] prints this message\n"
      }
      );
      
      if (args.length == 0) {
        argopt.printUsage();
        System.exit(1);
      }
      
      argopt.parseArgumentTokens(args , new char[] {'a'} );
      
      String certFileName = null;
      boolean verbose = false;
      int c;
      while ((c = argopt.getArguments()) != -1) {
        switch (c) {
          case 'a':
            certFileName = argopt.getStringParameter();
            break;
          case 'v':
            verbose = true;
            break;
          case 'h':
          case '-':
          case '?':
            argopt.printUsage();
            System.exit(1);
          default:
            break;
        }
      }
      
      if (certFileName == null) {
        System.out.println("Must provide a certificate");
        argopt.printUsage();
        System.exit(1);
      }
      
      if (verbose) {
        System.out.println("Loading Certificate...");
      }
      AkentiCertificate cert = AkentiCertificateFactory.initCertFromFile(null,certFileName);
      
      if (verbose) {
        System.out.println("Certificate Loaded. Printing Certificate.");
      }      
      System.out.println(cert.toText());
      
    } catch(RuntimeException ex) {
      System.out.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
    } catch(Exception ex) {
      ex.printStackTrace();
    }
  }
}