/*
 * PrintAkentiCertTemplate.java
 *
 * Created on June 24, 2004, 1:08 PM
 */

package lbl.akenti.tools;

import java.io.OutputStream;
import java.util.Properties;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.elements.*;
import lbl.akenti.certs.*;
import lbl.akenti.util.UTCTime;
import util.Arguments;  // from xerces samples
import java.io.OutputStream;
import java.security.cert.X509Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;

/**
 * This command line program prints a template for a certificate.
 *
 * @author  Tommaso Sciortino
 */
public class PrintAkentiCertTemplate {
  
  /** Creates a new instance of PrintAkentiCertTemplate */
  public PrintAkentiCertTemplate() {
  }
  
  /**
   * @param args the command line arguments
   */
  public static void main(String[] args) {
    
    try {
      Properties props = System.getProperties();
      
      Arguments argopt = new Arguments();
      
      argopt.setUsage(
      new String[] {
        "\nusage: PrintAkentiCertTemplate -R\n" +
        "options:\n" +
        "  [-R]        - Root Policy certificate\n" +
        "  [-P]        - Policy certificate\n" +
        "  [-U]        - Use-Condition certificate\n" +
        "  [-A]        - Attribute certificate\n" +
        "  [-o]        - output file name\n" +
        "  [-v]        - verbose\n" +
        "  [-h]        - prints this message "
      }
      );
      
      if (args.length == 0) {
        argopt.printUsage();
        System.exit(1);
      }
      
      argopt.parseArgumentTokens(args , new char[] {'a'} );
      
      String outFileName = null;
      String type = null;
      boolean verbose = false;
      int c;
      while ((c = argopt.getArguments()) != -1) {
        switch (c) {
          case 'o':
            outFileName = argopt.getStringParameter();
            break;
          case 'R':
            type = "RootPolicy";
            break;
          case 'P':
            type = "Policy";
            break;
          case 'U':
            type = "UseCondition";
            break;
          case 'A':
            type = "Attribute";
            break;
          case 'v':
            verbose = true;
            break;
          case 'h':
          case '-':
          case '?':
            argopt.printUsage();
            System.exit(1);
          default:
            break;
        }
      }
      
      
      AkentiCertificate cert = null;
      
      if (type == "UseCondition") {
        UseCondCertGenerator gen = new UseCondCertGenerator();
        gen.setIssuer(new AkentiIssuer()); 
        String[] actions = {"read","write"};
        gen.setActions(actions);
        DistinguishedName[] dns = { new DistinguishedName() };
        AkentiPrincipal[] aps = { new AkentiPrincipal() };
        Directory[] dirs = { new Directory() };
        AttributeInfo[] ais = { new AttributeInfo("CN","NO_CN",aps,dirs),
                                new AttributeInfo("CN","NO_CN",dns),
                                new AttributeInfo("CN","NO_CN")};
        gen.setCondition(new Condition("GROUP = NO_GROUP_NAME  || CN = NO_CN && TIME > NO_TIME", ais));
        cert = gen.generate();
      } else if (type == "Attribute") {
        AttributeCertGenerator gen = new AttributeCertGenerator();
        gen.setIssuer(new AkentiIssuer());
        cert = gen.generate();
      } else if (type == "Policy") {
        PolicyCertGenerator gen = new PolicyCertGenerator();
        gen.setIssuer(new AkentiIssuer());
        UseCondIssuerGroup[] ucig = { new UseCondIssuerGroup() };
        gen.setUseCondIssuerGroups(ucig);
        cert = gen.generate();
      } else if (type == "RootPolicy") {
        PolicyCertGenerator gen = new PolicyCertGenerator();
        UseCondIssuerGroup[] ucig = { new UseCondIssuerGroup() };
        gen.setUseCondIssuerGroups(ucig);
        //gen.setCertificateAuthorityInfos();
        CertificateFactory cfac = CertificateFactory.getInstance("X.509");
    
        gen.setIssuer(new AkentiIssuer());  
        cert = gen.generate();
      } else {
        
      }
      
      
      AkentiCertificateFactory.writeXMLCertToStream(cert,System.out);
      
    } catch(RuntimeException ex) {
      System.out.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
    } catch(Exception ex) {
      ex.printStackTrace();
    }
  }
}