package lbl.akenti.tools;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import org.xml.sax.SAXParseException;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;


public class ValidateXML {

 /**
  * Constructor
  * Converts the String to a InputStreamReader and calls 
  * this(InputStreamReader). 
  * 
  * @param String name of file to be parsed
  */
  public ValidateXML(String filename) 
  throws IOException, FileNotFoundException, SAXException {
    this(new FileInputStream(filename));
  }
  
 /**
  * Constructor
  * Calls domParsing with an INputSource created from 
  * InputStreamReader.
  *
  * @param InputStreamReader of the file to be parsed
  */
  public ValidateXML(InputStream istream) throws IOException, SAXException {
    validate(new InputSource(istream));
  }

 /**
  * Parses the file using DOM parser, storing it in 
  * org.w3c.dom.Document tree.
  * Validates it using the give Schema.
  *
  * @param InputStreamReader of the file to be parsed
  */
  private void validate(InputSource isource) throws SAXException, IOException {
    org.apache.xerces.parsers.DOMParser parser = new org.apache.xerces.parsers.DOMParser();

    try {
      parser.setFeature("http://xml.org/sax/features/validation", true);
    } catch (SAXException ex) {
      System.out.println("[Warning] Error in setting up XML parser feature");
    }

    XMLErrorHandler handle = new XMLErrorHandler();
    parser.setErrorHandler(handle);
    parser.parse(isource);

    if(handle.success) {
      System.out.println("\nValidation successful\n");
    }
  }

  public static void main (String[] args) {
    if(args.length > 0) {
      String fileName = args[0];
      try {
        ValidateXML val = new ValidateXML(fileName);
      } catch(Exception ex) {
        ex.printStackTrace();
      }
    } else {
      System.err.println("\nNo file to validate\n");	
    }
  }
}
