package lbl.akenti.util;

import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Enumeration;
import java.net.URL;

import java.io.ByteArrayInputStream;
import java.security.cert.X509Certificate;
import java.security.cert.CertificateException;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.NamingEnumeration;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.directory.Attributes;
import javax.naming.directory.Attribute;

/**
  *
  * @author Keith Jackson
  * @author Abdelilah Essiari
  */ 
public class LDAPSearchUtil {

    /**
     * A public constant to control debugging level;
     */
    public static final boolean DEBUG = false;
    
    /**
     * A public constant to control the search scope.  Object scope
     * searches only the named object.
     */
    public static final int OBJECT_SCOPE = 0;

    /**
     * A public constant to control the search scope.  One level scope
     * searches all entries at the level of the named object.
     */
    public static final int ONELEVEL_SCOPE = 1;

    /**
     * A public constant to control the search scope.  Sub-tree scope
     * searches the entire sub-tree below the named object.
     */
    public static final int SUBTREE_SCOPE = 2;

    /**
     * A private constant to set the initial context.  This is what changes
     * if you wish to use a different LDAP provider.
     */
    private static final String INITCTX = "com.sun.jndi.ldap.LdapCtxFactory";

    /**
     * A private constant to set the authorization method used by the LDAP
     * server.  This would need to be changed to use something other then
     * simple password authentication.
     */
    private static final String AUTH_TYPE = "simple";

    public static String[] 
    loadOrgUnits(URL url) throws NamingException {
      return loadOrgUnits(url, "ou=*");
    }

    public static String[] 
    loadOrgUnits(URL url, String filter) throws NamingException {
      ArrayList results = find(url, filter, new String[] { "ou" });

      return (String[]) results.toArray(new String[0]);
    }

    public static String[] loadCns(URL url) throws NamingException {
      return loadCns(url, "cn=*");
    }

    public static String[] loadCns(URL url, String filter) 
      throws NamingException {
      ArrayList results = find(url, filter, new String[] { "cn" });

      return (String[]) results.toArray(new String[0]);
    }

    public static X509Certificate[] 
    loadX509Certs(URL url) throws NamingException {
      return loadX509Certs(url, "cn=*");
    }

    public static X509Certificate[] loadX509Certs(URL url, String filter) 
      throws NamingException {
      ArrayList results = find(url, filter, new String[] { "usercertificate" , "cacertificate" });
      X509Certificate[] certs = new X509Certificate[results.size()];
      X509CertReader reader = new X509CertReader();

      for (int i = 0; i < results.size(); ++i) {
        try {
          ByteArrayInputStream bais = new ByteArrayInputStream(
            (byte []) results.get(i)
          );

          certs[i] = reader.readX509Cert(bais);
        } catch(Exception ex) {
          throw new NamingException(ex.getMessage());
        }
      }

      return certs;
    }

    private static ArrayList 
    find(URL url, String filter, String[] attrs) 
      throws NamingException {
      Hashtable hTable = new Hashtable ();

      hTable.put (Context.INITIAL_CONTEXT_FACTORY, INITCTX);

      if (!url.getProtocol().equalsIgnoreCase("ldap")) {
        throw new RuntimeException();
      }

      StringBuffer sb = new StringBuffer("ldap://");

      sb.append(url.getHost());
      sb.append(":");
      
      int port = url.getPort();
 
      if (port == -1) {
        sb.append(389);
      } else {
        sb.append(port);
      }

      hTable.put(Context.PROVIDER_URL, sb.toString());
      hTable.put(Context.SECURITY_AUTHENTICATION, "simple");
      hTable.put(Context.SECURITY_PRINCIPAL, "");
      hTable.put(Context.SECURITY_CREDENTIALS, "");
      hTable.put(Context.REFERRAL, "follow");

      DirContext ctx = new InitialDirContext (hTable);
      SearchControls constraints = new SearchControls();

      // constraints.setSearchScope(ONELEVEL_SCOPE);
      constraints.setSearchScope(2);
      constraints.setReturningAttributes(attrs);

      ArrayList ret = new ArrayList();
     
      String base = url.getFile(); 

      if (base.startsWith("/")) {
        base = base.substring(1); 
      }
     
      NamingEnumeration results = ctx.search(base, filter, constraints);

      while (results != null && results.hasMore ()) {
        SearchResult sResult = (SearchResult) results.next();
        NamingEnumeration atrE = sResult.getAttributes().getAll(); 

        // XXX 
        if (atrE.hasMoreElements()) {
          Attribute attr = (Attribute) atrE.next();
          Enumeration vals = attr.getAll(); 

          while (vals.hasMoreElements()) {
            ret.add(vals.nextElement());
          }
        }
      }

      return ret;
    }
}
