package lbl.akenti.util;

import java.util.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.util.TimeZone;

/**
  * 
  */
public class UTCTime {

  public static String DEFAULT_FORMAT = "yyMMddHHmmss'Z'";
  public static String FRIENDLY_FORMAT = "E MMM dd HH:mm:ss z yyyy";
  public static String XS_FORMAT = "yyyy-MM-dd'T'HH:mm:ss'Z'";
  
  private Date date;

  /**
    */
  public UTCTime() {
    date = new Date();
  }

  /**
    */
  public UTCTime(long offset) {
    date = new Date();
    date.setTime(date.getTime() + offset);
  }

  /**
    *
    */
  public UTCTime(UTCTime utc, long offset) {
    date = new Date();
    date.setTime(utc.getTime() + offset);
  }

  /**
    *
    */
  public UTCTime(String rep) throws ParseException {
    String format;
    if (rep.length() > 13) {
      format = XS_FORMAT;
    } else {
      format = DEFAULT_FORMAT;
    }
    SimpleDateFormat sdf  = new SimpleDateFormat(format);

    sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
    date = sdf.parse(rep);
  }

  /**
    *
    */
  public UTCTime(String format, String rep) throws ParseException {
    SimpleDateFormat sdf  = new SimpleDateFormat(format);

    sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
    date = sdf.parse(rep);
  }

  /**
    *
    */
  public int compareTo(UTCTime utc) {
    long temp = (getTime() - utc.getTime()) / 1000;
    
    if (temp < 0) {
      return -1;
    } else if (temp == 0) {
      return 0;
    } else {
      return 1;
    }
  }

  /**
    */
  public long getTime() {
    return date.getTime();
  }

  /**
    * Converts to friendly string format .
    */
  public String toFriendlyString() {
    return toString(FRIENDLY_FORMAT);
  }

  /**
    */
  public String toString() {
    return toString(DEFAULT_FORMAT);
  }

  /**
    */
  public String toString(String format) {
    SimpleDateFormat sdf  = new SimpleDateFormat(format);
    
    sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
    return sdf.format(date);
  }
}
