package lbl.akenti.util;

import java.io.LineNumberReader;
import java.io.InputStreamReader;

import java.io.PrintWriter;
import java.io.OutputStreamWriter;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.ByteArrayInputStream;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;

import java.security.cert.X509Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;

/**
  * Reads x509 certificates.
  */
public class X509CertReader {

  public  X509CertReader() {
  }

  /**
    * Reads X509 certificate in pem encoding.
    */
  public X509Certificate readX509Cert(String filename) 
    throws IOException, CertificateException {
    return readX509Certs(filename)[0];
  }

  /**
    * Reads X509 certificates in pem encoding.
    */
  public X509Certificate[] readX509Certs(String filename) 
    throws IOException, CertificateException {
    return readX509Certs(filter(new FileInputStream(filename)));
  }

  /**
    * Reads X509 certificates in pem encoding.
    */
  public X509Certificate[] readX509Certs(URL url) 
    throws IOException, CertificateException {
    return readX509Certs(filter(url.openStream()));
  }

  /**
    * Reads X509 certificates in pem and der encoding.
    */
  public X509Certificate[] readX509Certs(InputStream in) 
    throws IOException, CertificateException {
    CertificateFactory cfac = CertificateFactory.getInstance("X.509");
    BufferedInputStream bis = new BufferedInputStream(in);
    ArrayList temp = new ArrayList();

    do {
     X509Certificate cert = (X509Certificate) cfac.generateCertificate(bis);
     
     temp.add(cert);
    } while (bis.available() > 0);

    X509Certificate[] certs = new X509Certificate[temp.size()];

    temp.toArray(certs);
    return certs;
  }

  /**
    * Reads X509 certificate in pem and der encoding.
    */
  public X509Certificate readX509Cert(InputStream in) 
    throws IOException, CertificateException {
    CertificateFactory cfac = CertificateFactory.getInstance("X.509");

    return (X509Certificate) cfac.generateCertificate(in);
  }
  
  /**
    * Removes all extra stuff.
    *
    * Works only for pem encoding.
    */
  private ByteArrayInputStream filter(InputStream in)
    throws IOException {
    InputStreamReader isr = new InputStreamReader(in);
    LineNumberReader lnr = new LineNumberReader(isr);
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStreamWriter osw = new OutputStreamWriter(baos);
    PrintWriter pw = new PrintWriter(osw);

    String line = lnr.readLine();

    while (line != null) {
      if (line.equals("-----BEGIN CERTIFICATE-----")) {
        pw.println(line);
        line = lnr.readLine();

        while (line != null) {
          pw.println(line);

          if (line.equals("-----END CERTIFICATE-----")) {
            line = lnr.readLine();
            break;
          }

          line = lnr.readLine();
        }
      } else {
        line = lnr.readLine();
      }
    }

    osw.flush();
    return new ByteArrayInputStream(baos.toByteArray());
  }
}  
