package lbl.akenti.util;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class XMLPreProcessor {

  private static XMLPreProcessor preProcessor = null;

  private XMLPreProcessor() {
  }

  public static XMLPreProcessor getInstance() {
    if (preProcessor == null) {
      preProcessor = new XMLPreProcessor();
    }

    return preProcessor;
  }

  public Document preProcess(Document doc) {
    traverse(doc);
    return doc;
  }

  private void traverse(Node node) {
    traverse(node, node);
  }

  private void traverse(Node pNode, Node node) {
    if (node == null) {
      return;
    }

    int type = node.getNodeType();

    switch (type) {

      case Node.DOCUMENT_NODE: {
        traverse(((Document)node).getDocumentElement(), 
	         ((Document)node).getDocumentElement());
        break;
      }

      case Node.ELEMENT_NODE: {
        NodeList children = node.getChildNodes();
        if (children != null) {
          for (int i = 0, n=children.getLength(); i < n; i++) {
            traverse(node, children.item(i));
          }
        }
        break;
      }

      case Node.ENTITY_REFERENCE_NODE: {
        NodeList children = node.getChildNodes();
        if (children != null) {
          for (int i = 0, len = children.getLength(); i < len; i++) {
            traverse(node, children.item(i));
          }
        }
        break;
      }

      case Node.CDATA_SECTION_NODE: {
        break;
      }

      case Node.TEXT_NODE: {
        String val = node.getNodeValue();
	if (val.trim().length() == 0) {
	  Node sibling = node.getNextSibling();
	  pNode.removeChild(node);
	  traverse (pNode, sibling);
	}
        break;
      }
    } 
  }

  public static void main(String[] args) {
   try {
    XMLPreProcessor p = XMLPreProcessor.getInstance();
    org.apache.xerces.parsers.DOMParser parser =
      new org.apache.xerces.parsers.DOMParser();
    parser.parse(args[0]);
    Document doc = parser.getDocument();
    if (doc != null) {
      p.preProcess(doc);
    }
   } catch (Exception ex) {
     System.out.println(ex);
   }
  }
}
