package lbl.akenti.util;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Writer;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import java.text.ParseException;

import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.xml.sax.helpers.ParserFactory;
import org.xml.sax.Parser;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;

public class XMLUtil {
  private XMLUtil() { 
  }

  public static Document 
  parseDocument(InputStream is) throws IOException, ParseException {
    try {
      DOMParser parser = new DOMParser();
      String xmlValidate = System.getProperties().getProperty("XMLValidation", "off");

      parser.setFeature(
        "http://xml.org/sax/features/validation", 
        xmlValidate.equalsIgnoreCase("on")
      );

      parser.parse(new InputSource(new InputStreamReader(is)));
      return parser.getDocument();
    } catch(SAXException ex) {
      throw new ParseException(ex.getMessage(), -1);
    }
  }

  public static void 
  writeDocument(Document doc, OutputStream os)
    throws IOException {
    OutputFormat format = new OutputFormat(doc);

    format.setLineWidth(0);
    format.setIndent(3);

    Writer writer = new BufferedWriter(new OutputStreamWriter(os));
    XMLSerializer serial = new XMLSerializer(writer, format);

    serial.asDOMSerializer();
    serial.serialize(doc.getDocumentElement());
    writer.flush();
  }

  /**
    * Removes blank space text nodes from the tree specified by <i>node</i>.
    */
  public static void filter(Node node) {
    Node child = node.getFirstChild();

    while (child != null) {
      switch(child.getNodeType()) {
        case Node.TEXT_NODE:  
          Node temp = child;

          child = child.getNextSibling();
          
          if (temp.getNodeValue().trim().length() == 0) {
            node.removeChild(temp);
          }

          break;
        default:
          filter(child);
          child = child.getNextSibling();
          break;
      }
    }
  }
}
